define(['modules/agenda/module'], function (module) {
    "use strict";

    return module.registerController('AgendaCtrl', function ($scope, Upload, $timeout, $http, $stateParams, $rootScope, $state, paginationService) {
        $scope.carregando = false;
        $scope.row = {};
        $scope.campos_disable = false;
        $scope.order = 'nome';
        $scope.order_sentido = 'ASC';
        $scope.Pesquisa = '';
        $scope.sortDescending = false;

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        if ($rootScope.$state.current.url.indexOf('ver') > 0) {
            $scope.campos_disable = true;
        }

        $scope.LoadGrupos = function () {
            $http.get(options.api.restv2 + "/agendas/grupos?max=-1").
                success(function (data) {
                    $scope.grupos = data.dados;
                });
        }

        $scope.CarregarItem = function (id) {
            if (!id) return;

            $scope.loading = true;
            $http.get(options.api.restv2 + "/agenda/" + id).
                success(function (data) {
                    $scope.row = data.dados;
                    $scope.loading = false;
                });
        };

        if ($stateParams.id) {
            $scope.CarregarItem($stateParams.id);
        }

        $scope.filter = function () {
            var results = $scope.rows;
            results.length = 0;
            var searchText = $scope.Pesquisa;
            var reminderTypes = $scope.reminderTypes;

            for (var i = 0; i < reminderTypes.length; ++i) {
                if (searchText.length > 0) {
                    if (reminderTypes[i].nome && reminderTypes[i].id != 9999999 && reminderTypes[i].nome.toLowerCase().includes(searchText.toLowerCase())
                        || (reminderTypes[i].numero && reminderTypes[i].numero.includes(searchText))
                        || (reminderTypes[i].grupo && reminderTypes[i].grupo.toLowerCase().includes(searchText.toLowerCase()))) {
                        results.push(reminderTypes[i]);
                    }
                } else {
                    if (reminderTypes[i].id != 9999999) {
                        results.push(reminderTypes[i]);
                    }
                }
            }
            results.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        $scope.load = function () {
            $scope.carregando = true;
            $http.get(options.api.restv3 + "/agenda?max=-1").
                success(function (data) {
                    $scope.reminderTypes = data.dados;
                    $scope.rows = angular.copy($scope.reminderTypes);
                    $scope.filter();
                    $scope.carregando = false;
                });
        };

        $scope.init = function () {
            $scope.load();
        };

        $scope.setOrder = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        $scope.Editar = function () {
            $scope.campos_disable = false;
        };

        $scope.Cancelar_editar = function () {
            $scope.CarregarItem($stateParams.id);
            $scope.campos_disable = true;
        };

        $scope.Salvar = function () {
            if ($scope.row.id) {
                let clId = $scope.row.id
                delete $scope.row.id
                $scope.row.id_grupo = $scope.row.grupo
                delete $scope.row.grupo
                $http.put(`${options.api.restv2}/agenda/${clId}`, $scope.row).
                    success(function (data) {
                        $.smallBox({
                            title: "SUCESSO!!",
                            content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                            color: "#739E73",
                            timeout: 6000
                        });
                        $rootScope.$state.transitionTo('app.agenda');
                    })
            } else {
                $scope.row.id_grupo = $scope.row.grupo
                delete $scope.row.grupo
                $http.post(`${options.api.restv2}/agenda`, $scope.row).
                    success(function (data) {
                        $.smallBox({
                            title: "SUCESSO!!",
                            content: "<i class='fa fa-check'></i> <i>Criado com sucesso!</i>",
                            color: "#739E73",
                            timeout: 6000
                        });
                        $rootScope.$state.transitionTo('app.agenda');
                    })
            }
        };

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.agenda');
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover contato <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover esse contato? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv3 + "/agenda/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Agenda apagada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });

                    $scope.load();
                })
        };

        $scope.getClass = function (coluna) {
            if ($scope.order === coluna) {
                return $scope.order_sentido === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.uploadFiles = function () {
            if (!$scope.files) {
                $.bigBox({
                    title: "Selecione algum arquivo!",
                    content: "Você precisa selecionar algum arquivo para enviar",
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated",
                    timeout: 6000
                });
                return;
            }

            angular.forEach($scope.files, function (file) {
                if (file && !file.$error) {
                    file.upload = Upload.upload({
                        url: options.api.restv3 + "/agenda/upload/" + $scope.row.grupo.id,
                        data: $scope.row,
                        file: file
                    });

                    file.upload.then(function (response) {
                        $timeout(function () {
                            file.result = response.data;
                        });
                    }, function (response) {
                        if (response.status > 0) {
                            $scope.errorMsg = response.status + ': ' + response.data;
                        }
                        else {
                            console.log('OK: ' + response.data);
                        }
                    });

                    file.upload.progress(function (evt) {
                        file.progress = Math.min(100, parseInt(100.0 *
                            evt.loaded / evt.total));
                    });
                }
            });
        };

        $scope.confirma_limpagrupo = function (row) {
            if (!row) {
                $.bigBox({
                    title: "Selecione um grupo!",
                    content: "Você precisa selecionar um grupo para limpar",
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated",
                    timeout: 6000
                });
                return;
            }
            else {
                $.x5Box({
                    title: "LIMPAR grupo <b>" + row.nome + "?</b>",
                    content: "Tem certeza que quer remover todos os números do grupo <b>" + row.nome + "?</b> <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Limpar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                    color: "#C46A69",
                    //timeout: 8000,
                    icon: "fa fa-warning shake animated"
                }, function (ButtonPressed) {
                    if (ButtonPressed === "Limpar") {
                        $scope.limpa_grupo(row.id);
                    }
                });
            }
        };

        $scope.limpa_grupo = function (id) {
            $http.delete(options.api.restv3 + "/agenda/grupo/limpar/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Grupo limpo com sucesso!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });

                    $scope.load();
                })
        };

    });
});