define(['modules/campanhas/module'], function (module) {
    "use strict";

    return module.registerController('ContatosCtrl', function ($scope, $http, Upload, $timeout, $stateParams, $filter, $rootScope) {
        $scope.carregando = false;
        $scope.filtro = {};
        $scope.order = [{ "campo": "id", "ordem": "ASC" }];
        $scope.importacaoJBS = options.importacaoJBS;

        $scope.Limpa = function (r) {
            return JSON.stringify(r).replace(/[{}"\[\]]/g, '').replace(',', ', ');
        };

        $scope.uploadFiles = function (files, id, priority) {
            if (files) {
                console.log(`${options.api.restv4}/campanha/${id}/contatos_upload?prioridade=${priority}`);
                $.x5loading({
                    title: "Enviando arquivos, aguarde..",
                    content: "<i class='fa fa-4x fa-spinner fa-spin'></i>",
                    color: "#3176b0"
                });

                // Transforma os arquivos em array caso apenas não tenha sido especificado
                if (!Array.isArray(files)) files = [files];

                angular.forEach(files, file => {
                    if (file && !file.$error) {
                        console.log(`Uploading file -> `, file);
                        Upload.upload({
                            url: `${options.api.restv4}/campanha/${id}/contatos_upload?prioridade=${priority}`,
                            file: file
                        }).then(
                            resp => {
                                console.log('Sucesso!!!');
                                console.log(resp);

                                $("#x5loading").remove();

                                $.smallBox({
                                    title: "SUCESSO!!",
                                    content: "<i class='fa fa-check'></i> <i>Contatos importados!</i>",
                                    color: "#739E73",
                                    timeout: 3000
                                });

                                $scope.load();
                            },
                            resp => {
                                console.log('Error status: ' + resp.status);
                            },
                            event => {
                                console.log(event);
                                // var progressPercentage = parseInt(100.0 * event.loaded / event.total);
                                // console.log('progress: ' + progressPercentage + '% ' + event.config.file.file.name);
                            }
                        );
                    } else {
                        console.error(`Error while starting file upload -> `, (file && file.$error ? file.$error : 'Invalid file'));
                    }
                });
            } else {
                console.error(`No file to upload!`);
            }
        };

        $scope.uploadJBSJson = function (jsonInput, campanhaId) {
            let jsonData;
            try {
                jsonData = { dados: JSON.parse(jsonInput) };
            } catch (e) {
                alert('JSON inválido');
                return;
            }

            $http.post(`${options.api.restv4}/campanhas/importLotesJSON?sampleCampainID=${campanhaId}`, jsonData)
                .then(function (response) {
                    $('#jsonInputModal').modal('hide');
                }, function (error) {
                    console.error(error);
                });
        };

        $scope.uploadJBS = function (file, id) {
            if (file && !file.$error) {
                $.x5loading({
                    title: "Enviando arquivo, aguarde..",
                    content: "<i class='fa fa-4x fa-spinner fa-spin'></i>",
                    color: "#3176b0"
                });
                Upload.upload({
                    url: options.api.restv4 + "/campanhas/importLotes?sampleCampainID=" + id,
                    file: file
                }).then(function (resp) {
                    console.log('Sucesso!!!');
                    console.log(resp);
                    $("#x5loading").remove();
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Contatos importados!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });
                    $scope.load();
                }, function (resp) {
                    console.log('Error status: ' + resp.status);
                }, function (evt) {
                    console.log(evt);
                });
            }
        };

        $scope.gotoPage = function (page) {
            $scope.filtro.page = { valor: page };
            $scope.load();
        };

        $scope.loadCampanha = function (id) {
            return new Promise((resolve, reject) => {
                $http.get(`${options.api.restv2}/campanha/${id}`)
                    .success(data => {
                        try {
                            $scope.campanha = data.campanha;
                            resolve($scope.campanha);
                        } catch (e) {
                            console.error(e);
                            reject(e);
                        }
                    })
                    .error(error => {
                        console.error(e);
                        reject(e);
                    });
            });
        };

        $scope.loadContatos = function (campanhaId, order, pars) {
            return new Promise((resolve, reject) => {
                $http.get(`${options.api.restv2}/campanha/${campanhaId}/contatos?max=15&order=${order}${pars}`)
                    .success(data => {
                        try {
                            $scope.table = data;
                            resolve(data);
                        } catch (e) {
                            console.error(e);
                            reject(e);
                        }
                    })
                    .error(error => {
                        console.error(e);
                        reject(e);
                    });
            });
        };

        /**
         * Carrega dados da lista
         * @returns void
         */
        $scope.load = async function () {
            $scope.carregando = true;

            // Converte os filtros em query string
            let par = [];
            if ($scope.filtro) {
                angular.forEach($scope.filtro, (v, k) => {
                    if (v) par.push(k + "=" + v.valor);
                });
            }

            const pars = (par && par.length > 0) ? `&${par.join("&")}` : "";
            console.log(pars);

            // Carrega as informações da campanha e os contatos em paralelo
            Promise.all([
                $scope.loadCampanha($stateParams.id),
                $scope.loadContatos($stateParams.id, encodeURI(JSON.stringify($scope.order)), pars)
            ]).finally(x => {
                $scope.carregando = false;
            });
        };

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.campanhas');
        };

        $scope.$watch('Pesquisa', function (t1, t2) {
            // Se os dois valores estirem em branco nao faz nada, isso aconte no load.
            if (!t1 && !t2)
                return false;

            setTimeout(function () {
                if (t1 === $scope.Pesquisa) {
                    if (t1 !== '') {
                        $scope.filtro.nome = { valor: t1 };
                    }
                    else {
                        $scope.filtro.nome = undefined;
                    }

                    $scope.load();
                }
            }, 1000);
        });

        $scope.confirma = function (row) {
            $.x5Box({
                title: "<b>" + $scope.getWord('Remover contato') + "</b>?",
                content: $scope.getWord('Tem certeza que quer remover esse contato?') + " <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv2 + "/campanha/" + $scope.campanha.id + "/contato/" + id).
                success(function (s) {
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>" + $scope.getWord('Contato apagada com sucesso!') + "</i>",
                        color: "#739E73",
                        timeout: 2000
                    });

                    $scope.load();
                })
        };

        $scope.confirma_limpa = function () {
            $.x5Box({
                title: "<b>" + $scope.getWord('Limpar contatos') + "</b>?",
                content: $scope.getWord('Tem certeza que quer remover TODOS os contatos da campanha') + ' ' + $scope.campanha.nome + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.limpa($scope.campanha.id);
                }
            });
        };

        $scope.limpa = function (id) {
            $http.delete(options.api.restv2 + "/campanha/" + id + "/contatos").
                success(function (s) {
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>" + $scope.getWord('Contatos apagados com sucesso!') + "</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.load();
                })
        };

        $scope.getClass = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                return $scope.order[0].ordem === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.setOrder = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                $scope.order[0].ordem = $scope.order[0].ordem === 'ASC' ? 'DESC' : 'ASC';
            }
            else {
                $scope.order = [{ "campo": coluna, "ordem": "ASC" }];
            }
            //console.log($scope.order);
            $scope.load();
        };
    });
});
