define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'angularUtils.directives.dirPagination'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.classificacoes', [
        'ui.router',
        'ngResource',
        'angularUtils.directives.dirPagination'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.classificacoes', {
                url: '/classificacoes',
                data: {
                    title: 'Classificações',
                    permissions: {
                        only: ['app.classificacoes']
                    }
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/classificacoes/views/lista.html',
                        controller: 'ClassificacoesCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/classificacoes/controllers/ClassificacoesCtrl',
                                'modules/classificacoes/directives/ShowPopupsub'
                            ])
                        }
                    }
                }
            })

            .state('app.classificacoes.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Classificação'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/classificacoes/views/form.html',
                        controller: 'ClassificacoesCtrl',
                        parent: 'app.classificacoes',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/classificacoes/controllers/ClassificacoesCtrl',
                                'modules/classificacoes/directives/classificacoesFormValidate'
                            ])
                        }
                    }
                }
            })

            .state('app.classificacoes.novo', {
                url: '/novo',
                data: {
                    title: 'Nova Classificação'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/classificacoes/views/form.html',
                        controller: 'ClassificacoesCtrl',
                        parent: 'app.classificacoes',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/classificacoes/controllers/ClassificacoesCtrl',
                                'modules/classificacoes/directives/classificacoesFormValidate'
                            ])
                        }
                    }
                }
            })

            .state('app.classificacoes.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Classificação'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/classificacoes/views/form.html',
                        controller: 'ClassificacoesCtrl',
                        parent: 'app.classificacoes',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/classificacoes/controllers/ClassificacoesCtrl',
                                'modules/classificacoes/directives/classificacoesFormValidate'
                            ])
                        }
                    }
                }
            })

            .state('app.subclassificacoes', {
                url: '/subclassificacoes/:id_clas',
                data: {
                    title: 'Subclassificações'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/classificacoes/views/lista.sub.html',
                        controller: 'ClassificacoesCtrl',
                        parent: 'app.classificacoes',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/classificacoes/controllers/ClassificacoesCtrl'
                            ])
                        }
                    }
                }
            });
    });

    module.run(function ($couchPotato, Permission, $localStorage, ModuloPermissao) {
        module.lazy = $couchPotato;

        Permission.defineRole('app.classificacoes', function () {
            if (!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.classificacoes", $localStorage.User.modulos)) {
                $localStorage.msg = {
                    tipo: "Erro",
                    txt: "Você não tem acesso ao módulo de Classificações."
                };
                return false;
            }
            else {
                return true;
            }
        });
    });

    return module;

});