define(['modules/contextos/module'], function (module) {
    "use strict";

    return module.registerController('ContextosCtrl', function ($scope, $http, paginationService) {
        $scope.sortBy = 'nome';
        $scope.sortDescending = false;
        $scope.searchText = '';

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        $scope.order = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.filter = function () {
            var results = $scope.filteredRT;
            results.length = 0;
            var searchText = $scope.searchText;
            var reminderTypes = $scope.reminderTypes;

            for (var i = 0; i < reminderTypes.length; ++i) {
                if (searchText.length > 0) {
                    if (reminderTypes[i].context && reminderTypes[i].context.toLowerCase().includes(searchText.toLowerCase())
                        || (reminderTypes[i].descricao && reminderTypes[i].descricao.toLowerCase().includes(searchText.toLowerCase()))) {
                        results.push(reminderTypes[i]);
                    }
                } else {
                    results.push(reminderTypes[i]);
                }
            }
            results.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $scope.carregando = true;
            $http.get(`${options.api.restv4}/contextos`).
                success(function (data) {
                    if (data) {
                        $scope.reminderTypes = data;
                        $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
                        $scope.filter();
                    }
                    $scope.carregando = false;
                });
        };

        $scope.sentidos = [
            { valor: 'Entrada', nome: $scope.getWord('Entrada') },
            { valor: 'Saida', nome: $scope.getWord('Saída') },
            { valor: 'URA', nome: $scope.getWord('URA') }
        ];

        $scope.editar = async function (row) {
            let t = await $http.get(`${options.api.restv4}/contexto/${row.id}`);
            $scope.nome_disable = true;
            $scope.popup = t.data;

            $scope.showContextoPopup = true;
        };

        $scope.novo = function () {
            $scope.nome_disable = false;
            $scope.popup = {};
            $scope.showContextoPopup = true;
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Apagar grupo de rota <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer apagar esse grupo de rotas? <b>APAGANDO UM GRUPO DE ROTAS TODAS AS ROTAS TAMBÉM SERÃO APAGADAS<b/> <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.Salvar = function (f1) {
            if (f1.$valid) {
                if ($scope.popup.id) {
                    let dados = JSON.parse(JSON.stringify($scope.popup));
                    delete dados.id;
                    $http.put(`${options.api.restv4}/contexto/${$scope.popup.id}`, dados).
                        success(function (data) {
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                color: "#739E73",
                                timeout: 3000
                            });
                            $scope.showContextoPopup = false;
                            $scope.load();
                        })
                }
                else {
                    $http.post(options.api.restv4 + "/contexto", $scope.popup).
                        success(function (data) {
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                color: "#739E73",
                                timeout: 3000
                            });
                            $scope.showContextoPopup = false;
                            $scope.load();
                        })
                }
            }
        };

        $scope.delete = function (id) {
            $http.delete(`${options.api.restv4}/contexto/${id}`).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Grupo de rotas apagado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });

                    $scope.load();
                })
        };
    });
});