define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.contextos', [
        'ui.router',
        'ngResource'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.contextos', {
                url: '/contextos',
                data: {
                    title: 'Contextos',
                    permissions: {
                        only: ['app.contextos']
                    }
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/contextos/views/lista.html',
                        controller: 'ContextosCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/contextos/controllers/ContextosCtrl'
                            ])
                        },
                    }
                }
            })

            .state('app.contextos.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Contexto',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/contextos/views/form.html',
                        controller: 'ContextosCtrl',
                        parent: 'app.contextos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/contextos/controllers/ContextosCtrl',
                                'modules/contextos/directives/contextosFormValidate'
                            ])
                        }
                    }
                }
            })

            .state('app.contextos.novo', {
                url: '/novo',
                data: {
                    title: 'Nova Contexto',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/contextos/views/form.html',
                        controller: 'ContextosCtrl',
                        parent: 'app.contextos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/contextos/controllers/ContextosCtrl',
                                'modules/contextos/directives/contextosFormValidate'
                            ])
                        }
                    }
                }
            })

            .state('app.contextos.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Contexto',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/contextos/views/form.html',
                        controller: 'ContextosCtrl',
                        parent: 'app.contextos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/contextos/controllers/ContextosCtrl',
                                'modules/contextos/directives/contextosFormValidate'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato, Permission, $localStorage, ModuloPermissao) {
        module.lazy = $couchPotato;
        Permission.defineRole('app.contextos', function () {
            if (!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.rotas", $localStorage.User.modulos)) {
                $localStorage.msg = {
                    tipo: "Erro",
                    txt: "Você não tem acesso ao módulo de Contextos."
                };
                return false;
            }
            else {
                return true;
            }
        });
    });

    return module;

});