define(['modules/filas/module'], function (module) {
    "use strict";

    return module.registerController('FilaGruposCtrl', function ($scope, $http, $localStorage, $stateParams, $rootScope, $state) {
        $scope.filtro = {};
        $scope.order = [{ "campo": "nome", "ordem": "ASC" }];

        $scope.Edit = async (e) => {
            getfilas();
            $scope.popup = (await $http.get(options.api.restv4 + `/grupoFilas/${e.id}`)).data
            $scope.popup.filas = $scope.popup.filas.map((fila) => { return { text: fila.fila } })
            $scope.showEdit = true;
        }


        $scope.New = () => {
            getfilas();
            $scope.popup = {};
            $scope.showEdit = true;
        }

        function getfilas () {
            const filas = $localStorage.User.filas;
            filas.sort((a, b) => a.localeCompare(b, undefined, { sensitivity: 'base' }));
            $scope.filas = filas;
        }

        $scope.filterFila = (query) => {
            return $scope.filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        }

        $scope.getClass = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                return $scope.order[0].ordem === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.$watch('Pesquisa', function (newValue, oldValue) {
            if (!newValue && !oldValue)
                return false;

            setTimeout(function () {
                if (newValue === $scope.Pesquisa) {
                    if (newValue === '') {
                        delete $scope.filtro.nome;
                    }
                    else {
                        $scope.filtro.page = 1;
                        $scope.filtro.nome = newValue;
                    }

                    $scope.load();
                }
            }, 1000);
        });

        $scope.setOrder = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                $scope.order[0].ordem = $scope.order[0].ordem === 'ASC' ? 'DESC' : 'ASC';
            }
            else {
                $scope.order = [{ "campo": coluna, "ordem": "ASC" }];
            }
            $scope.load();
        };

        $scope.getGrupos = function (callback, max) {
            getfilas()

            if (!max) max = 15;
            var filtros = angular.copy($scope.filtro);
            if (filtros.ordenar && filtros.ordenar.length > 0) {
                delete filtros.ordenar;
                $scope.order = [];
                angular.forEach($scope.filtro.ordenar, function (o) {
                    $scope.order.push({ "campo": o.valor, "ordem": "ASC" });
                });
            }
            else {
                delete filtros.ordenar;
            }

            var par = [];
            angular.forEach(filtros, function (v, k) {
                if (v) par.push(k + "=" + encodeURI(v));
            });
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }

            $http.get(options.api.restv4 + "/gruposFilas?max=" + max + "&order=" + encodeURI(JSON.stringify($scope.order))
                + "&filas=" + encodeURI(JSON.stringify($scope.filas)) + pars).
                then(function (d) {
                    callback(d.data);
                });
        }

        $scope.gotoPage = function (page) {
            $scope.filtro.page = page;
            $scope.load();
        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $scope.loading = true;
            $scope.getGrupos((data) => {
                console.log(data)
                $scope.table = data;
                $scope.loading = false;
            })
        };

        $scope.getFilasGrupo = (filas) => {
            if (!filas) return
            return filas.map(fila => { return fila.fila; }).join(', ')
        }

        $scope.Salvar = function (f) {
            if (f.$valid) {
                var pop = angular.copy($scope.popup);
                if (!pop.filas || pop.filas.length === 0) {
                    $.smallBox({
                        title: "Erro",
                        content: "<i class='fa fa-warning'></i> <i>É necessário adicionar pelo menos uma fila.</i>",
                        color: "#C46A69",
                        timeout: 4000
                    });
                    return
                }
                pop.filas = pop.filas.map(function (item) { return item.text; })

                if (pop.id) {
                    delete pop.id;
                    $http.put(options.api.restv4 + "/grupoFilas/" + $scope.popup.id, pop).
                        success(function (data) {
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });
                            $scope.showEdit = false;
                            $scope.load();
                        });
                }
                else {
                    $http.post(options.api.restv4 + "/grupoFilas", pop).
                        success(function (data) {
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });
                            $scope.showEdit = false;
                            $scope.load();
                        });
                }
            }
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover " + row.nome + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv4 + "/grupoFilas/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Apagado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.load();
                })
        };
    });
});