define(['modules/filasperformance/module'], function (module) {
    "use strict";

    return module.registerController('FilasPerformanceCtrl', function ($scope, $http, $filter, $localStorage, $window, exportFilesService) {
        const moment = require("moment");

        $scope.groups = [];
        $scope.calculos_dados = [];
        $scope.multserver = false;

        $scope.getGruposFila = async () => {
            if (!$localStorage.User.gruposFila || $localStorage.User.gruposFila.length === 0) return
            $scope.gruposNomes = $localStorage.User.gruposFila;
            await $http.get(options.api.restv4 + "/gruposFilas?max=-1&nomes=" + encodeURI(JSON.stringify($localStorage.User.gruposFila))).
                success(function (data) {
                    $scope.gruposObj = data.dados;
                });
        }

        $scope.filterGrupos = (query) => {
            var grupos = $scope.gruposNomes
            return grupos.filter(function (grupo) {
                return grupo.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.groupAdded = (tag) => {
            const grupoObj = $scope.gruposObj.find(grupo => grupo.nome === tag.text)
            if (!$scope.filtro.filas) $scope.filtro.filas = grupoObj.filas
            else grupoObj.filas.forEach(fila => {
                if ($scope.filtro.filas.indexOf({ text: fila }) === -1) {
                    $scope.filtro.filas.push({ text: fila });
                }
            });
        }

        $scope.groupRemoved = () => {
            const gruposNomes = $scope.filtro.grupos.map(grupo => grupo.text)
            const gruposObj = $scope.gruposObj.filter(grupo => gruposNomes.includes(grupo.nome))
            const filas = gruposObj.flatMap(grupo => grupo.filas)
            $scope.filtro.filas = []
            if (filas && filas.length > 0) {
                filas.forEach(fila => {
                    if ($scope.filtro.filas.indexOf({ text: fila }) === -1) {
                        $scope.filtro.filas.push({ text: fila });
                    }
                })
            }
        }

        $scope.changeGroupOrQueue = () => {
            $scope.filtro.filas = []
            $scope.filtro.grupos = []
        }
        window.changeGroupOrQueue = $scope.changeGroupOrQueue

        function getNivel (row) {
            // Entrante: sentido = 1
            // Status: Atendida, Abandonada, Transbordou.
            // Sainte: sentido = 2, Não se aplica para esses relatórios de performance.
            var espera = TimeTosec(row.espera);
            //var duracao = TimeTosec(row.duracao);
            var fila_conf = getFilaConfig(row.fila);
            if (!fila_conf) return;

            if (row.status === 'Atendida') {
                if (espera <= fila_conf.nivel_tempo) return 'TAP';
                else return 'TAF';
            }
            if (row.status === 'Abandonada') {
                if (espera <= fila_conf.nivel_abandono_tempo) return 'TBP';
                else return 'TBF';
            }
            if (row.status === 'Transbordou') {
                return 'TT';
            }
        }

        function PadTime (num) {
            return ("0" + num).slice(-2);
        }

        function IsAbandonoIntervalo (row) {
            var espera = TimeTosec(row.espera);
            var fila_conf = getFilaConfig(row.fila);
            if (!fila_conf) return false;

            if (row.status === 'Abandonada') {
                if (espera <= fila_conf.nivel_tempo) return true;
                else return false;
            }
            else {
                return false;
            }
        }

        $scope.filterServer = function (query) {
            var servers = options.multserver;
            return servers.filter(function (server) {
                return server.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.totalizadores = [
            { valor: 'fila', nome: $scope.getWord('Fila') },
            { valor: 'hora', nome: $scope.getWord('Hora') },
            { valor: 'quarter_hour', nome: '15min' },
            { valor: 'dia', nome: $scope.getWord('Dia') },
            { valor: 'dia_semana', nome: $scope.getWord('Dia da semana') },
            { valor: 'mes', nome: $scope.getWord('Mês') },
            { valor: 'ano', nome: $scope.getWord('Ano') }
        ];

        $scope.divisores = [
            { valor: 'fila', nome: $scope.getWord('Fila') },
            { valor: 'hora', nome: $scope.getWord('Hora') },
            { valor: 'quarter_hour', nome: '15min' },
            { valor: 'dia', nome: $scope.getWord('Dia') },
            { valor: 'dia_semana', nome: $scope.getWord('Dia da semana') },
            { valor: 'mes', nome: $scope.getWord('Mês') },
            { valor: 'ano', nome: $scope.getWord('Ano') }
        ];

        if (options.multserver) {
            $scope.multserver = true;
            $scope.totalizadores.push({ valor: 'server', nome: $scope.getWord('Servidor') });
            $scope.divisores.push({ valor: 'server', nome: $scope.getWord('Servidor') });
        }

        $scope.AddServer = function () {
            $scope.getFilas();
            $scope.getGruposFila();
        };

        $scope.DelServer = function () {
            $scope.getFilas();
            $scope.getGruposFila();
        };

        $scope.getSelfila = function () {
            if (!$scope.filtro.filas || $scope.filtro.filas.length === 0) return $scope.getWord('Todas');
            else return _.values(_.pluck($scope.filtro.filas, "text")).join(",");
            //_.map($scope.filtro.fila, function(valor, key){ return num * 3; }); _.values($scope.filtro.fila).join(",");
        };

        $scope.getSelfila2 = function () {
            if (!$scope.filtros.fila || $scope.filtros.fila.length === 0) return $scope.getWord('Todas');
            else return _.values(_.pluck($scope.filtros.fila, "text")).join(",");
            //_.map($scope.filtro.fila, function(valor, key){ return num * 3; }); _.values($scope.filtro.fila).join(",");
        };

        $scope.printIt = function () {
            var table = document.getElementById('relatorio').innerHTML;
            var myWindow = $window.open('', '', 'width=800, height=600');
            myWindow.document.write(/*html*/'<html><head><link rel="stylesheet" type="text/css" media="screen" href="styles/css/bootstrap.min.css">'
                + '<link rel="stylesheet" type="text/css" media="print" href="styles/font-awesome-4.3.0/css/font-awesome.min.css">'
                + '<link rel="stylesheet" type="text/css" media="print" href="styles/css/smartadmin-production-plugins.min.css">'
                + '<link rel="stylesheet" type="text/css" media="print" href="styles/css/x5panel.css">'
                + '<link rel="stylesheet" type="text/css" media="print" href="styles/css/fixes.css">'
                + '<link rel="stylesheet" href="styles/css/gfontes.css">'
                + '<link rel="stylesheet" href="styles/css/isteven-multi-select.css">'
                + '<link rel="stylesheet" type="text/css" media="print" href="styles/css/x5panel-skins.css">'
                + '</head><body>' + table + '</body></html>');
            //myWindow.print();
        };

        function TimeTosec (time) {
            if (!time) return 0;
            var tt = time.split(":");
            return parseInt(tt[0]) * 3600 + parseInt(tt[1]) * 60 + parseInt(tt[2]) * 1;
        };

        function SecToTime (seconds) {
            if (!seconds) return '00:00:00';
            var date = new Date(1970, 0, 1);
            date.setSeconds(seconds);
            return date.toTimeString().replace(/.*(\d{2}:\d{2}:\d{2}).*/, "$1");
        };

        function getTempoMedioSegundos (segundos, total) {
            if (segundos == '0' || total == '0') return 0;
            return Math.round(segundos / total);
        };

        var dias_semana = [$scope.getWord('Domingo'), $scope.getWord('Segunda'), $scope.getWord('Terça'), $scope.getWord('Quarta'), $scope.getWord('Quinta'), $scope.getWord('Sexta'), $scope.getWord('Sábado')];
        var messes = [$scope.getWord('Janeiro'), $scope.getWord('Fevereiro'), $scope.getWord('Março'), $scope.getWord('Abril'), $scope.getWord('Maio'), $scope.getWord('Junho'), $scope.getWord('Julho'), $scope.getWord('Agosto'), $scope.getWord('Setembro'), $scope.getWord('Outubro'), $scope.getWord('Novembro'), $scope.getWord('Dezembro')];

        $scope.getNivelServico = function (row) {
            //console.log(row);
            if (!row.formula)
                return 0;

            var mapObj = {
                ta: row.ta,
                tap: row.tap,
                taf: row.taf,
                tb: row.tb,
                tbi: row.tbi,
                tbp: row.tbp,
                tbf: row.tbf,
                tt: row.tt
            };
            //console.log(mapObj);

            var formula = row.formula.toLowerCase().replace(/ta\b|tap\b|taf\b|tb\b|tbp\b|tbf\b|tbi\b|tt\b/gi, function (matched) {
                return mapObj[matched];
            });

            formula = formula.replace(/null\b/g, '0');
            var r = math.eval(formula);
            row.nivel = isFinite(r) ? $filter('number')(r * 100, 0) : 0;
            return row.nivel;
        };

        $scope.getClassNivel = function (nivel) {
            if (nivel > 80)
                return 'progress-bar bg-color-green';
            if (nivel >= 50)
                return 'progress-bar bg-color-yellow';
            return 'progress-bar bg-color-red';
        };

        $scope.calcularAbandonosEfetivos = function (rows) {
            let count = 0

            //Ordena as chamadas por data e agrupa por numero
            let orderedCalls = _.chain(rows).sortBy('calldate').reverse().groupBy('numero').value()

            //Pega a ultima chamada de cada numero, e se for abandonada, adiciona abandono efetivo
            Object.keys(orderedCalls).forEach((call) => {
                if (orderedCalls[call][0].status == "Abandonada") { count++ }
            })

            return count
        }

        // Agrupa por multiplos grupos
        _.groupByMulti = function (obj, values, context) {
            if (!values.length)
                return obj;
            var byFirst = _.groupBy(obj, values[0], context),
                rest = values.slice(1);
            for (var prop in byFirst) {
                byFirst[prop] = _.groupByMulti(byFirst[prop], rest, context);
            }
            return byFirst;
        };

        $scope.loadEventos = function (filas, dataInicial, dataFinal) {
            return new Promise((resolve, reject) => {
                // Monta a query
                let query = `max=-1`;

                // Adiciona o filtro de filas
                query += `&filas=${JSON.stringify(filas)}`;

                // Adiciona o filtro de data inicial
                query += `&data_ini=${dataInicial.format("YYYY-MM-DDTHH:mm:ss")}`;
                // Adiciona o filtro de data final
                query += `&data_fim=${dataFinal.format("YYYY-MM-DDTHH:mm:ss")}`;

                // Adiciona o filtro de eventos de login
                query += `&eventos=["Login"]`;

                // Envia a requisicao
                $http.get(options.api.restv2 + `/agentes/eventos?${query}`)
                    .success(response => resolve(response.dados));
            });
        };

        /*  async function estimarAgentesOnline(eventosDeAgentes, eventos) {
             let dataInicio = null;
             let dataFim = null;
     
             // Verifica se consegue extrair o horário pelo filtro
             if (eventos.some(x => x.hora)) {
                 let evento = eventos.find(x => x.hora);
                 let [inicio, fim] = evento.hora.split("-");
     
                 let data = moment(evento.calldate).format("YYYY-MM-DD");
     
                 dataInicio = moment(`${data} ${inicio}`);
                 dataFim = moment(`${data} ${fim}:59`);
                 console.log(`${dataInicio.format("YYYY-MM-DD HH:mm:ss")} --- ${dataFim.format("YYYY-MM-DD HH:mm:ss")}`);
             } else {
                 // Ordena as datas dos eventos
                 let datas = eventos.filter(x => x.calldate)
                     .map(x => new Date(x.calldate))
                     .sort((a, b) => a.getTime() - b.getTime());
     
                 // Extrai o intervalo das datas
                 dataInicio = moment(datas[0]);
                 dataFim = moment(datas[datas.length - 1]);
             }
     
             // Extrai o nome da fila
             // let filas = [...new Set(eventos.map(x => x.fila).filter(x => x))];
     
             // Busca os registros referente aos agentes desta fila nesse determinado intervalo de tempo
             // let eventosDaFila = await $scope.loadEventos(filas, dataInicio, dataFim);
     
             // Separa os eventos de login
             let logins = eventosDeAgentes.filter(x => x.event === "Login" && x.event_start);
     
             // Separa os eventos por agente
             let eventosDeLoginSeparadosPorAgente = _.groupBy(logins, "agent");
     
             // Determina quais agentes estavam logados no intervalo dado
             let agentesLogadosNoIntervalo = [];
             // Itera em todos os agentes
             for (let agente in eventosDeLoginSeparadosPorAgente) {
                 // Itera em todos os eventos de login do agente
                 for (let login of eventosDeLoginSeparadosPorAgente[agente]) {
                     let start = moment(login.event_start);
                     let end = login.event_end ? moment(login.event_end) : null;
     
                     // console.log(`Start -> ${start.toISOString()}\nEnd -> ${end ? end.toISOString() : null}\nData -> { start: ${dataInicio.toISOString()}, end: ${dataFim.toISOString()} }`);
     
                     if (start.isSameOrBefore(dataInicio) && (!end || end.isSame(dataFim, "day"))) {
                         // Se encontrado intervalo onde o agente estava online, adiciona ele no array
                         agentesLogadosNoIntervalo.push(agente);
     
                         break;// Para a busca
                     }
                 }
             }
     
             if (agentesLogadosNoIntervalo && agentesLogadosNoIntervalo.length > 0) console.log(`Agentes: `, agentesLogadosNoIntervalo);
     
             // DEBUG: Para debuggar a função, descomente a linha abaixo
             console.log(`Entre ${dataInicio.toISOString()} e ${dataFim.toISOString()} | possuia [${agentesLogadosNoIntervalo.join(",")}] agentes (${a.length} no total)`);
     
             return agentesLogadosNoIntervalo;
         } */

        async function estimarAgentesOnline (eventosDeAgentes, eventos, divisor, totalizador) {

            //console.log(`Divisor -> `, divisor);
            //console.log(`Totalizador -> `, totalizador);

            let agentes = [...new Set(eventosDeAgentes.filter(x =>
                x && x[$scope.filtros.dividir.valor] == divisor && x[$scope.filtros.totalizar.valor] == totalizador
            ).map(x => x.agent))];
            //console.log(agentes);

            return agentes;
        }

        async function buscarEventosDeAgentes (filtros) {
            // Busca as filas a serem consideradas
            let filas = filtros.filas && filtros.filas.length > 0 ? filtros.filas.map(x => x.nome || x.text) : $scope.filas;

            // Busca os eventos dos agentes
            let eventos = await $scope.loadEventos(filas, moment(filtros.data_ini).startOf("day"), moment(filtros.data_fim).endOf("day"));

            // Define os divisores e totalizadores do agente
            for (let evento of eventos) {
                let data = new Date(evento.event_start);
                evento.dia = data.getDate();
                evento.mes = data.getMonth();
                evento.ano = data.getFullYear();
                evento.dia_semana = data.getDay();

                let hora = PadTime(data.getHours());
                evento.hora = `${hora}:00-${hora}:59`;

                evento.minutos = data.getMinutes();
                if (evento.minutos > 0 && evento.minutos < 15) evento.quarter_hour = hora + ':00-' + hora + ':15';
                else if (evento.minutos > 15 && evento.minutos < 30) evento.quarter_hour = hora + ':15-' + hora + ':30';
                else if (evento.minutos > 30 && evento.minutos < 45) evento.quarter_hour = hora + ':30-' + hora + ':45';
                else evento.quarter_hour = hora + ':45-' + PadTime(data.getHours() + 1) + ':00';

                evento.fila = evento.queue;
            }

            // Agrupa os eventos de agentes
            return eventos;
        }

        async function ProcessaGrupo (filtros, data) {
            let eventosDeAgentes = await buscarEventosDeAgentes(filtros);
            //console.log(eventosDeAgentes);

            return await Promise.all(
                _.chain(data)
                    .sortBy(filtros.dividir.valor)
                    .groupBy(filtros.dividir.valor)
                    .map(async function (value, key) {
                        var grupo = {};

                        grupo.rows = await Promise.all(
                            _.chain(value)
                                .sortBy(filtros.totalizar.valor)
                                .groupBy(filtros.totalizar.valor)
                                .map(async function (value1, venue_id) {
                                    var ob = {};
                                    ob.grupo = key;
                                    ob.server = value1[0].server;
                                    ob.fila = value1[0].fila;
                                    ob.formula = value1[0].formula;
                                    ob.totalizar = venue_id;
                                    ob.mes = value1[0].mes;
                                    ob.total = _.filter(_.pluck(value1, "sentido"), function (item) { return item === 1; }).length;
                                    ob.tap = _.filter(_.pluck(value1, "nivel"), function (item) { return item === 'TAP'; }).length;
                                    ob.taf = _.filter(_.pluck(value1, "nivel"), function (item) { return item === 'TAF'; }).length;
                                    ob.ta = _.filter(value1, function (item) { return item.status === 'Atendida' && item.sentido === 1 }).length;
                                    ob.pta = $scope.getPercentual(ob.ta, ob.total);
                                    ob.tbi = _.filter(_.pluck(value1, "abandono_intervalo"), function (item) { return item === true; }).length;
                                    ob.tbp = _.filter(_.pluck(value1, "nivel"), function (item) { return item === 'TBP'; }).length;
                                    ob.tbf = _.filter(_.pluck(value1, "nivel"), function (item) { return item === 'TBF'; }).length;
                                    ob.tb = _.filter(_.pluck(value1, "status"), function (item) { return item === 'Abandonada' }).length;
                                    ob.ptb = $scope.getPercentual(ob.tb, ob.total);
                                    ob.efetivos = $scope.calcularAbandonosEfetivos(value1);
                                    ob.tt = _.filter(_.pluck(value1, "status"), function (item) { return item === 'Transbordou' }).length;

                                    ob.tte = SomaArrayDeNumeros(
                                        _.pluck(_.filter(value1, function (item) { return item.status === 'Atendida' && item.sentido === 1; }), "tme_s")
                                    );
                                    ob.tta = SomaArrayDeNumeros(
                                        _.pluck(_.filter(value1, function (item) { return item.status === 'Atendida' && item.sentido === 1; }), "tma_s")
                                    );
                                    ob.ttab = SomaArrayDeNumeros(
                                        _.pluck(_.filter(value1, function (item) { return item.status === 'Abandonada' && item.sentido === 1; }), "tmab_s")
                                    );
                                    ob.ttt = SomaArrayDeNumeros(
                                        _.pluck(_.filter(value1, function (item) { return item.status === 'Transbordou' && item.sentido === 1; }), "tmt_s")
                                    );

                                    // Estima a quantidade de agentes online nesse intervalo
                                    ob.agentes_online_array = await estimarAgentesOnline(eventosDeAgentes, value1, key, venue_id);
                                    ob.agentes_online = ob.agentes_online_array.length;

                                    ob.tme = getTempoMedioSegundos(ob.tte, ob.ta);
                                    ob.tma = getTempoMedioSegundos(ob.tta, ob.ta);
                                    ob.tmab = getTempoMedioSegundos(ob.ttab, ob.tb);
                                    ob.tmt = getTempoMedioSegundos(ob.ttt, ob.tt);

                                    ob.ns = $scope.getNivelServico(ob);

                                    ob.tef = _.filter(value1, function (item) { return item.sentido === 2; }).length;
                                    ob.tefc = _.filter(value1, function (item) { return item.status === 'Atendida' && item.sentido === 2; }).length;
                                    ob.tefnc = _.filter(value1, function (item) { return item.status !== 'Atendida' && item.sentido === 2; }).length;

                                    ob.tteef = SomaArrayDeNumeros(
                                        _.pluck(_.filter(value1, function (item) { return item.sentido === 2; }), "tmeef_s")
                                    );
                                    ob.ttaef = SomaArrayDeNumeros(
                                        _.pluck(_.filter(value1, function (item) { return item.sentido === 2; }), "tmaef_s")
                                    );
                                    ob.tmeef = getTempoMedioSegundos(ob.tteef, ob.tef);
                                    ob.tmaef = getTempoMedioSegundos(ob.ttaef, ob.tef);
                                    return ob;
                                })
                                .value()
                        );

                        grupo.nome = key;
                        grupo.nivel_tempo = value[0].nivel_tempo;
                        grupo.nivel_abandono_tempo = value[0].nivel_abandono_tempo;
                        grupo.formula = value[0].formula;
                        grupo.totais = getTotalGrupo(grupo.rows);
                        return grupo;
                    })
                    .value()
            );
        }

        $scope.getTam = function (nivel) {
            if (nivel <= 32)
                return 100;
            else
                return nivel;
        };

        $scope.getFormatTotalizador = function (valor) {
            switch ($scope.filtros.totalizar.valor) {
                case 'dia_semana':
                    return dias_semana[valor];
                case 'mes':
                    return messes[valor];
                default:
                    return valor;
            }
        };

        $scope.getFormatDivisor = function (valor) {
            switch ($scope.filtros.dividir.valor) {
                case 'dia_semana':
                    return dias_semana[valor];
                case 'mes':
                    return messes[valor];
                default:
                    return valor;
            }
        };

        $scope.filterTotalizadores = function (query) {
            var totalizadores = $scope.totalizadores;
            return totalizadores.filter(function (totalizador) {
                return totalizador.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.filterFilas = function (query) {
            var filas = $localStorage.User.filas;
            return filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getPercentual = function (v1, v2) {
            return $filter('number')((parseInt(v1) / parseInt(v2)) * 100, 0) + '%';
        };

        function somatempo (tempo) {
            return _.reduce(tempo, function (result, current) {
                return SecToTime(TimeTosec(result) + TimeTosec(current));
            }, 0);
        }

        function SomaArrayDeNumeros (vetor) {
            return vetor.reduce(function (pv, cv) { return parseInt(pv) + parseInt(cv); }, 0);
        }

        function getTotalGrupo (registros) {
            var total = _.chain(registros)
                .groupBy('grupo')
                .map(function (value1) {
                    var ob = {};
                    ob.formula = value1[0].formula;
                    ob.total = SomaArrayDeNumeros(_.pluck(value1, "total"));
                    ob.ta = SomaArrayDeNumeros(_.pluck(value1, "ta"));
                    ob.pta = $scope.getPercentual(ob.ta, ob.total);
                    ob.tap = SomaArrayDeNumeros(_.pluck(value1, "tap"));
                    ob.ptap = $scope.getPercentual(ob.tap, ob.ta);
                    ob.taf = SomaArrayDeNumeros(_.pluck(value1, "taf"));
                    ob.ptaf = $scope.getPercentual(ob.taf, ob.ta);
                    ob.tb = SomaArrayDeNumeros(_.pluck(value1, "tb"));
                    ob.ptb = $scope.getPercentual(ob.tb, ob.total);
                    ob.tbf = SomaArrayDeNumeros(_.pluck(value1, "tbf"));
                    ob.ptbf = $scope.getPercentual(ob.tbf, ob.tb);
                    ob.tbp = SomaArrayDeNumeros(_.pluck(value1, "tbp"));
                    ob.ptbp = $scope.getPercentual(ob.tbp, ob.tb);
                    ob.tbi = SomaArrayDeNumeros(_.pluck(value1, "tbi"));
                    ob.ptbi = $scope.getPercentual(ob.tbi, ob.tb);
                    ob.efetivos = SomaArrayDeNumeros(_.pluck(value1, "efetivos"));
                    ob.tt = SomaArrayDeNumeros(_.pluck(value1, "tt"));
                    ob.ptt = $scope.getPercentual(ob.tt, ob.total);
                    ob.tte = SomaArrayDeNumeros(_.pluck(value1, "tte"));
                    ob.tta = SomaArrayDeNumeros(_.pluck(value1, "tta"));
                    ob.ttab = SomaArrayDeNumeros(_.pluck(value1, "ttab"));
                    ob.ttt = SomaArrayDeNumeros(_.pluck(value1, "ttt"));

                    // Calcula o total de agentes
                    let distinctAgents = new Set(_.flatten(_.pluck(value1, "agentes_online_array")));
                    ob.agentes_online = distinctAgents.size;

                    ob.tme = getTempoMedioSegundos(ob.tte, ob.ta);
                    ob.tma = getTempoMedioSegundos(ob.tta, ob.ta);
                    ob.tmab = getTempoMedioSegundos(ob.ttab, ob.tb);
                    ob.tmt = getTempoMedioSegundos(ob.ttt, ob.tt);

                    ob.tef = SomaArrayDeNumeros(_.pluck(value1, "tef"));
                    ob.tefc = SomaArrayDeNumeros(_.pluck(value1, "tefc"));
                    ob.tefnc = SomaArrayDeNumeros(_.pluck(value1, "tefnc"));

                    ob.tteef = SomaArrayDeNumeros(_.pluck(value1, "tteef"));
                    ob.ttaef = SomaArrayDeNumeros(_.pluck(value1, "ttaef"));
                    ob.tmeef = getTempoMedioSegundos(ob.tteef, ob.tef);
                    ob.tmaef = getTempoMedioSegundos(ob.ttaef, ob.tef);
                    return ob;
                })
                .value();
            return total[0];
        }

        function getDadosApi () {
            return new Promise((resolve, reject) => {
                $scope.groups = [];

                var filtros2 = angular.copy($scope.filtro);
                $scope.filtros = angular.copy($scope.filtro);
                filtros2.data_ini = $filter('date')(filtros2.data_ini, 'yyyy-MM-dd');
                filtros2.data_fim = $filter('date')(filtros2.data_fim, 'yyyy-MM-dd');
                filtros2.status = JSON.stringify(["Atendida", "Abandonada", "Transbordou", "Não atendida", "Ocupado", "Falha"]);
                delete filtros2.totalizar;
                delete filtros2.dividir;
                delete filtros2.usaGrupo
                delete filtros2.grupos

                if ($scope.filtro.filas && $scope.filtro.filas.length > 0) {
                    filtros2.filas = [];
                    angular.forEach($scope.filtro.filas, function (f) {
                        filtros2.filas.push(f.text);
                    });
                    filtros2.filas = JSON.stringify(filtros2.filas);
                }
                if (!filtros2.filas || filtros2.filas.length == 0) {
                    filtros2.filas = JSON.stringify($scope.filas);
                }
                if (!filtros2.server || filtros2.server.length == 0) {
                    delete filtros2.server;
                }

                var par = [];
                if (filtros2) {
                    angular.forEach(filtros2, function (v, k) {
                        if (v) par.push(k + "=" + encodeURI(v));
                    });
                }
                var pars = "";
                if (par.length > 0) {
                    pars = "?" + par.join("&");
                }

                $http.get(options.api.restv4 + "/filas/ligacoes" + pars).
                    success(async function (data) {
                        // Aplica trasformações de valores
                        data.dados = _.map(data.dados, function (row) {
                            var fila_conf = getFilaConfig(row.fila);

                            var calldate = new Date(row.calldate);
                            row.mes = calldate.getMonth();
                            row.ano = calldate.getFullYear();
                            row.dia = calldate.getDate();
                            var hora = PadTime(calldate.getHours());
                            row.hora = PadTime(calldate.getHours()) + ':00-' + PadTime(calldate.getHours()) + ':59';
                            row.dia_semana = calldate.getDay();
                            row.minutos = calldate.getMinutes();

                            if (row.minutos > 0 && row.minutos < 15) row.quarter_hour = hora + ':00-' + hora + ':15';
                            else if (row.minutos > 15 && row.minutos < 30) row.quarter_hour = hora + ':15-' + hora + ':30';
                            else if (row.minutos > 30 && row.minutos < 45) row.quarter_hour = hora + ':30-' + hora + ':45';
                            else row.quarter_hour = hora + ':45-' + PadTime(calldate.getHours() + 1) + ':00';

                            //Garante que todos os registros tenham uma formula. Isso não afeta quem realmente não a tem.
                            row.formula = (fila_conf && fila_conf.calculo_ns) ? fila_conf.calculo_ns.formula : undefined;

                            if (row.sentido == 1) {
                                row.nivel = getNivel(row);
                                row.nivel_tempo = fila_conf ? fila_conf.nivel_tempo : undefined;
                                row.nivel_abandono_tempo = fila_conf ? fila_conf.nivel_abandono_tempo : undefined;
                                row.abandono_intervalo = IsAbandonoIntervalo(row);

                                row.tma = row.duracao ? row.duracao : '00:00:00';
                                row.tme = row.status === 'Atendida' ? row.espera : '00:00:00';
                                row.tmab = row.status === 'Abandonada' ? row.espera : '00:00:00';
                                row.tmt = row.status === 'Transbordou' ? row.espera : '00:00:00';

                                row.tma_s = row.tma === '00:00:00' ? 0 : TimeTosec(row.tma);
                                row.tme_s = row.tme === '00:00:00' ? 0 : TimeTosec(row.tme);
                                row.tmab_s = row.tmab === '00:00:00' ? 0 : TimeTosec(row.tmab);
                                row.tmt_s = row.tmt === '00:00:00' ? 0 : TimeTosec(row.tmt);
                            }
                            else {
                                row.tmaef = row.duracao ? row.duracao : '00:00:00';
                                row.tmeef = row.espera ? row.espera : '00:00:00';

                                row.tmaef_s = row.duracao === '00:00:00' ? 0 : TimeTosec(row.tmaef);
                                row.tmeef_s = row.espera === '00:00:00' ? 0 : TimeTosec(row.tmeef);
                            }
                            return row;
                        });
                        //return ProcessaGrupo($scope.filtros, data.dados);
                        const ret = await ProcessaGrupo($scope.filtros, data.dados);
                        resolve(ret);
                        //$scope.loading = false;
                    });
            });
        }

        $scope.getNsFilas = function () {
            $scope.loading = true;
            getDadosApi().then(dados => {
                //console.log(dados);
                $scope.groups = dados;
                $scope.loading = false;
            });
        };

        function getFilasData () {
            $http.get(options.api.restv2 + "/filas?max=-1").
                success(function (data) {
                    $scope.filas_dados = data.dados;
                });
        }

        function getFilaConfig (fila) {
            if (!$scope.filas_dados) return;

            for (var i = 0; i < $scope.filas_dados.length; i++) {
                if ($scope.filas_dados[i].name === fila) {
                    for (var j = 0; j < $scope.calculos_dados.length; j++) {
                        if ($scope.filas_dados[i].id_calculo_nivel === $scope.calculos_dados[j].id) {
                            $scope.filas_dados[i].calculo_ns = $scope.calculos_dados[j];
                        }
                    }
                    return $scope.filas_dados[i];
                }
            }
        }

        function getCalculosData () {
            $http.get(options.api.restv2 + "/calculos_nivelservico?max=-1").
                success(function (data) {
                    $scope.calculos_dados = data.dados;
                });
        }

        $scope.getFilas = function () {
            if (options.multserver) {
                var servers = ($scope.filtro.server && $scope.filtro.server.length > 0) ? $scope.filtro.server : options.multserver;

                $scope.filas = [];
                $localStorage.User.filas = [];
                angular.forEach(servers, function (s) {
                    $http.get("http://" + s.endereco + "/xcontact-rest/filas?max=0&order[0]=name").
                        success(function (data) {
                            angular.forEach(data.itens, function (f) {
                                $scope.filas.push(f.name);
                            });
                            $localStorage.User.filas = _.uniq($scope.filas);
                        });
                });
            }
            else {
                $scope.filas = $localStorage.User.filas;
            }
        };

        $scope.getAgentes = function (fila) {
            var link = "/agentes?max=0";
            if (fila) {
                link = "/agentespelafila/" + fila;
            }

            if (options.multserver) {
                angular.forEach(options.multserver, function (s) {
                    $http.get("http://" + s.endereco + "/xcontact-rest" + link).
                        success(function (data) {
                            $scope.agentes = data.itens;
                        });
                });
            }
            else {
                $http.get(options.api.rest + link).
                    success(function (data) {
                        $scope.agentes = data.itens;
                    });
            }
        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $scope.filas = [];
            $scope.filtro = {};
            //$scope.filtro.dataini = hoje.getDate();
            $scope.filtro.data_ini = new Date();
            $scope.filtro.data_fim = new Date();

            $scope.filtro.server = [];
            $scope.filtro.totalizar = $scope.totalizadores[1];
            $scope.filtro.dividir = $scope.divisores[0];
            $scope.filtros = angular.copy($scope.filtro);

            $scope.getFilas();
            $scope.getGruposFila();
            getCalculosData();
            getFilasData();
        };

        $scope.ToCSV = function () {

            $scope.carregando_export = true;
            var arr = [];
            var cab = [];
            cab = [
                $scope.filtro.dividir.nome,
                $scope.filtro.totalizar.nome,
                'Oferecidas',
                'Atendidas na meta',
                'Atendidas fora da meta',
                'Atendidas',
                '% Atendidas',
                'Abandonadas abaixo do minimo',
                'Abandonadas acima do minimo',
                'Abandonadas abaixo da meta de atendimento',
                'Abandonadas',
                '% Abandonadas',
                'Abandonos Efetivos',
                'Transbordos',
                'Nivel de servico',
                'TM Espera',
                'TM Atendimento',
                'TM Abandono',
                'TM Transbordo',
                'Efetuadas',
                'Efetuadas Completadas',
                'Efetuadas Nao completadas',
                'TM Espera efetuadas',
                'TM Duracao efetuadas'
            ];
            arr.push(cab);
            getDadosApi().then(res => {
                console.log(res);
                for (var j in res) {
                    for (var i in res[j].rows) {
                        var ob = res[j].rows[i];
                        var a1 = [];
                        a1 = [
                            $scope.getFormatDivisor(ob.grupo),
                            $scope.getFormatTotalizador(ob.totalizar),
                            ob.total,
                            ob.tap,
                            ob.taf,
                            ob.ta,
                            ob.pta,
                            ob.tbp,
                            ob.tbf,
                            ob.tbi,
                            ob.tb,
                            ob.ptb,
                            ob.efetivos,
                            ob.tt,
                            ob.ns + '%',
                            $filter('sectotime')(ob.tme),
                            $filter('sectotime')(ob.tma),
                            $filter('sectotime')(ob.tmab),
                            $filter('sectotime')(ob.tmt),
                            ob.tef,
                            ob.tefc,
                            ob.tefnc,
                            $filter('sectotime')(ob.tmeef),
                            $filter('sectotime')(ob.tmaef)
                        ];
                        arr.push(a1);
                    }
                }
                exportFilesService.gerarCSV(arr, "Performance de filas.csv").then(
                    dados => {
                        console.log(`Sucesso na geraCSV, ${dados}`);
                        $scope.carregando_export = false;
                    },
                    erro => {
                        console.log(`Erro na geraCSV ${erro}`);
                        $scope.carregando_export = false;
                    }
                )
            });
        };

        $scope.ToPDF = function () {
            const filtros = angular.copy($scope.filtro);
            getDadosApi().then(res => {
                var linhas = [];
                var cabecalho = [
                    $scope.filtro.dividir.nome,
                    $scope.filtro.totalizar.nome,
                    'Total',
                    'At <= meta',
                    'At > meta',
                    'Atendidas',
                    'Ab <= min',
                    'Ab > min',
                    'Ab <= meta at',
                    'Ab',
                    'Ab Efet',
                    'Transb',
                    'NS',
                    'TME',
                    'TMA',
                    'TMab',
                    'TMT'
                ];
                for (var j in res) {
                    for (var i in res[j].rows) {
                        var ob = res[j].rows[i];
                        var linha = [];
                        linha = [
                            $scope.getFormatDivisor(ob.grupo),
                            $scope.getFormatTotalizador(ob.totalizar),
                            ob.total,
                            ob.tap,
                            ob.taf,
                            ob.ta + ' (' + ob.pta + ')',
                            ob.tbp,
                            ob.tbf,
                            ob.tbi,
                            ob.tb + ' (' + ob.ptb + ')',
                            ob.efetivos,
                            ob.tt,
                            ob.ns + '%',
                            $filter('sectotime')(ob.tme),
                            $filter('sectotime')(ob.tma),
                            $filter('sectotime')(ob.tmab),
                            $filter('sectotime')(ob.tmt)
                        ];
                        linhas.push(linha);
                    }
                    var total = [];
                    total = [
                        'Total',
                        '',
                        res[j].totais.total,
                        res[j].totais.tap,
                        res[j].totais.taf,
                        res[j].totais.ta + ' (' + $scope.getPercentual(res[j].totais.ta, res[j].totais.total) + ')',
                        res[j].totais.tbp,
                        res[j].totais.tbf,
                        res[j].totais.tbi,
                        res[j].totais.tb + ' (' + $scope.getPercentual(res[j].totais.tb, res[j].totais.total) + ')',
                        res[j].totais.efetivos,
                        res[j].totais.tt,
                        $scope.getNivelServico(res[j].totais) + '%',
                        $filter('sectotime')(res[j].totais.tme),
                        $filter('sectotime')(res[j].totais.tma),
                        $filter('sectotime')(res[j].totais.tmab),
                        $filter('sectotime')(res[j].totais.tmt)
                    ];
                    linhas.push(total);
                }

                // Titulo
                const titulo = { texto: $scope.getWord('Performance de filas'), x: 40, y: 60 };

                // Subtitulo
                const subtitulo = [
                    { texto: `${$scope.getWord('De')} ${$filter('data')(filtros.data_ini)} ${$scope.getWord('até')} ${$filter('data')(filtros.data_fim)}`, x: 40, y: 80 }
                ];
                if (filtros.hora_ini && filtros.hora_fim) {
                    subtitulo.push({ texto: `${$scope.getWord('entre')} ${filtros.hora_ini} ${$scope.getWord('e')} ${filtros.hora_fim}`, x: 150, y: 80 });
                } else if (filtros.hora_ini) {
                    subtitulo.push({ texto: `${$scope.getWord('a partir das')} ${filtros.hora_ini}`, x: 150, y: 80 });
                } else if (filtros.hora_fim) {
                    subtitulo.push({ texto: `${$scope.getWord('até as')} ${filtros.hora_fim}`, x: 150, y: 80 });
                }

                subtitulo.push({ texto: $scope.getWord('Fila(s)') + `: ${$scope.getSelfila()}`, x: 40, y: 92 });
                subtitulo.push({ texto: $scope.getWord('Dividido por') + ` ${filtros.dividir.nome} ${$scope.getWord('e')} ${$scope.getWord('totalizado por')} ${filtros.totalizar.nome}`, x: 40, y: 104 });

                exportFilesService.gerarPDF(cabecalho, linhas, titulo, subtitulo, "Performance de filas.pdf").then(ret => {
                    console.log(ret);
                    $scope.carregando_export = false;
                })
            });
        };
    });
});
