const { get } = require("lodash");

define(['modules/jbs_discador_relatorios/module'], function (module) {
    "use strict";

    return module.registerController('VisaoMacroCtrl', function ($scope, $http, $filter, $localStorage, exportFilesService) {
        const moment = require("moment");
        moment.locale(navigator.language || "pt-BR");
        const mdf = require("moment-duration-format");
        let atual = moment().toDate();
        $scope.registros = [];
        $scope.pausas_nr17 = ["ALMOCO", "PAUSA MANHA", "PAUSA TARDE"];
        $scope.queryCampanha = ['DATA DE CRIAÇÃO', 'DATA DE AGENDAMENTO']

        $scope.busca = {
            dataini: new Date(),
            datafim: new Date(),
            horaini: '00:00',
            horafim: '23:59'
        };

        $scope.filtro = {
            filas: [],
            agentes: [],
            query: $localStorage.VisaoMacroQuery ? $localStorage.VisaoMacroQuery : 'DATA DE AGENDAMENTO'
        }

        $scope.classificacoesEfetivas = $localStorage.VisaoMacroClassificacoes ? $localStorage.VisaoMacroClassificacoes : [];

        let classificacoes = [];

        function removeDiscador (filasArr) {
            if (filasArr && filasArr.length > 0) {
                let fila = filasArr.filter(f => !f.match('DISCADOR'))
                return fila
            }
        }

        function filtraLigacoes (lig) {
            if ($scope.classificacoesEfetivas && $scope.classificacoesEfetivas.length > 0) {
                let nomesClassificacao = $scope.classificacoesEfetivas.map(c => c.nome)
                let r = lig.filter(l => nomesClassificacao.includes(l.classificacao))
                return r
            } else { return lig }
        }

        function getClassificacoes () {
            $http.get(`${options.api.restv2}/classificacoes?max=-1`).success(function (data) {
                classificacoes = data.dados
            })
        }

        $scope.filterClassificacoes = function (query) {
            return classificacoes.filter(function (cl) {
                return cl.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            })
        }

        function filterSave () {
            $localStorage.VisaoMacroClassificacoes = $scope.classificacoesEfetivas
            $localStorage.VisaoMacroQuery = $scope.filtro.query
        }

        $scope.onClassificacoesChange = () => {
            filterSave();
            filter();
        }

        $scope.filterFilas = function (query) {
            var filas = $localStorage.User.filas;
            return filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.filterAgentes = (query) => {
            let agentes = getAgentesDasFilas($scope.agentes);
            return agentes.filter(function (ag) {
                return ag.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        }

        function getAgentesDasFilas (agentes) {
            let filas = ($scope.filtro.filas && $scope.filtro.filas.length > 0) ? $scope.filtro.filas.map(f => f.text) : $localStorage.User.filas;

            return agentes.filter(ag => {
                let filtrados = undefined;
                ag.filas_nome = ag.filas.map(f => f.fila);
                for (let i = 0; i < ag.filas_nome.length; i++) {
                    if (filas.includes(ag.filas_nome[i])) filtrados = ag;
                }
                if (filtrados) {
                    return filtrados;
                }
            });
        }

        function getAgentes () {
            $http.get(`${options.api.restv2}/agentes?max=-1`).
                success(function (data) {
                    $scope.agentes = getAgentesDasFilas(data.dados);
                    $scope.agentes = _.sortBy($scope.agentes, 'nome');
                });
        }

        $scope.onFilasChange = () => {
            $scope.filtro.agentes = [];
            filter();
        }

        $scope.onAgentesChange = () => {
            filter();
        }

        $scope.load = () => {
            console.log('Load UP');
            getAgentes();
            getClassificacoes();
            $scope.registros = [];

            $scope.getDados();
        }

        $scope.getPercentual = (feito, total) => {
            if (feito == total) return 100;
            if (!feito || feito == 0) return 0;
            let perc = (feito * 100) / total;
            return perc < 100 ? perc.toFixed(2) : 100;
        }

        function getDataHoraMedia (datas_array) {
            let milsec = datas_array.map(l => { return l ? l.getTime() : 0 }).reduce((a, b) => a + b, 0);
            // Pega tamanho apenas de objetos não nulos
            let tan = datas_array.map(l => { return l ? 1 : 0 }).reduce((a, b) => a + b, 0);
            //console.log(`New Tan=${tan}, Old=${datas_array.length}`);
            return new Date(milsec / tan);
        }

        function diffDateInSeconds (inicio, fim) {
            let a = moment(inicio);
            let b = moment(fim);
            return b.diff(a, 'seconds');
        }

        function padTime (time) {
            time = parseInt(time);
            return (time < 10 ? "0" : "") + time;
        }

        function SecToTime (seconds) {
            if (!seconds) return '00:00:00';
            let duration = moment.duration(seconds, 'seconds');
            return duration.format('h:mm:ss', { trim: false, useGrouping: false })
        };

        function TimeTosec (time) {
            if (!time) return 0;
            if (typeof time == 'object') time = `${time.getHours()}:${time.getMinutes()}:${time.getSeconds()}`
            var tt = time.split(":");
            return parseInt(tt[0]) * 3600 + parseInt(tt[1]) * 60 + parseInt(tt[2]) * 1;
        };

        function MilSecToTime (mili) {
            if (!mili) return '00:00:00';
            var date = moment().toDate();
            date.setMilliseconds(mili);
            return padTime(date.getHours()) + ":" + padTime(date.getMinutes()) + ":" + padTime(date.getSeconds());
        };

        /**
         * Retorna o tempo total que o agente ficou no evento em segundos
         * @param {Array} eventos 
         * @returns Tempo em segundos
         */
        function getTimeInEvent (eventos) {
            if (eventos.length == 0) return;

            let timeline = JSON.parse(JSON.stringify(eventos[0]));
            timeline.event_start = moment(timeline.event_start).toDate();
            timeline.event_end = timeline.event_end ? moment(timeline.event_end).toDate() : atual;
            timeline.tempo = diffDateInSeconds(timeline.event_start, timeline.event_end);

            for (let i = 1; i < eventos.length; i++) {
                if (eventos[i].event_start > timeline.event_end) {
                    //console.log(`Caiu no if pro agente ${timeline.agent} com ${MilSecToTime(timeline.tempo)}`);
                    timeline.tempo += diffDateInSeconds(eventos[i].event_start, eventos[i].event_end);
                    timeline.event_end = eventos[i].event_end;
                }
                else if (eventos[i].event_end > timeline.event_end) {
                    //console.log(`Caiu no else pro agente ${timeline.agent}, evento ${eventos[i].event_start} até ${eventos[i].event_end}, timeline ${timeline.event_start} até ${timeline.event_end}`);
                    timeline.tempo += diffDateInSeconds(timeline.event_end, eventos[i].event_end);
                    timeline.event_end = eventos[i].event_end;
                }
            }
            return timeline.tempo;
        }

        function filter () {
            $scope.registros = [];
            $scope.totais = {
                qtd_pausa: 0,
                total_logado: 0,
                total_pausa: 0,
                jornada_trabalho: 0,
                total_agentes: 0,
                total_agentes_discador: 0,
                total_agentes_fora: 0,
                total_agentes_outras_pausas: 0,
                total_logado_discador: 0,
                total_pausa_discador: 0,
                total_ligacoes: 0,
                total_tempo_ligacoes: 0,
                total_tempo_ligacoes_fora: 0,
                jornada_trabalho_discador: 0,
                presenca_discador: 0,
                meta_mailing: 0,
                real_mailing: 0,
                total_ligacoes_efetivas: 0,
                aderencia_discador: 0,
                tempo_medio_falado: 0,
                total_ligacoes_fora: 0,
                total_ligacoes_efetivas_fora: 0,
                presenca_fora: 0,
                total_logado_fora: 0,
                total_pausa_fora: 0,
                tempo_medio_falado_fora: 0,
                jornada_fora: 0,
                total_chats: 0,
                total_agentes_chats: 0,
                total_chats_entrantes: 0,
                total_chats_entrantes_encerrados: 0,
                total_chats_entrantes_abertos: 0,
                total_chats_saintes: 0,
                total_chats_saintes_encerrados: 0,
                total_chats_saintes_abertos: 0,
                perc_chats_entrantes_encerrados: 0,
                perc_chats_saintes_encerrados: 0,
                total_pausas_nr17: [],
                total_outras_pausas: 0,
                total_primeiro_login: 0,
                total_ultimo_logoff: 0,
                media_outras_pausas () {
                    return SecToTime(this.total_outras_pausas / this.total_agentes_outras_pausas);
                },
                media_primeiro_login () {
                    return SecToTime(this.total_primeiro_login / this.total_agentes);
                },
                media_ultimo_logoff () {
                    return SecToTime(this.total_ultimo_logoff / this.total_agentes);
                },
                total_primeira_ligacao: 0,
                total_ultima_ligacao: 0,
                media_primeira_ligacao () {
                    return SecToTime(this.total_primeira_ligacao / this.total_agentes_discador);
                },
                media_ultima_ligacao () {
                    return SecToTime(this.total_ultima_ligacao / this.total_agentes_discador);
                },
                total_primeira_ligacao_fora: 0,
                total_ultima_ligacao_fora: 0,
                media_primeira_ligacao_fora () {
                    return SecToTime(this.total_primeira_ligacao_fora / this.total_agentes_fora);
                },
                media_ultima_ligacao_fora () {
                    return SecToTime(this.total_ultima_ligacao_fora / this.total_agentes_fora);
                },
            };

            let filas_permitidas = ($scope.filtro.filas && $scope.filtro.filas.length > 0) ? $scope.filtro.filas.map(f => f.text) : $localStorage.User.filas;
            let agentes = ($scope.filtro.agentes && $scope.filtro.agentes.length > 0) ? $scope.filtro.agentes.map(f => f.nome) : undefined;

            let eventos_filter = $scope.eventos.filter(ev => filas_permitidas.includes(ev.queue));
            if (agentes) {
                eventos_filter = eventos_filter.filter(ev => agentes.includes(ev.agent));
            }

            if (eventos_filter.length > 0) {
                let ordenado = _.chain(eventos_filter)
                    .sortBy('event_start')
                    .groupBy('agent')
                    .map(function (value1) {
                        //console.log('Valor linha')
                        //console.log(v1)
                        let ob = {};
                        ob.login = Object.assign([], value1.filter(item => item.event == 'Login'));
                        if (!ob.login || ob.login.length <= 0) return;
                        ob.agente = value1[0].agent;
                        ob.agenteDados = $scope.agentes.filter(a => a.nome == ob.agente);
                        ob.login_geral = Object.assign([], value1.filter(item => item.event == 'Login'));
                        ob.pausas = Object.assign([], value1.filter(item => item.event == 'Pause' && item.event_data1 != 'Classificando'));
                        ob.pausas_geral = Object.assign([], value1.filter(item => item.event == 'Pause' && item.event_data1 != 'Classificando'));

                        $scope.totais.total_agentes++;

                        ob.pausas_nr17 = Object.assign([], ob.pausas.filter(item => $scope.pausas_nr17.includes(item.event_data1)));
                        ob.outras_pausas = Object.assign([], ob.pausas.filter(item => !$scope.pausas_nr17.includes(item.event_data1)));

                        ob.pausas_nr17_group = _.chain(ob.pausas_nr17).groupBy('event_data1').map(p => {
                            return {
                                nome: p[0].event_data1,
                                tempo: getTimeInEvent(p)
                            };
                        }).value();

                        let pausas_nr17 = 0;
                        ob.pausas_desgroup = [];
                        for (let i = 0; i < ob.pausas_nr17_group.length; i++) {
                            if (!$scope.totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome]) {
                                $scope.totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome] = {
                                    tempo: 0,
                                    total: 0,
                                    media () {
                                        return SecToTime(this.tempo / this.total);
                                    }
                                };
                            }
                            $scope.totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome].total++;
                            $scope.totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome].tempo += ob.pausas_nr17_group[i].tempo;
                            ob.pausas_desgroup[ob.pausas_nr17_group[i].nome] = SecToTime(ob.pausas_nr17_group[i].tempo);
                            pausas_nr17 += ob.pausas_nr17_group[i].tempo;
                        }

                        ob.pausas_nr17_ob = Object.assign({}, ob.pausas_desgroup);



                        let outras_pausas = (ob.outras_pausas && ob.outras_pausas.length > 0) ? getTimeInEvent(ob.outras_pausas) : 0;
                        ob.total_outras_pausas = SecToTime(outras_pausas);
                        if (outras_pausas > 0) $scope.totais.total_agentes_outras_pausas++;
                        $scope.totais.total_outras_pausas += outras_pausas;

                        if (ob.login && ob.login.length > 0) {
                            ob.primeiro_login_sf = ob.login.reduce((a, b) => (a.event_start < b.event_start ? a : b)).event_start;
                            ob.primeiro_login = moment(ob.primeiro_login_sf).format('HH:mm:ss')
                            $scope.totais.total_primeiro_login += TimeTosec(ob.primeiro_login_sf);

                            ob.ultimo_logoff_sf = ob.login.reduce((a, b) => ((a.event_end ? a.event_end : atual) > (b.event_end ? b.event_end : atual) ? a : b)).event_end;
                            ob.ultimo_logoff = ob.ultimo_logoff_sf && ob.ultimo_logoff_sf != atual ? moment(ob.ultimo_logoff_sf).format('HH:mm:ss') : '';
                            $scope.totais.total_ultimo_logoff += TimeTosec(ob.ultimo_logoff);
                        }

                        let logado = getTimeInEvent(ob.login_geral);
                        ob.total_logado = SecToTime(logado);
                        $scope.totais.total_logado += logado;

                        let pausado = outras_pausas + pausas_nr17;
                        ob.total_pausa = SecToTime(pausado);
                        ob.qtd_pausa = ob.pausas.length;
                        $scope.totais.total_pausa += pausado;
                        $scope.totais.qtd_pausa += ob.qtd_pausa;

                        ob.jornada_trabalho = pausado ? SecToTime(logado - pausado) : SecToTime(logado);
                        $scope.totais.jornada_trabalho += (logado - pausado);

                        ob.filas = []
                        if (ob.agenteDados && ob.agenteDados.length > 0) {
                            ob.filas = ob.agenteDados[0].filas_nome
                        }

                        // Discador
                        ob.ligacoes_efetivas = 0;
                        ob.real_mailing = 0;

                        if ($scope.campanhas) {
                            ob.campanhas = $scope.campanhas.filter(c => ob.filas.includes(c.fila));
                            if (ob.campanhas) {
                                ob.campanhas_nome = [...new Set(_.pluck(ob.campanhas, "nome"))];
                                ob.filas_discador = [...new Set(_.pluck(ob.campanhas, "fila"))];
                                ob.meta_mailing = ob.campanhas.map(l => { return parseInt(l.total_contatos) }).reduce((a, b) => a + b, 0);
                                $scope.totais.meta_mailing += ob.meta_mailing;

                                if ($scope.campanhareg && $scope.campanhareg.length > 0) {
                                    ob.campanha_registros = $scope.campanhareg.filter(c => ob.campanhas_nome.includes(c.campanha));
                                    ob.ligacoes_completadas = ob.campanha_registros.filter(item => item.status == 'Completou');
                                    ob.ligacoes_efetivas = filtraLigacoes(ob.ligacoes_completadas);
                                    ob.total_ligacoes_efetivas = ob.ligacoes_efetivas.length;
                                    $scope.totais.total_ligacoes_efetivas += ob.total_ligacoes_efetivas;

                                    ob.real_mailing = ob.campanhas.map(l => { return parseInt(l.total_sucessos) }).reduce((a, b) => a + b, 0);
                                    $scope.totais.real_mailing += ob.real_mailing;

                                    ob.aderencia_discador = ob.meta_mailing ? ((ob.real_mailing * 100) / ob.meta_mailing).toFixed(2) : 0;
                                    $scope.totais.aderencia_discador += parseFloat(ob.aderencia_discador);

                                    ob.prineira_ligacao = ob.campanha_registros.reduce((a, b) => (a.data_status && a.data_status < b.data_status ? a : b), 0).data_status;
                                    ob.prineira_ligacao = $filter('date')(ob.prineira_ligacao, 'HH:mm:ss');
                                    $scope.totais.total_primeira_ligacao += TimeTosec(ob.prineira_ligacao);

                                    ob.ultima_ligacao = ob.campanha_registros.reduce((a, b) => (a.data_status && a.data_status > b.data_status ? a : b), 0).data_status;
                                    ob.ultima_ligacao = $filter('date')(ob.ultima_ligacao, 'HH:mm:ss');
                                    $scope.totais.total_ultima_ligacao += TimeTosec(ob.ultima_ligacao);


                                    ob.tempo_ligacoes = ob.ligacoes_efetivas.map(l => { return parseInt(l.duracao_sec) }).reduce((a, b) => a + b, 0);
                                    ob.tempo_medio_falado = ob.ligacoes_efetivas.length > 0 ? SecToTime(ob.tempo_ligacoes / ob.ligacoes_efetivas.length) : 0;
                                    $scope.totais.total_ligacoes += ob.ligacoes_completadas.length;
                                    $scope.totais.total_tempo_ligacoes += ob.tempo_ligacoes;

                                    ob.logins_discador = ob.login.filter(item => ob.filas_discador.includes(item.queue));
                                    ob.pausas_discador = ob.pausas.filter(item => ob.filas_discador.includes(item.queue));

                                    let logado_discador = ob.logins_discador.length > 0 ? getTimeInEvent(ob.logins_discador) : 0;
                                    ob.total_logado_discador = SecToTime(logado_discador);
                                    $scope.totais.total_logado_discador += logado_discador;

                                    if (logado_discador) $scope.totais.total_agentes_discador++;

                                    let pausado_discador = ob.pausas_discador.length > 0 ? getTimeInEvent(ob.pausas_discador) : 0;
                                    ob.total_pausa_discador = SecToTime(pausado_discador);
                                    $scope.totais.total_pausa_discador += pausado_discador;

                                    ob.jornada_trabalho_discador = logado_discador > 0 ? SecToTime(logado_discador - pausado_discador) : 0;
                                    $scope.totais.jornada_trabalho_discador += (logado_discador - pausado_discador);

                                    ob.presenca_discador = logado_discador ? $scope.getPercentual((logado_discador - pausado_discador), (logado - pausado)) : 0;
                                    //$scope.totais.presenca_discador += parseFloat(ob.presenca_discador);
                                }
                            }
                        }

                        // Nao discador
                        ob.total_ligacoes_fora = 0;
                        ob.total_ligacoes_efetivas_fora = 0;
                        ob.presenca_fora = 0;
                        ob.total_logado_fora = 0;
                        ob.total_pausa_fora = 0;
                        ob.tempo_medio_falado_fora = 0;
                        ob.jornada_fora = 0;

                        ob.filas_fora = removeDiscador(ob.filas)

                        if (ob.filas_fora && ob.filas_fora.length > 0) {
                            ob.ligacoes_fora_discador = $scope.filaligacoes.filter(e => e.agente == ob.agente && !ob.filas_fora.includes(e.fila));
                            ob.total_ligacoes_fora = ob.ligacoes_fora_discador.length;
                            ob.ligacoes_efetivas_fora = filtraLigacoes(ob.ligacoes_fora_discador);
                            ob.total_ligacoes_efetivas_fora = ob.ligacoes_efetivas_fora.length;
                            $scope.totais.total_ligacoes_efetivas_fora += ob.total_ligacoes_efetivas_fora;
                            $scope.totais.total_ligacoes_fora += ob.total_ligacoes_fora;

                            if (ob.ligacoes_fora_discador && ob.ligacoes_fora_discador.length > 0) {
                                $scope.totais.total_agentes_fora++;

                                ob.primeira_ligacao_fora = ob.ligacoes_fora_discador.reduce((a, b) => (a.calldate && a.calldate < b.calldate ? a : b)).calldate;
                                ob.primeira_ligacao_fora = $filter('date')(ob.primeira_ligacao_fora, 'HH:mm:ss');
                                $scope.totais.total_primeira_ligacao_fora += TimeTosec(ob.primeira_ligacao_fora);

                                ob.ultima_ligacao_fora = ob.ligacoes_fora_discador.reduce((a, b) => (a.calldate && a.calldate > b.calldate ? a : b)).calldate;
                                ob.ultima_ligacao_fora = $filter('date')(ob.ultima_ligacao_fora, 'HH:mm:ss');
                                $scope.totais.total_ultima_ligacao_fora += TimeTosec(ob.ultima_ligacao_fora);

                                ob.logins_fora = ob.login.filter(item => !ob.filas_fora.includes(item.queue));
                                ob.pausas_fora = ob.pausas.filter(item => !ob.filas_fora.includes(item.queue));

                                let logado_fora = ob.logins_fora.length > 0 ? getTimeInEvent(ob.logins_fora) : 0;
                                ob.total_logado_fora = SecToTime(logado_fora);
                                $scope.totais.total_logado_fora += logado_fora;

                                let pausado_fora = ob.pausas_fora.length > 0 ? getTimeInEvent(ob.pausas_fora) : 0;
                                ob.total_pausa_fora = SecToTime(pausado_fora);
                                $scope.totais.total_pausa_fora += pausado_fora;

                                ob.jornada_fora = logado_fora > 0 ? SecToTime(logado_fora - pausado_fora) : 0;
                                $scope.totais.jornada_fora += logado_fora > 0 ? (logado_fora - pausado_fora) : 0;

                                ob.tempo_ligacoes_fora = ob.ligacoes_efetivas_fora.map(l => { return parseInt(l.duracao_sec) }).reduce((a, b) => a + b, 0);
                                $scope.totais.total_tempo_ligacoes_fora += ob.tempo_ligacoes_fora;
                                ob.tempo_medio_falado_fora = ob.ligacoes_efetivas_fora.length > 0 ? SecToTime(ob.tempo_ligacoes_fora / ob.ligacoes_efetivas_fora.length) : 0;
                                ob.presenca_fora = logado_fora ? ((logado_fora - pausado_fora) * 100) / (logado - pausado) : 0;
                                $scope.totais.presenca_fora += parseFloat(ob.presenca_fora);
                                ob.presenca_fora = ob.presenca_fora.toFixed(2);
                            }
                        }

                        // WhatsApp
                        ob.total_chats = 0;
                        ob.total_chats_entrantes = 0;
                        ob.total_chats_entrantes_encerrados = 0;
                        ob.total_chats_entrantes_abertos = 0;

                        ob.total_chats_saintes = 0;
                        ob.total_chats_saintes_encerrados = 0;
                        ob.total_chats_saintes_abertos = 0;

                        ob.perc_chats_entrantes_encerrados = 0;
                        ob.perc_chats_saintes_encerrados = 0;

                        ob.chats = $scope.chats.filter(e => e.ultimoAgente == ob.agente);
                        if (ob.chats && ob.chats.length > 0) {
                            ob.total_chats = ob.chats.length;
                            if (ob.total_chats > 0) $scope.totais.total_agentes_chats++;

                            $scope.totais.total_chats += ob.total_chats;

                            ob.chats_entrantes = ob.chats.filter(item => item.sentido == 'Entrante');
                            ob.chats_saintes = ob.chats.filter(item => item.sentido == 'Sainte');

                            ob.chats_entrantes_encerrados = (ob.chats_entrantes && ob.chats_entrantes.length > 0) ? ob.chats_entrantes.filter(item => item.status == 'Encerrado') : [];
                            ob.chats_saintes_encerrados = (ob.chats_saintes && ob.chats_saintes.length > 0) ? ob.chats_saintes.filter(item => item.status == 'Encerrado') : [];

                            ob.total_chats_entrantes = ob.chats_entrantes.length;
                            $scope.totais.total_chats_entrantes += ob.total_chats_entrantes;

                            ob.total_chats_entrantes_encerrados = ob.chats_entrantes_encerrados.length;
                            $scope.totais.total_chats_entrantes_encerrados += ob.total_chats_entrantes_encerrados;

                            ob.total_chats_entrantes_abertos = ob.total_chats_entrantes > 0 ? ob.total_chats_entrantes - ob.total_chats_entrantes_encerrados : 0;
                            $scope.totais.total_chats_entrantes_abertos += ob.total_chats_entrantes_abertos;

                            ob.perc_chats_entrantes_encerrados = ob.total_chats_entrantes_encerrados > 0 ? (ob.total_chats_entrantes_encerrados * 100) / ob.total_chats_entrantes : 0;
                            $scope.totais.perc_chats_entrantes_encerrados += ob.perc_chats_entrantes_encerrados;
                            ob.perc_chats_entrantes_encerrados = ob.perc_chats_entrantes_encerrados.toFixed(2);

                            ob.total_chats_saintes = ob.chats_saintes.length;
                            $scope.totais.total_chats_saintes += ob.total_chats_saintes;

                            ob.total_chats_saintes_encerrados = ob.chats_saintes_encerrados.length;
                            $scope.totais.total_chats_saintes_encerrados += ob.total_chats_saintes_encerrados;

                            ob.total_chats_saintes_abertos = ob.total_chats_saintes > 0 ? ob.total_chats_saintes - ob.total_chats_saintes_encerrados : 0;
                            $scope.totais.total_chats_saintes_abertos += ob.total_chats_saintes_abertos;

                            ob.perc_chats_saintes_encerrados = ob.total_chats_saintes_encerrados > 0 ? (ob.total_chats_saintes_encerrados * 100) / ob.total_chats_saintes : 0;
                            $scope.totais.perc_chats_saintes_encerrados += ob.perc_chats_saintes_encerrados;
                            ob.perc_chats_saintes_encerrados = ob.perc_chats_saintes_encerrados.toFixed(2);
                        }

                        return ob;
                    })
                    .value();

                // Calcula totais
                $scope.totais.total_logado_f = SecToTime($scope.totais.total_logado / $scope.totais.total_agentes);
                $scope.totais.total_pausa_f = SecToTime($scope.totais.total_pausa / $scope.totais.total_agentes);
                $scope.totais.jornada_trabalho = SecToTime($scope.totais.jornada_trabalho / $scope.totais.total_agentes);
                //$scope.totais.prineira_ligacao = getDataHoraMedia(ordenado.map(p => p.prineira_ligacao));
                //$scope.totais.ultima_ligacao = getDataHoraMedia(ordenado.map(p => p.ultima_ligacao));
                $scope.totais.total_logado_discador_f = SecToTime($scope.totais.total_logado_discador / $scope.totais.total_agentes_discador);
                $scope.totais.total_pausa_discador_f = SecToTime($scope.totais.total_pausa_discador / $scope.totais.total_agentes_discador);
                $scope.totais.jornada_trabalho_discador_f = SecToTime($scope.totais.jornada_trabalho_discador / $scope.totais.total_agentes_discador);
                $scope.totais.presenca_discador_f = $scope.getPercentual(($scope.totais.total_logado_discador - $scope.totais.total_pausa_discador), ($scope.totais.total_logado - $scope.totais.total_pausa));
                $scope.totais.tempo_medio_falado = SecToTime($scope.totais.total_tempo_ligacoes / $scope.totais.total_ligacoes_efetivas);
                //$scope.totais.primeira_ligacao_fora = getDataHoraMedia(ordenado.map(p => p.primeira_ligacao_fora));
                //$scope.totais.ultima_ligacao_fora = getDataHoraMedia(ordenado.map(p => p.ultima_ligacao_fora));
                $scope.totais.total_logado_fora_f = SecToTime($scope.totais.total_logado_fora / $scope.totais.total_agentes_fora);
                $scope.totais.total_pausa_fora_f = SecToTime($scope.totais.total_pausa_fora / $scope.totais.total_agentes_fora);
                $scope.totais.jornada_fora = SecToTime($scope.totais.jornada_fora / $scope.totais.total_agentes_fora);
                $scope.totais.presenca_fora = ($scope.totais.presenca_fora / $scope.totais.total_agentes_fora).toFixed(2);
                $scope.totais.tempo_medio_falado_fora = SecToTime($scope.totais.total_tempo_ligacoes_fora / $scope.totais.total_ligacoes_efetivas_fora);
                $scope.totais.perc_chats_entrantes_encerrados = ($scope.totais.perc_chats_entrantes_encerrados / $scope.totais.total_agentes_chats).toFixed(2);

                $scope.totais.perc_chats_saintes_encerrados = ($scope.totais.perc_chats_saintes_encerrados / $scope.totais.total_agentes_chats).toFixed(2);
                $scope.registros = ordenado;
            }
        }

        $scope.getDados = async function () {
            $scope.loading = true;

            let data_ini = $scope.busca.dataini ? $scope.busca.dataini : new Date();
            $scope.busca.dataini = data_ini;

            let hora_ini = $scope.busca.horaini ? $scope.busca.horaini : '00:00';
            $scope.busca.horaini = hora_ini;

            let data_fim = $scope.busca.datafim ? $scope.busca.datafim : new Date();
            $scope.busca.datafim = data_fim;

            let hora_fim = $scope.busca.horafim ? $scope.busca.horafim : '00:00';
            $scope.busca.horafim = hora_fim;

            let query = `max=-1`;
            query += "&data_ini=" + $filter('date')(data_ini, 'yyyy-MM-dd') + ' ' + hora_ini + ':00';
            query += "&data_fim=" + $filter('date')(data_fim, 'yyyy-MM-dd') + ' ' + hora_fim + ':59';

            let geteventos = await $http.get(options.api.restv2 + `/agentes/eventos?${encodeURI(query)}`);
            $scope.eventos = geteventos.data.dados.map(item => {
                item.event_start = moment(item.event_start).toDate();
                item.event_end = item.event_end ? moment(item.event_end).toDate() : atual;
                return item;
            });

            let getcampanhareg = await $http.get(options.api.restv2 + `/campanhas/registros?${encodeURI(query)}`);
            $scope.campanhareg = getcampanhareg.data.dados.map(item => {
                item.data_status = moment(item.data_status).toDate();
                item.duracao_sec = TimeTosec(item.duracao);
                return item;
            });

            if ($scope.filtro.query == 'DATA DE AGENDAMENTO') {
                let qc = `max=-1`;
                qc += "&data_agendamento_ini=" + $filter('date')(data_ini, 'yyyy-MM-dd') + ' ' + hora_ini + ':00';
                qc += "&data_agendamento_fim=" + $filter('date')(data_fim, 'yyyy-MM-dd') + ' ' + hora_fim + ':59';
                let getcampanhas = await $http.get(options.api.restv2 + `/campanhas?${encodeURI(qc)}`);
                $scope.campanhas = getcampanhas.data.dados;
            } else {
                let qc = `max=-1`;
                qc += "&data_create_ini=" + $filter('date')(data_ini, 'yyyy-MM-dd') + ' ' + hora_ini + ':00';
                qc += "&data_create_fim=" + $filter('date')(data_fim, 'yyyy-MM-dd') + ' ' + hora_fim + ':59';
                let getcampanhas = await $http.get(options.api.restv2 + `/campanhas?${encodeURI(qc)}`);
                $scope.campanhas = getcampanhas.data.dados;
            }

            let ql = `${query}&status=["Atendida"]`
            let getfilaligacoes = await $http.get(options.api.restv2 + `/filas/ligacoes?${encodeURI(ql)}`);
            $scope.filaligacoes = getfilaligacoes.data.dados.map(item => {
                item.calldate = moment(item.calldate).toDate();
                item.duracao_sec = TimeTosec(item.duracao);
                return item;
            });

            let qchat = `max=-1`;
            qchat += "&dataIni=" + $filter('date')(data_ini, 'yyyy-MM-dd') + ' ' + hora_ini + ':00';
            qchat += "&dataFim=" + $filter('date')(data_fim, 'yyyy-MM-dd') + ' ' + hora_fim + ':59';
            qchat += `&status=["Entrou","Pesquisa","Encerrado"]`
            let getChats = await $http.get(options.api.restv2 + `/chats?${encodeURI(qchat)}`);
            $scope.chats = getChats.data.dados.map(item => {
                item.dataIni = moment(item.dataIni).toDate();
                item.dataFim = item.dataFim ? moment(item.dataFim).toDate() : atual;
                item.duracao = MilSecToTime(item.dataFim - item.dataIni);
                item.duracao_sec = TimeTosec(item.duracao);
                return item;
            });

            $scope.loading = false;

            filter();
        }

        $scope.ToCSV = async function () {
            $scope.carregando_export = true;
            let arr = [];

            // Formata cabeçalho
            let cab = [
                $scope.getWord('Agente'),
                $scope.getWord('Primeiro Login'),
                $scope.getWord('Último Logoff'),
                $scope.getWord('Tempo total logado Líq'),
                $scope.getWord('Qtd pausas'),
                $scope.getWord('Tempo total pausa'),
                $scope.getWord('Jornada de trabalho Líq'),
                $scope.getWord('Meta mailing'),
                $scope.getWord('Real mailing'),
                $scope.getWord('Total de ligações válidas - Discador'),
                $scope.getWord('% Aderência discador'),
                $scope.getWord('Primeira ligação - Discador'),
                $scope.getWord('Última ligação - Discador'),
                $scope.getWord('Tempo total logado - Discador'),
                $scope.getWord('Tempo total em pausa - Discador'),
                $scope.getWord('Jornada de trabalho - Discador'),
                $scope.getWord('% Líq disponível - Discador'),
                $scope.getWord('Tempo médio falando - Discador'),
                $scope.getWord('Total de ligações - Fora de rota'),
                $scope.getWord('Total de ligações válidas - Fora de rota'),
                $scope.getWord('Primeira ligação - Fora de rota'),
                $scope.getWord('Última ligação - Fora de rota'),
                $scope.getWord('Tempo total logado Líq - Fora de rota'),
                $scope.getWord('Tempo total pausa - Fora de rota'),
                $scope.getWord('Jornada de trabalho - Fora de rota'),
                $scope.getWord('% Líq disponível - Fora de rota'),
                $scope.getWord('Tempo médio falando - Fora de rota'),
                $scope.getWord('Total - WhatsApp'),
                $scope.getWord('Entrante - WhatsApp'),
                $scope.getWord('Entrante finalizados - WhatsApp'),
                $scope.getWord('Entrante abertos - WhatsApp'),
                $scope.getWord('% Entrantes finalizados - WhatsApp'),
                $scope.getWord('Saintes - WhatsApp'),
                $scope.getWord('Saintes finalizados - WhatsApp'),
                $scope.getWord('Saintes abertos - WhatsApp'),
                $scope.getWord('% Saintes finalizados - WhatsApp'),
            ];
            for (let i = 0; i < $scope.pausas_nr17.length; i++) {
                cab.push($scope.pausas_nr17[i]);
            }

            cab.push($scope.getWord('Outras pausa'));

            arr.push(cab);

            // Formata dados
            $scope.registros.forEach((reg) => {
                if (reg) {
                    var linha = [];
                    linha.push(reg.agente);
                    linha.push(reg.primeiro_login);
                    linha.push(reg.ultimo_logoff);
                    linha.push(reg.total_logado);
                    linha.push(reg.qtd_pausa);
                    linha.push(reg.total_pausa);
                    linha.push(reg.jornada_trabalho);
                    linha.push(reg.meta_mailing);
                    linha.push(reg.real_mailing);
                    linha.push(reg.total_ligacoes_efetivas);
                    reg.aderencia_discador ? linha.push(reg.aderencia_discador + '%') : linha.push('%');
                    linha.push(reg.prineira_ligacao);
                    linha.push(reg.ultima_ligacao);
                    linha.push(reg.total_logado_discador);
                    linha.push(reg.total_pausa_discador);
                    linha.push(reg.jornada_trabalho_discador);
                    reg.presenca_discador ? linha.push(reg.presenca_discador + '%') : linha.push('%');
                    linha.push(reg.tempo_medio_falado);
                    linha.push(reg.total_ligacoes_fora);
                    linha.push(reg.total_ligacoes_efetivas_fora);
                    linha.push(reg.primeira_ligacao_fora);
                    linha.push(reg.ultima_ligacao_fora);
                    linha.push(reg.total_logado_fora);
                    linha.push(reg.total_pausa_fora);
                    linha.push(reg.jornada_fora);
                    reg.presenca_fora ? linha.push(reg.presenca_fora + '%') : linha.push('%');
                    linha.push(reg.tempo_medio_falado_fora);
                    linha.push(reg.total_chats);
                    linha.push(reg.total_chats_entrantes);
                    linha.push(reg.total_chats_entrantes_encerrados);
                    linha.push(reg.total_chats_entrantes_abertos);
                    reg.perc_chats_entrantes_encerrados ? linha.push(reg.perc_chats_entrantes_encerrados + '%') : linha.push('%');
                    linha.push(reg.total_chats_saintes);
                    linha.push(reg.total_chats_saintes_encerrados);
                    linha.push(reg.total_chats_saintes_abertos);
                    reg.perc_chats_saintes_encerrados ? linha.push(reg.perc_chats_saintes_encerrados + '%') : linha.push('%');
                    $scope.pausas_nr17.forEach((nr) => {
                        linha.push(reg.pausas_nr17_ob[nr]);
                    });
                    linha.push(reg.total_outras_pausas);

                    arr.push(linha);
                }
            });

            exportFilesService.gerarCSV(arr, "Tempos e movimentos.csv").then(
                dados => {
                    console.log(`Sucesso na geraCSV, ${dados}`);
                    $scope.carregando_export = false;
                },
                erro => {
                    console.log(`Erro na geraCSV ${erro}`);
                    $scope.carregando_export = false;
                }
            )
        };
    });
});