define(['modules/login/module'], function (module) {
    "use strict";

    return module.registerController('LoginCtrl', function ($scope, $rootScope, $localStorage, $state, $http, Language, $Store, $Xcmobx) {

        function getVersions () {
            console.log('Get Versions');
            $http.get("https://www.xcontactcenter.com.br/release_notes/versions.json")
                .success(function (data) {
                    $scope.versoes = data.map((v, index) => {
                        if (v.versao == $rootScope.versao) {
                            console.log(`Achei minha versão ${v.versao} index = ${index}`);
                            v.selected = true;
                            $scope.desatualizado = index > 0 ? $scope.getWord('Sua versão está desatualizada, contate o suporte para atualização.') : '';
                        }
                        return v;
                    })
                });
        }

        $scope.init = function () {
            $scope.currentLanguage = $localStorage.currentLanguage;
            $scope.getIdiomas();
        };

        $scope.load = function () {
            console.log('Load');
            $scope.currentLanguage = $localStorage.currentLanguage;
            $scope.getIdiomas();
            $scope.getDadosLicenca();
            getVersions();
        };

        // Graficos
        $scope.opcoes = [
            { barColor: '#ff021a', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 10, lineCap: 'circle', size: 130 },
            { barColor: '#71b91e', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 20, lineCap: 'circle', size: 130 },
            { barColor: '#c07b17', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 20, lineCap: 'circle', size: 130 },
            { barColor: '#3b52f8', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 20, lineCap: 'circle', size: 130 },
        ];

        $scope.getIdiomas = function () {
            $http.get("api/languages.json")
                .success(function (data) {
                    $scope.idiomas = data;
                });
        };

        $scope.MudaIdioma = function (lang) {
            //$localStorage.currentLanguage = lang;
            Language.getLanguages(function (data) {
                $rootScope.currentLanguage = lang;
                $scope.currentLanguage = lang;
                $localStorage.currentLanguage = lang;

                $rootScope.languages = data;

                Language.getLang(lang.key, function (data) {
                    $localStorage.lang = data;
                    $rootScope.lang = data;
                });

            });
        };

        $scope.graficos = [];

        $scope.getDadosLicenca = function () {
            $http.get(options.api.restv3 + "/licenca")
                .success(function (data) {
                    $scope.Licenca = data.dados;
                    $scope.getTotalRamais();
                });



            var h = new Date().getHours();
            if (h >= 6 && h < 12) {
                $scope.saudacao = $scope.getWord("Bom dia");
            } else if (h >= 12 && h < 18) {
                $scope.saudacao = $scope.getWord("Boa tarde");
            } else {
                $scope.saudacao = $scope.getWord("Boa noite");
            }

            $scope.User = $localStorage.User;
        };

        $scope.getTotalRamais = function () {
            $http.get(options.api.restv3 + "/monitor/totalramais")
                .success(function (data) {
                    $scope.Licenca.ramais_usados = data.total;
                    var op = { barColor: '#71b91e', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 10, lineCap: 'circle', size: 130 };
                    var ob = { nome: 'Ramais', total: $scope.Licenca.total_ramais, usados: $scope.Licenca.ramais_usados, percentual: Math.round(($scope.Licenca.ramais_usados / $scope.Licenca.total_ramais) * 100), opcoes: op };
                    $scope.graficos.push(ob);

                    $scope.getTotalAgentes();
                });
        };

        $scope.getTotalAgentes = function () {
            $http.get(options.api.restv3 + "/monitor/totalagentes")
                .success(function (data) {
                    $scope.Licenca.agentes_usados = data.total;
                    var op = { barColor: '#71b91e', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 10, lineCap: 'circle', size: 130 };
                    var ob = { nome: 'Agentes', total: $scope.Licenca.total_agentes, usados: $scope.Licenca.agentes_usados, percentual: Math.round(($scope.Licenca.agentes_usados / $scope.Licenca.total_agentes) * 100), opcoes: op };
                    $scope.graficos.push(ob);

                    $scope.getTotalSupervisores();
                });
        };

        $scope.getTotalSupervisores = function () {
            $http.get(options.api.restv3 + "/monitor/totalsupervisores")
                .success(function (data) {
                    $scope.Licenca.supervisores_usados = data.total;
                    var op = { barColor: '#71b91e', scaleColor: false, trackColor: '#d2d2d2', lineWidth: 10, lineCap: 'circle', size: 130 };
                    var ob = { nome: 'Supervisores', total: $scope.Licenca.total_supervisores, usados: $scope.Licenca.supervisores_usados, percentual: Math.round(($scope.Licenca.supervisores_usados / $scope.Licenca.total_supervisores) * 100), opcoes: op };
                    $scope.graficos.push(ob);
                });
        };

        $scope.getPercentual = function (v1, v2) {
            return Math.round((v1 / v2) * 100);
        };

        if ($state.current.url === '/logoff') {
            $scope.erromsg = null;
            $localStorage.msg = null;

            if ($localStorage.User) {
                $http.put(`${options.api.restv4}/logoff/supervisor/${$localStorage.User.id}`)
                    .success(function (data) {
                        console.log('logOff efetuado');
                        //$Store.disconect();
                        $Xcmobx.disconect();
                        delete $localStorage.User;
                        delete $scope.User;
                    });
            }
            //$state.go('login');
            //location.reload();
        }

        if ($localStorage.msg && $localStorage.msg.tipo === "Erro") {
            $scope.erromsg = $localStorage.msg.txt;
        }

        var buildCh = function (items) {
            var ul = "<ul>";
            angular.forEach(items, function (i) {
                var li = '<li data-ui-sref-active="active">';
                li += '<a data-ui-sref="' + i.nome + '"><i class="' + i.icon + '"></i> ' + $scope.getWord(i.descricao) + '</a>';
                li += '</li>';
                ul += li;
            });
            ul += "</ul>";
            return ul;
        };

        var buildMenu = function (items) {

            var ul = '<aside id="left-panel"><nav><ul data-smart-menu>';
            angular.forEach(items, function (i) {
                var li = "<li data-menu-collapse>";
                if (i.nome) {
                    li += '<a data-ui-sref="' + i.nome + '"><i class="' + i.icon + '"></i> <span class="menu-item-parent"> ' + $scope.getWord(i.descricao) + '</span></a>';
                }
                else {
                    li += '<a href="#"><i class="' + i.icon + '"></i> <span class="menu-item-parent"> ' + $scope.getWord(i.descricao) + '</span></a>';
                }
                li += buildCh(i.modulos);
                li += "</li>";
                ul += li;
            });
            ul += "</ul>";
            ul += "</nav>";
            ul += '<span class="minifyme" data-action="minifyMenu" minify-menu>';
            ul += '<i class="fa fa-arrow-circle-left hit"></i>';
            ul += '</span>';
            ul += '</aside>';

            return ul;
        };


        $scope.dologin = async function () {
            if ($scope.login_form.$valid) {

                if ($localStorage.User && $localStorage.User.session_id) $scope.row.session_id = $localStorage.User.session_id;

                try {
                    let d = await $http.post(`${options.api.restv4}/login/supervisor`, $scope.row);
                    var User = {};
                    User.ldap = d.data.ldap_user ? true : false
                    User.id = d.data.id;
                    User.login = d.data.nome;
                    User.nome = d.data.fullname;
                    User.senha = encripita($scope.row.senha)
                    User.token = d.data.token;
                    User.modulos = d.data.modulos;
                    User.filas = d.data.filas;
                    User.gruposFila = d.data.gruposFila;
                    User.session_id = d.data.session_id;
                    User.menu = buildMenu(d.data.modulos);

                    $localStorage.User = User;
                    if (!$localStorage.Preferencias) $localStorage.Preferencias = {};
                    $scope.User = User;
                    //$Store.connect();
                    $Xcmobx.connect($scope.row);
                    $state.go('app.home');
                } catch (error) {
                    $scope.duplo_fator_required = error.data.duplo_fator;
                    console.log(error);
                }
            };
        };
    });
});