define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ng-file-upload'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.luc', [
        'ui.router',
        'ngResource',
        'ngFileUpload'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.luc', {
                url: '/luc',                
                data: {
                    title: 'Luc',
                    permissions: {
                        only: ['app.luc']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/lista.html',
                        controller: 'LucCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/LucCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.luc.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Luc'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/form.html',
                        controller: 'LucCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/LucCtrl',
                                'modules/luc/directives/lucFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.luc.novo', {
                url: '/novo',
                data: {
                    title: 'Nova Luc',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/form.html',
                        controller: 'LucCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/LucCtrl',
                                'modules/luc/directives/lucFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.luc.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Luc',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/form.html',
                        controller: 'LucCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/LucCtrl',
                                'modules/luc/directives/lucFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.luc.upload', {
                url: '/upload',
                data: {
                    title: 'Importar contatos',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/upload.html',
                        controller: 'LucCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/LucCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.luc.grupos', {
                url: '/luc/grupos',
                data: {
                    title: 'Grupos'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/lista.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.luc.grupos_editar', {
                url: '/grupos/editar/:id',
                data: {
                    title: 'Editar Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/form.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.luc.grupos_novo', {
                url: '/grupos/novo',
                data: {
                    title: 'Novo Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/luc/views/form.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.luc',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/luc/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$rootScope) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.luc', function () {            
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.luc",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: $rootScope.getWord("Você não tem acesso a Luc.")
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});