define(['modules/mensagem_classificacoes/module'], function (module) {
    "use strict";

    return module.registerController('mensagem_classificacoesCtrl', function ($scope, $localStorage, $filter, ApiService, clientSidePaginationService, exportFilesService) {
        $scope.carregando = false;
        $scope.searchText = '';
        $scope.searchFilas = [];
        $scope.searchAgentes = [];
        $scope.agentes = [];
        $scope.loadchat = [];
        // Injeta paginação
        $scope.pag = clientSidePaginationService;
        $scope.pag.dados = [];
        $scope.pag.sortBy = 'data_registro';

        $scope.cabecalho = [
            $scope.getWord('Data'),
            $scope.getWord('Atendimento'),
            $scope.getWord('Agente'),
            $scope.getWord('Fila'),
            $scope.getWord('Classificação'),
            $scope.getWord('Subclassificação'),
            $scope.getWord('Observação')
        ];

        function getAgentes () {
            ApiService.get(`${options.api.restv2}/agentes?max=-1`).
                success(function (data) {
                    $scope.agentes = data.dados;
                });
        };

        $scope.init = function () {
            $scope.filtro = {
                data_ini: new Date(),
                data_fim: new Date()
            };
            getAgentes();
            $scope.load();
        };

        $scope.filterAgentes = function (query) {
            let agentes = $scope.agentes;
            if ($scope.searchFilas.length > 0) {
                const tags = $scope.searchFilas.map(tag => {
                    return tag.text;
                })
                agentes = $scope.agentes.filter(function (obj) {
                    return obj.filas.find(f => {
                        return tags.find(tag => f.fila == tag);
                    });
                });
            }

            return agentes.filter(function (ob) {
                return ob.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.filterFilas = function (query) {
            var filas = $localStorage.User.filas;
            return filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.filter = async function () {
            let filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));

            if ($scope.searchAgentes.length > 0) {
                const tags = $scope.searchAgentes.map(tag => {
                    return tag.nome.toLowerCase();
                })

                filteredRT = filteredRT.filter(function (obj) {
                    return tags.find(tag => {
                        return obj.agente.toString().toLowerCase().includes(tag);
                    })
                });
            }

            if ($scope.searchFilas.length > 0) {
                const tags = $scope.searchFilas.map(tag => {
                    return tag.text.toLowerCase();
                })

                filteredRT = filteredRT.filter(function (obj) {
                    return tags.find(tag => {
                        return obj.clienteAtendimento[0].queue.toString().toLowerCase().includes(tag);
                    })
                });
            }

            if ($scope.searchText.length > 0) {
                filteredRT = filteredRT.filter(function (obj) {
                    return Object.values(obj).find(v => {
                        if (v) {
                            return v.toString().toLowerCase().includes($scope.searchText.toLowerCase());
                        }
                    })
                });
            }

            // Pagina os dados filtrados
            $scope.pag.dados = filteredRT;
            // Ordena
            $scope.pag.order();

            // Atualiza grafico
            $scope.getGraf();
        };

        function getDados () {
            const filtros = angular.copy($scope.filtro);
            filtros.data_ini = $filter('date')(filtros.data_ini, 'yyyy-MM-dd');
            filtros.data_fim = $filter('date')(filtros.data_fim, 'yyyy-MM-dd');
            filtros.filas = JSON.stringify($localStorage.User.filas);
            return ApiService.get(`${options.api.restv4}/chats/tags`, filtros);
        }

        $scope.load = function () {
            $scope.carregando = true;
            getDados().
                success(function (data) {
                    $scope.carregando = false;
                    $scope.reminderTypes = data.dados.map(d => {
                        d.tag_nome = d.tags.length > 0 ? d.tags[0].nome : $scope.getWord('Não classificada');
                        return d;
                    })
                    $scope.filteredRT = angular.copy($scope.reminderTypes);
                    $scope.filter();
                });
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.grafico_log = {
            chart: {
                plotBackgroundColor: null,
                plotBorderWidth: null,
                plotShadow: false,
                type: 'pie'
            },
            credits: {
                enabled: true,
                text: 'X5 Soluções - Xcontact',
                href: 'http://www.x5s.com.br'
            },
            title: {
                text: $scope.getWord('Tags de Mensagens'),
                useHTML: true
            },
            tooltip: {
                pointFormat: '{series.status}: <b>{point.percentage:.1f}%</b>'
            },
            plotOptions: {
                pie: {
                    allowPointSelect: true,
                    cursor: 'pointer',
                    dataLabels: {
                        enabled: true,
                        format: '<b>{point.name}</b>: {point.percentage:.1f} %',
                        style: {
                            color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black'
                        }
                    },
                    showInLegend: true
                }
            }
        };

        $scope.grafico_barra = {
            chart: {
                plotBackgroundColor: null,
                plotBorderWidth: null,
                plotShadow: false,
                type: 'column'
            },
            xAxis: {
                type: 'category',
                labels: {
                    style: {
                        fontSize: '9px',
                        fontFamily: 'Verdana, sans-serif'
                    }
                }
            },
            yAxis: {
                min: 0,
                allowDecimals: false,
                title: {
                    text: $scope.getWord('Qtd'),
                    style: {
                        fontSize: '9px',
                        fontFamily: 'Verdana, sans-serif'
                    }
                }
            },
            legend: {
                enabled: false
            },
            credits: {
                enabled: true,
                text: 'X5 Soluções - Xcontact',
                href: 'http://www.x5s.com.br'
            },
            title: {
                text: $scope.getWord('Tags de Mensagens'),
                useHTML: true
            }
        };

        $scope.getGraf = function () {
            console.log('Get Graf');
            $scope.loading_graf = true;

            var serie = [];
            var serie_barra = [];

            var result = _.groupBy($scope.pag.dados, 'tag_nome');


            angular.forEach(result, function (v, k) {
                var vet = [k, parseFloat(((parseInt(v.length) / $scope.pag.dados.length) * 100).toFixed(2))];
                let vet2 = [k, parseInt(v.length)];
                serie_barra.push(vet2);
                serie.push(vet);
            });

            $scope.grafico_log.series = [
                {
                    name: $scope.getWord('Total'),
                    data: serie
                }
            ];

            var sortedArray = serie_barra.sort(function (a, b) {
                return b[1] - a[1];
            });

            $scope.grafico_barra.series = [{
                name: $scope.getWord('Tags'),
                colorByPoint: true,
                data: sortedArray
            }];

            $scope.loading_graf = false;
        };

        $scope.ToCSV = async function () {
            $scope.carregando_export = true;
            let arr = [];
            arr.push($scope.cabecalho);

            // Formata dados
            $scope.pag.dados.forEach((e) => {
                var linha = [];
                linha.push($filter('date')(e.data_registro, 'dd/MM/yyyy HH:mm:ss'));
                linha.push(e.id_atendimento);
                linha.push(e.agente);
                linha.push(e.clienteAtendimento[0].queue);
                linha.push(e.tags.length > 0 ? e.tags[0].nome : '');
                linha.push(!e.obs ? '' : e.obs.replaceAll('\n', ' '));
                arr.push(linha);
            });

            exportFilesService.gerarCSV(arr, "Classificação de mensagens.csv").then(
                dados => {
                    console.log(`Sucesso na geraCSV, ${dados}`);
                },
                erro => {
                    $scope.carregando_export = false;
                    console.log(`Erro na geraCSV ${erro}`)
                }
            )
            $scope.carregando_export = false;
        };

        $scope.ToPDF = async function () {
            $scope.carregando_export = true;

            const filtros = angular.copy($scope.filtro);
            if ($scope.searchText) filtros.pesquisa = $scope.searchText;
            if ($scope.searchCampoValor) filtros.searchCampoValor = $scope.searchCampoValor;
            if ($scope.searchCampo) filtros.searchCampo = $scope.searchCampo;


            var linhas = [];
            //Dados
            $scope.pag.dados.forEach((e) => {
                var linha = [];
                linha.push($filter('date')(e.data_registro, 'dd/MM/yyyy HH:mm:ss'));
                linha.push(e.id_atendimento);
                linha.push(e.agente);
                linha.push(e.clienteAtendimento[0].queue);
                linha.push(e.tags.length > 0 ? e.tags[0].nome : '');
                linha.push(e.obs);
                linhas.push(linha);
            });

            //Totalizações
            linhas.push(['Total', $scope.pag.dados.length]);

            // Titulo
            const titulo = { texto: $scope.getWord('Classificação de mensagens'), x: 40, y: 60 };

            // Subtitulo
            const subtitulo = [
                { texto: `${$scope.getWord('De')} ${$filter('data')(filtros.data_ini)} ${$scope.getWord('até')} ${$filter('data')(filtros.data_fim)}`, x: 40, y: 80 }
            ];

            if (filtros.hora_ini && filtros.hora_fim) {
                subtitulo.push({ texto: `${$scope.getWord('entre')} ${filtros.hora_ini} ${$scope.getWord('e')} ${filtros.hora_fim}`, x: 150, y: 80 });
            } else if (filtros.hora_ini) {
                subtitulo.push({ texto: `${$scope.getWord('a partir das')} ${filtros.hora_ini}`, x: 150, y: 80 });
            } else if (filtros.hora_fim) {
                subtitulo.push({ texto: `${$scope.getWord('até as')} ${filtros.hora_fim}`, x: 150, y: 80 });
            }

            if (filtros.pesquisa) {
                subtitulo.push({ texto: $scope.getWord('Pesquisa') + ` ${$scope.getWord('contendo')} ${filtros.pesquisa}`, x: 40, y: 100 });
            }
            const ret = await exportFilesService.gerarPDF($scope.cabecalho, linhas, titulo, subtitulo, "Classificação de mensagens.pdf");
            $scope.carregando_export = false;
        };

        /**
         * This function sanitizes an external raw html
         *
         * @param {string} html The external html to be sanitized
         * @author Leandro Quevedo
         */
        function sanitize (html) {
            try {
                // Parses the provided html
                let doc = new DOMParser().parseFromString(html, "text/html");

                // Removes all the script tags on the document
                [...doc.getElementsByName("script")].forEach(x => x.parentElement.removeChild(x));

                // Transforms style tags into inline css
                [...doc.getElementsByTagName("style")].forEach(tag => {
                    // Verify if the css on the style can be sanitized
                    let sheet = tag.sheet;
                    if (sheet && sheet.rules && sheet.rules.length > 0) {
                        // Iterates trough every rule on the style
                        for (let rule of [...sheet.rules]) {
                            try {
                                // Extract information for the rule
                                let cssText = rule.style.cssText;
                                let selector = rule.style.parentRule.selectorText;

                                // Apply it inline
                                [...doc.queryCommandValue(selector)].forEach(element => {
                                    // Concatenate the old and the new style
                                    let currentStyle = element.getAttribute("style") || "";
                                    currentStyle += cssText;

                                    // Set the inline style on the element
                                    element.setAttribute("style", currentStyle);
                                });
                            } catch (error) { /* ignore */ }
                        }
                    }

                    // Removes the element from the document
                    tag.parentElement.removeChild(tag);
                });

                // Returns the body html
                let output = doc.body.innerHTML;
                console.log("Was -> " + html.length + " output -> " + output.length);
                return output;
            } catch (error) {
                console.error("Unable to sanitize html -> ", error);
                console.trace(error);
            }

            return html;
        }

        $scope.openchatview = async (id) => {
            // Busca Chat
            $scope.loadchat[id] = true;
            let chat = await ApiService.get(`${options.api.restv2}/chat/${id}`);
            $scope.webchat = chat.data;

            // Busca Tags
            let cla = await ApiService.get(`${options.api.restv2}/chat/${id}/classificacoes?max=-1`);
            $scope.webchat.classificacoes = cla.data.error ? [] : cla.data.dados;

            // Busca Msgs
            $scope.webchat.msgs = [];
            var order = [{ campo: 'q.id' }];

            ApiService.get(options.api.restv2 + "/chat/" + id + "/mensagens?max=-1&order=" + encodeURI(JSON.stringify(order))).
                success(function (data) {
                    let dados = (data.dados || []).filter((r) => {
                        return r.texto != "" || r.mensagemAnexos.length > 0;
                    });

                    $scope.webchat.msgs = dados.map(function (d) {

                        let nome = d.inOut === 1 ? $scope.webchat.fromName : d.agente;
                        let texto = sanitize(d.texto);
                        d.texto = '<span class="msg_sender">' + nome + '</span>';

                        var videos_suportados = ["video/mp4", "video/webm", "video/ogg"];
                        var audios_suportados = ["audio/mpeg", "audio/ogg", "audio/wav", "audio/ogg; codecs=opus"];

                        if (d.repliedMsg && d.repliedMsg.length > 0) {
                            d.texto += `<p class="repliedMsg">${sanitize(d.repliedMsg[0].texto)}</p>`;
                        }

                        if (d.mensagemAnexos && d.mensagemAnexos.length > 0) {
                            for (let i = 0; i < d.mensagemAnexos.length; i++) {
                                let Anexo = d.mensagemAnexos[i];
                                if (audios_suportados.includes(Anexo.mimeType)) {
                                    d.texto += '<p><audio preload="none" controls>' +
                                        '<source src="' + Anexo.caminho + '" type="' + Anexo.mimeType + '">' +
                                        '</audio></p>';
                                }
                                else if (videos_suportados.includes(Anexo.mimeType)) {
                                    d.texto += '<p><video height="300" preload="none" controls>' +
                                        '<source src="' + Anexo.caminho + '" type="' + Anexo.mimeType + '">' +
                                        '</audio></p>';
                                }
                                else if (Anexo.mimeType.toString().includes('image')) {
                                    d.texto += '<p><img src="' + Anexo.caminho + '" height="300" download></p>';
                                }
                                else {
                                    d.texto += '<p><a href="' + Anexo.caminho + '" class="btn btn-primary" download>' +
                                        '<i class="fa fa-download"></i> ' + Anexo.caminho.substring(Anexo.caminho.lastIndexOf('/') + 1) +
                                        '</a></p>';
                                }
                            }
                        }
                        d.texto += texto;
                        d.texto += '<span class="time_date">' + $filter('datahora')(d.dataHora) + '</span>';
                        d.texto = d.texto.replace(/\$IP/gi, "");
                        d.texto = d.texto.replace(/null/gi, "");

                        return d;
                    });
                    //$scope.webchat.msgs = data.dados;
                });
            // Icone
            switch ($scope.webchat.queueTypeID) {
                case 1:
                    $scope.webchat.icone = 'fa-phone';
                    break;
                case 2:
                    $scope.webchat.icone = 'fa-comments';
                    break;
                case 3:
                    $scope.webchat.icone = 'fa-whatsapp';
                    break;
                case 4:
                    $scope.webchat.icone = 'fa-envelope-o';
                    break;
                default:
                    $scope.webchat.icone = 'fa-comments';
                    break;
            }
            $scope.loadchat[id] = false;
            $scope.showChat = true;
        }
    });
});