define(['modules/painelsupervisor/module'], function (module){
    "use strict";

    module.registerController("HardwareCtrl", function ($scope, $http) {
        
        $scope.loading = false;
        $scope.systemInfo = {
            cpuInfo: {},
            ramInfo: {},
            storageInfo: {}
        };

        $scope.onInit = function() {
            console.log("[Dashboard-Hardware] initialized!");

            // Shows loader
            $scope.loading = true;

            // Request info to the API
            $scope.getCpuInfo();
            $scope.getCpuTemp();
            $scope.getSystemLoad();
            $scope.getRamInfo();
            $scope.getStorageInfo();
        };

        $scope.getCpuInfo = function() {
            console.log("[Dashboard-Hardware] requesting cpu info...");

            // Do the request
            $http.get(`${options.api.restv2}/hardware/cpu/info`)
                .success(function(response) {
                    // Treats the response
                    console.log(response);

                    // Packs the useful information into an Object
                    let cpuInfo = {
                        cores: response.cores,
                        physicalCores: response.physicalCores,
                        clock: response.speed,
                        turboBoostSpeed: Math.floor((response.speed / response.speedmax) * 100)
                    };

                    // Saves it into the system info object
                    $scope.systemInfo["cpuInfo"] = cpuInfo;
                });
        };

        $scope.getSystemLoad = function() {
            console.log("[Dashboard-Hardware] requesting system load...");

            // Do the request
            $http.get(`${options.api.restv2}/hardware/load`)
                .success(function(response) {
                    // Treats the response
                    console.log(response);

                    // Sums all the core loads
                    let coresLoad = response.cpus.reduce((a, b) => a + b.load, 0);
                    // Get the average cpu load by it's cores
                    let cpuLoad = Math.ceil(coresLoad / response.cpus.length) + "%";

                    console.log(`Average CPU usage: ${cpuLoad}º`);

                    // Saves it into the system info object
                    $scope.systemInfo["cpuInfo"]["usage"] = cpuInfo;
                });
        };

        $scope.getCpuTemp = function() {
            console.log("[Dashboard-Hardware] initialized!");

            // Do the request
            $http.get(`${options.api.restv2}/hardware/cpu/temps`)
                .success(function(response) {
                    // Treats the response
                    console.log(response);

                    // Sum all the single core temperature
                    let temp = response.cores.reduce((a, b) => a + b, 0);
                    // Gets the average core temperature
                    temp /= response.cores.length;
                    // Rounds up
                    temp = Math.ceil(temp);

                    console.log(`Average CPU temperature: ${temp}º`);

                    // Saves it into the system info object
                    $scope.systemInfo["cpuInfo"]["temperature"] = temp;
                });
        };

        $scope.getRamInfo = function() {
            console.log("[Dashboard-Hardware] initialized!");
        };

        $scope.getStorageInfo = function() {
            console.log("[Dashboard-Hardware] initialized!");
        };



    });
});