define(['modules/painelsupervisor/module', 'lodash', 'angular-highcharts'], function (module, _) {
    "use strict";

    module.registerController('PainelSupervisorCtrl', function ($scope, $http, $filter, $localStorage) {
        $scope.data = [];
        $scope.series = [];
        $scope.filtro = {};

        function PadTime(num) {
            return ("0" + num).slice(-2);
        }

        $scope.chartConfig11 = {
            chart: {
                type: 'spline'
            },
            title: {
                text: $scope.getWord('Nenhum registro encontrado')
            },
            subtitle: {
                text: $scope.getWord('Informe novos filtros e use o botão Atualizar')
            },
            yAxis: {
                title: {
                    text: $scope.getWord('Ligações (Qtd)')
                },
                min: 0,
                allowDecimals: false
            },
            xAxis: {
                categories: ["00:00", "01:00", "02:00", "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00", "11:00", "12:00",
                    "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00", "21:00", "22:00", "23:00"]
            },
            legend: {
                enabled: false
            },
            credits: {
                enabled: true,
                text: 'X5 Soluções - Xcontact',
                href: 'http://www.x5s.com.br'
            }
        };

        $scope.options = {
            //Boolean - Nao preencher com cor as curvas
            datasetFill: false
        };

        $scope.onClick = function (points, evt) {
            console.log(points, evt);
        };


        $scope.getFilas = function () {
            $scope.filas = $localStorage.User.filas;
            //$scope.filtro.fila = $scope.filas[0];
            //console.log($scope.filas);
        };

        $scope.status = [
            { valor: "Atendida", nome: $scope.getWord("Atendida") },
            { valor: "Abandonada", nome: $scope.getWord("Abandonada") },
            { valor: "Transbordou", nome: $scope.getWord("Transbordou") },
            { valor: "Não atendida", nome: $scope.getWord("Não atendida") },
            { valor: "Ocupado", nome: $scope.getWord("Ocupado") },
            { valor: "Falha", nome: $scope.getWord("Falha") },
        ];

        $scope.filterStatus = function (query) {
            return $scope.status.filter(function (o) {
                return o.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getSelfila = function () {
            if (!$scope.filtro.filas || $scope.filtro.filas.length === 0) return $scope.getWord('Todas');
            else return _.values(_.pluck($scope.filtro.filas, "text")).join(",");
            //_.map($scope.filtro.fila, function(valor, key){ return num * 3; }); _.values($scope.filtro.fila).join(",");
        };

        $scope.filterFilas = function (query) {
            var filas = $localStorage.User.filas;
            return filas.sort().filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getAgentes = function () {
            $http.get(options.api.restv3 + "/agente/all").
                success(function (data) {
                    $scope.agentes = data.dados;
                });
        };

        $scope.filterAgentes = function (query) {
            return _.sortBy($scope.agentes, 'fullname').filter(function (agente) {
                if ($scope.filtro.filas && $scope.filtro.filas.length > 0) {
                    for (var i = 0; i < $scope.filtro.filas.length; i++) {
                        if (agente.filas && agente.filas.indexOf($scope.filtro.filas[i].text) !== -1) {
                            return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                        }
                        else if (agente.fila && agente.fila.indexOf($scope.filtro.filas[i].text) !== -1) {
                            return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                        }
                    }
                }
                else {
                    return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                }
            });

        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $scope.filas = [];
            $scope.agentes = [];
            $scope.filtro = {};
            $scope.filtro.totalpor = 'hour';
            $scope.filtro.data_ini = new Date();
            $scope.filtro.data_fim = new Date();

            $scope.getFilas();
            $scope.getAgentes();
            $scope.getGrafico();
        };

        $scope.getGrafico = function () {
            $scope.data = [];
            $scope.series = [];

            var obs = [];

            var filtros = angular.copy($scope.filtro);
            filtros.data_ini = $filter('date')(filtros.data_ini, 'yyyy-MM-dd');
            filtros.data_fim = $filter('date')(filtros.data_fim, 'yyyy-MM-dd');
            delete filtros.totalpor;

            //Filas
            if (filtros.filas && filtros.filas.length > 0) {
                // filtros.filas = JSON.stringify($scope.filas.filter((x)=> {return filtros.filas.includes(x.id);}));
                filtros.filas = JSON.stringify($scope.filas.filter(x => filtros.filas.find(y => y.text === x)));
            } else {
                filtros.filas = "[]";
            }

            //Agentes
            if (filtros.agente && filtros.agente.length > 0) {
                // filtros.agentes = JSON.stringify(filtros.agente.map((x)=>{return x.fullname || x.nome;}));
                filtros.agentes = JSON.stringify(filtros.agente.map(x => x.fullname || x.nome));
            }
            delete filtros.agente;

            if (!filtros.hora_ini || filtros.hora_ini.length == 0) {
                filtros.data_ini = filtros.data_ini + ' 00:00:00';
            }
            else {
                filtros.data_ini = filtros.data_ini + ' ' + filtros.hora_ini + ":00";
                delete filtros.hora_ini;
            }
            if (!filtros.hora_fim || filtros.hora_fim.length == 0) {
                filtros.data_fim = filtros.data_fim + ' 23:59:59';
            }
            else {
                filtros.data_fim = filtros.data_fim + ' ' + filtros.hora_fim + ":00";
                delete filtros.hora_fim;
            }
            if (filtros.status && filtros.status.length > 0) {
                filtros.status = [];
                angular.forEach($scope.filtro.status, function (o) {
                    filtros.status.push(o.valor);
                });
                filtros.status = JSON.stringify(filtros.status);
            }
            else {
                delete filtros.status;
            }

            var par = [];
            if (filtros) {
                angular.forEach(filtros, function (v, k) {
                    if (v) par.push(k + "=" + encodeURI(v));
                });
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }

            $scope.loading = true;
            $scope.data = [];
            $scope.series = [];
            $http.get(options.api.restv2 + "/filas/ligacoes?max=-1" + pars).success(function (data) {
                if (data.dados && data.dados.length > 0) {
                    // Aplica trasformações de valores
                    data.dados = _.map(data.dados, function (row) {
                        var calldate = new Date(row.calldate);
                        row.mes = calldate.getMonth();
                        row.day = calldate.getDate() - 1; // Array do gráfico começa em Zero!
                        row.hour = calldate.getHours();
                        //row.hora = PadTime(calldate.getHours())+':00-'+PadTime(calldate.getHours())+':59';
                        row.dow = calldate.getDay();

                        return row;
                    });

                    var a = Array.apply(null, new Array(24)).map(Number.prototype.valueOf, 0);

                    if ($scope.filtro.totalpor == 'dow') {
                        $scope.chartConfig11.xAxis.categories = [$scope.getWord('Domingo'), $scope.getWord('Segunda'), $scope.getWord('Terça'), $scope.getWord('Quarta'), $scope.getWord('Quinta'), $scope.getWord('Sexta'), $scope.getWord('Sábado')];
                        a = Array.apply(null, new Array(7)).map(Number.prototype.valueOf, 0);
                    }
                    else if ($scope.filtro.totalpor == 'mes') {
                        $scope.chartConfig11.xAxis.categories = [
                            $scope.getWord('Janeiro'),
                            $scope.getWord('Fevereiro'),
                            $scope.getWord('Março'),
                            $scope.getWord('Abril'),
                            $scope.getWord('Maio'),
                            $scope.getWord('Junho'),
                            $scope.getWord('Julho'),
                            $scope.getWord('Agosto'),
                            $scope.getWord('Setembro'),
                            $scope.getWord('Outubro'),
                            $scope.getWord('Novembro'),
                            $scope.getWord('Dezembro')
                        ];
                        a = new Array(12).fill(0);
                    }
                    else if ($scope.filtro.totalpor == 'day') {
                        $scope.chartConfig11.xAxis.categories = ["01", "02", "03", "04", "05", "06", "07", "08", "09"];
                        for (var day = 10; day <= 31; day++) {
                            $scope.chartConfig11.xAxis.categories.push(day.toString());
                        }
                        // Nice code...
                        //a = Array.apply(null, new Array(31)).map(Number.prototype.valueOf, 0);
                        // Esse faz a mesma coisa!!
                        a = new Array(31).fill(0);
                    }
                    else {
                        $scope.chartConfig11.xAxis.categories = ["00:00", "01:00", "02:00", "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00", "11:00", "12:00",
                            "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00", "21:00", "22:00", "23:00"];
                    }
                    $scope.chartConfig11.title.text = '';
                    $scope.chartConfig11.subtitle.text = '';

                    console.log(data.dados);

                    $scope.chartConfig11.series = _.chain(data.dados)
                        .sortBy('status')
                        .groupBy('status')
                        .map(function (value, key) {
                            var ob = { name: key };
                            ob.data = angular.copy(a);
                            _.chain(value)
                                .sortBy($scope.filtro.totalpor)
                                .groupBy($scope.filtro.totalpor)
                                .map(function (v, k) {
                                    ob.data[k] = v.length;
                                });
                            return ob;
                        })
                        .value();

                    console.log($scope.chartConfig11.series);
                }
                else {
                    $scope.chartConfig11.xAxis.categories = [];
                    $scope.chartConfig11.series = [];
                    $scope.chartConfig11.title.text = $scope.getWord('Nenhum registro encontrado');
                    $scope.chartConfig11.subtitle.text = $scope.getWord('Informe novos filtros e use o botão Atualizar');
                }
                $scope.loading = false;
            });
        };
    });
});
