define(['modules/pausas/module'], function (module) {
    "use strict";

    return module.registerController('PausasCtrl', function ($scope, $http, paginationService, $localStorage) {
        $scope.row = {};
        $scope.campos_disable = false;

        $scope.sortBy = 'nome';
        $scope.sortDescending = false;
        $scope.searchText = '';

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        $scope.order = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.filas = $localStorage.User.filas;

        $scope.filter = function () {
            var results = $scope.filteredRT;
            results.length = 0;
            var searchText = $scope.searchText;
            var reminderTypes = $scope.reminderTypes;

            for (var i = 0; i < reminderTypes.length; ++i) {
                if (searchText.length > 0) {
                    if (reminderTypes[i].nome && reminderTypes[i].nome.toLowerCase().includes(searchText.toLowerCase())
                        || (reminderTypes[i].fila && reminderTypes[i].fila.toLowerCase().includes(searchText.toLowerCase()))) {
                        results.push(reminderTypes[i]);
                    }
                } else {
                    results.push(reminderTypes[i]);
                }
            }
            results.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $scope.carregando = true;
            $http.get(`${options.api.restv3}/pausa?max=-1`).
                success(function (data) {
                    if (data) {
                        $scope.reminderTypes = data.dados.filter(function (obj) {
                            return obj.editavel == true && (!obj.fila || $scope.filas.includes(obj.fila));
                        });
                        $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
                        $scope.filter();
                    }
                    $scope.carregando = false;
                });
        };
        $scope.novo = () => {
            $scope.pop = {
                editavel: true,
                comando: "Pausa",
                produtiva: false
            };
            $scope.showPausaPopup = true;
        }

        $scope.editar = async (row) => {
            row.load = true;

            let t = await $http.get(`${options.api.restv3}/pausa/${row.id}`);
            $scope.pop = t.data.dados;

            row.load = false;
            $scope.showPausaPopup = true;
        }

        $scope.Salvar = async (f) => {
            if (f.$valid) {
                let dados = JSON.parse(JSON.stringify($scope.pop));
                delete dados.id;

                try {
                    if ($scope.pop.id) {
                        await $http.put(`${options.api.restv3}/pausa/${$scope.pop.id}`, dados);
                    }
                    else {
                        await $http.post(`${options.api.restv3}/pausa`, dados);
                    }

                    $scope.showPausaPopup = false;
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Pausa atualizada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });
                    $scope.load();
                } catch (error) {
                    console.log(error);
                }
            }
        }

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover " + row.nome + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv3 + "/pausa/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Pausa apagada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });

                    $scope.load();
                })
        };
    });
});