define(['modules/ramais/module'], function (module) {
    "use strict";

    return module.registerController('RamaisCtrl', function ($scope, $http, $stateParams, $rootScope, Upload, paginationService, ApiService) {
        $scope.row = {};
        $scope.order = 'name';
        $scope.order_sentido = 'ASC';
        $scope.grupos = {};

        $scope.passwordLength = 12;
        $scope.addUpper = true;
        $scope.addNumbers = true;
        $scope.addSymbols = false;

        // New paginate
        $scope.sortBy = 'numero';
        $scope.sortDescending = false;
        $scope.searchText = '';
        $scope.maxSize = 3; // Máximo de páginas exibidas no bloco de páginação
        $scope.currentPage = 1; // Página Atual
        $scope.numPerPage = 15; // Registros por página

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }
        $scope.idiomas = [{ valor: "pt-BR", nome: 'Português' }, { valor: 'es', nome: 'Espanhol' }, { valor: 'en', nome: 'Inglês' }];
        $scope.dtmfs = [{ valor: "auto", nome: 'Auto' }, { valor: 'rfc2833', nome: 'RFC2833' }, { valor: 'info', nome: 'Info' }, { valor: 'shortinfo', nome: 'Shortinfo' }, { valor: 'inband', nome: 'Inband' }];
        $scope.insecures = [{ valor: "no", nome: 'no' }, { valor: 'port', nome: 'port' }, { valor: 'invite', nome: 'invite' }, { valor: 'port,invite', nome: 'port,invite' }];
        $scope.regras = [{ valor: "ordem", nome: $scope.getWord('Ordem') }, { valor: 'todos', nome: $scope.getWord('Todos') }];
        $scope.tipos = [{ valor: "udp", nome: 'SIP' }, { valor: 'wss', nome: 'WebRTC' }];
        $scope.codecs = [
            "opus",
            "g729",
            "gsm",
            "g722",
            "g726",
            "ulaw",
            "alaw",
            "h261",
            "h263",
            "h263p",
            "h264",
            "mpeg4"
        ];

        $scope.getTipo = (tipo) => {
            let t = $scope.tipos.find(x => x.valor === tipo);
            return t ? t.nome : null;
        }

        $scope.createPassword = function () {
            var lowerCharacters = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'];
            var upperCharacters = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'];
            var numbers = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'];
            var symbols = ['!', '"', '"', '#', '$', '%', '&', '\'', '(', ')', '*', '+', ',', '-', '.', '/', ':', ';', '<', '=', '>', '?', '@', '[', '\\', ']', '^', '_', '`', '{', '|', '}', '~'];
            var finalCharacters = lowerCharacters;
            if ($scope.addUpper) {
                finalCharacters = finalCharacters.concat(upperCharacters);
            }
            if ($scope.addNumbers) {
                finalCharacters = finalCharacters.concat(numbers);
            }
            if ($scope.addSymbols) {
                finalCharacters = finalCharacters.concat(symbols);
            }
            var passwordArray = [];
            for (var i = 1; i < $scope.passwordLength; i++) {
                passwordArray.push(finalCharacters[Math.floor(Math.random() * finalCharacters.length)]);
            };
            $scope.ramal.senha = passwordArray.join("");
        };

        $scope.filterCodecs = function (query) {
            return $scope.codecs.filter(function (t) {
                return t.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };

        $scope.filterGrupos = function (query) {
            return $scope.grupos.filter(function (t) {
                return t.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };

        $scope.getGrupos = function () {
            $http.get(options.api.restv3 + "/grupo?max=-1").
                success(function (data) {
                    $scope.grupos = data.dados;
                });
        };

        $scope.getContextos = function () {
            let filters = [];

            filters.push("max=-1");
            filters.push("sentido=" + encodeURIComponent("[\"Saida\"]"));

            // Adiciona a ordenação por nome
            filters.push("order=" + encodeURIComponent(JSON.stringify([{ "campo": "context", "ordem": $scope.order_sentido }])));

            $http.get(`${options.api.restv3}/contexto?${filters.join("&")}`)
                .success(data => {
                    $scope.contextos = data.dados;
                });
        };

        $scope.CarregarItem = function (id) {
            angular.forEach($rootScope.rows, r => {
                if (r.id == id) {
                    $scope.row = angular.copy(r);
                    return;
                }
            });

            $http.get(options.api.restv3 + "/grupo?max=-1")
                .success(data => {
                    $scope.grupos = data.dados;

                    if ($scope.row.callgroup) {
                        angular.forEach($scope.grupos, function (f) {
                            if (f.id == $scope.row.callgroup) {
                                $scope.row.callgroup = f;
                                return;
                            }
                        });
                    }

                    if ($scope.row.pickupgroup) {
                        var res = $scope.row.pickupgroup.split(",");
                        angular.forEach(res, function (pi) {
                            angular.forEach($scope.grupos, function (f) {
                                if (f.id == pi) {
                                    f.marcado = true;
                                    return;
                                }
                            });
                        });
                    }

                    if ($scope.row.allow) {
                        console.log($scope.row);
                        var cos = $scope.row.allow.split(",");

                        angular.forEach(cos, function (co) {
                            angular.forEach($scope.codecs, function (c) {
                                if (c.nome == co) {
                                    c.marcado = true;
                                    return;
                                }
                            });
                        });
                    }
                });
        };

        $scope.load = function () {

            $http.get(`${options.api.restv2}/ramais`).
                success(function (data) {
                    if (data) {
                        $scope.reminderTypes = data.filter(r => r.xc_tipo == 'ramal');
                        $scope.reminderTypes = $scope.reminderTypes.map(r => {
                            r.tipo = 'WebRTC';
                            if (r.transport == 'udp') r.tipo = 'SIP';
                            return r;
                        });
                        $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));

                        $scope.filter();
                    }
                });
        };

        $scope.order = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        function filterIt(arr, searchKey) {
            return arr.filter(function (obj) {
                return Object.keys(obj).some(function (key) {
                    if (obj[key]) {
                        if (Array.isArray(obj[key])) {
                            let find = filterIt(obj[key], searchKey);
                            if (find && find.length > 0) {
                                return find;
                            }
                        }
                        else {
                            return obj[key].toString().toUpperCase().includes(searchKey.toString().toUpperCase());
                        }
                    }
                })
            });
        }

        $scope.filter = function () {
            $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));

            if ($scope.searchText && $scope.searchText.length > 0) {
                $scope.filteredRT = filterIt($scope.filteredRT, $scope.searchText);
            }

            $scope.filteredRT.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        $scope.Editar = function () {
            $scope.campos_disable = false;
        };

        $scope.Cancelar_editar = function () {
            $scope.CarregarItem($stateParams.id);
            $scope.campos_disable = true;
        };

        $scope.Salvar = function () {
            $http.post(`${options.api.restv3}/ramal`, $scope.row)
                .success(data => {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });
                    $rootScope.$state.transitionTo('app.ramais');
                })
        };

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.ramais');
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover o ramal " + row.numero + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, ButtonPressed => {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv2 + "/ramal/" + id)
                .success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Ramal apagado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });

                    $scope.load();
                });
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.setOrder = function (coluna) {
            if ($scope.order === coluna) {
                $scope.order_sentido = $scope.order_sentido === 'ASC' ? 'DESC' : 'ASC';
            } else {
                $scope.order = coluna;
                $scope.order_sentido = 'ASC';
            }

            $scope.loadRamais();
        };

        $scope.showpop = async function (row) {
            row.load = true;
            $scope.getGrupos();
            $scope.getContextos();

            $scope.numero_disable = true;
            $scope.campos_disable = false;

            let r = await ApiService.get(`${options.api.restv2}/ramal/${row.id}`);
            row.load = false;
            $scope.ramal = r.data;

            $scope.ramal.allow = $scope.ramal.allow.split(',');

            if ($scope.ramal.pickupgroup) {
                var res = $scope.ramal.pickupgroup.split(",");
                var pk_grupos = [];
                angular.forEach(res, function (pi) {
                    angular.forEach($scope.grupos, function (f) {
                        if (f.id == pi) {
                            pk_grupos.push(f);
                            return;
                        }
                    });
                });
                $scope.ramal.pickupgroup = pk_grupos;
            }
            $scope.ramal.nat2 = $scope.ramal.nat && $scope.ramal.nat === 'force_rport,comedia' ? true : false;
            $scope.ramal.icesupport2 = $scope.ramal.icesupport && $scope.ramal.icesupport === 'yes' ? true : false;
            if ($scope.ramal.ramalMulti && $scope.ramal.ramalMulti.length > 0) {
                $scope.ramal.ramalMulti.sort(function (a, b) {
                    return a.ordem - b.ordem;
                });
            }

            $scope.showRamalPopup = true;
        };

        $scope.showNewPop = function (row) {
            $scope.numero_disable = false;
            $scope.campos_disable = false;
            $scope.ramal = {};
            $scope.createPassword();
            $scope.getGrupos();
            $scope.getContextos();

            $scope.ramal.allow = ['opus', 'g729', 'ulaw'];
            $scope.ramal.pickupgroup = [];
            $scope.ramal.nat2 = false;
            $scope.ramal.transport = 'udp';
            $scope.ramal.ramalMulti = [];
            $scope.showRamalPopup = true;
        };

        $scope.atualizarRamal = function (f1) {
            if (f1.$valid) {
                let object = { ...$scope.ramal }

                if (object.transport === "wss") {
                    object.encryption = "yes";
                    object.avpf = "yes";
                    object.dtlsenable = "yes";
                    object.dtlsverify = "fingerprint";
                    object.dtlscertfile = "/etc/asterisk/keys/asterisk.pem";
                    object.dtlssetup = "actpass";
                    object.rtcp_mux = "yes";
                    object.hassip = "no";
                } else {
                    object.encryption = "no";
                    object.avpf = "no";
                    object.dtlsenable = "no";
                    object.dtlsverify = null;
                    object.dtlscertfile = null;
                    object.dtlssetup = null;
                    object.rtcp_mux = "no";
                    object.hassip = "no";
                }

                if (object.pickupgroup) {
                    object.pickupgroup = object.pickupgroup.map(x => x.id).join(",");
                }

                if (object.ramalMulti) {
                    object.ramalMulti = object.ramalMulti.map((x, i) => {
                        delete x.id;
                        x.ordem = i + 1;
                        return x;
                    });
                }

                // Removes useless information
                delete object.id;
                delete object.type;
                delete object.host;
                delete object.ipaddr;
                delete object.port;
                delete object.regseconds;
                delete object.defaultuser;
                delete object.fullcontact;
                delete object.useragent;
                delete object.lastms;
                delete object.callbackextension;
                delete object.regserver;
                delete object.status;
                delete object.qualify;
                delete object.qualifyfreq;
                delete object["call - limit"];
                delete object["call-limit"];
                delete object.cc_agent_policy;
                delete object.cc_monitor_policy;
                delete object.accountcode;
                delete object.amaflags;
                delete object.dtmfmode;
                delete object.fromuser;
                delete object.fromdomain;
                delete object.mailbox;
                delete object.permit;
                delete object.deny;
                delete object.mask;
                delete object.restrictcid;
                delete object.rtptimeout;
                delete object.rtpholdtimeout;
                delete object.musiconhold;
                delete object.regexten;
                delete object.cancallforward;
                delete object.callerid;
                delete object.hasvoicemail;
                delete object.nat2;
                delete object.icesupport2;

                // Format properties
                object.allow = object.allow.map(x => x.text).join(",");

                if ($scope.ramal.nat2) {
                    object.nat = 'force_rport,comedia';
                }
                else {
                    object.nat = 'no';
                }

                if ($scope.ramal.icesupport2) {
                    object.icesupport = 'yes';
                }
                else {
                    object.icesupport = 'no';
                }

                if ($scope.ramal && $scope.ramal.id) {
                    // Faz a requisição
                    $http.put(`${options.api.restv2}/ramal/${$scope.ramal.id}`, object)
                        .success(data => {
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Atualizado com sucesso!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });

                            $scope.load();
                            $scope.showRamalPopup = false;
                        });
                } else {
                    // Faz a requisição
                    $http.post(`${options.api.restv2}/ramal`, object)
                        .success(data => {
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Criado com sucesso!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });

                            $scope.load();
                            $scope.showRamalPopup = false;
                        });
                }
            }
        };

        $scope.uploadFiles = function (files) {
            if (files) {
                $.x5loading({
                    title: "Enviando arquivos, aguarde..",
                    content: "<i class='fa fa-4x fa-spinner fa-spin'></i>",
                    color: "#3176b0"
                });
                angular.forEach(files, function (file) {
                    if (file && !file.$error) {
                        Upload.upload({
                            url: options.api.restv3 + "/ramal/upload",
                            file: file
                        }).then(function (resp) {
                            console.log('Sucesso!!!');
                            console.log(resp);
                            $("#x5loading").remove();
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Contatos importados!</i>",
                                color: "#739E73",
                                timeout: 3000
                            });
                            $scope.load();
                        }, function (resp) {
                            console.log('Error status: ' + resp.status);
                        }, function (evt) {
                            console.log(evt);
                            //var progressPercentage = parseInt(100.0 * evt.loaded / evt.total);
                            //console.log('progress: ' + progressPercentage + '% ' + evt.config.file.file.name);
                        });
                    }
                });
            }
        };

        $scope.deploy = function (id) {
            let p = "";
            if (id) p = "/" + id;

            $http.get(options.api.restv3 + "/ramal/deploy" + p)
                .success(function (data) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: `<i class='fa fa-check'></i> <i>${data.total} arquivo${data.total > 1 ? 's' : ''} gerado${data.total > 1 ? 's' : ''}!</i>`,
                        color: "#739E73",
                        timeout: 2000
                    });
                });
        };

        $scope.AddItem = function () {
            if ($scope.ramal.ramalMulti.length > 4) {
                $.smallBox({
                    title: "ERRO!!",
                    content: "São permitidos no máximo 5 dispositivos múltiplos",
                    color: "#C46A69",
                });
            }
            else {
                var d = { id: '', nome: '', numero: `${$scope.ramal.numero}_${$scope.ramal.ramalMulti.length + 1}`, ramalID: $scope.ramal.id };
                $scope.ramal.ramalMulti.push(d);
            }
        };

        $scope.DelItem = function (id) {
            $scope.ramal.ramalMulti.splice(id, 1);
        };

        $scope.ItemMove = function (fromIndex, toIndex) {
            console.log(`Move from ${fromIndex} to ${toIndex}`);

            var element = $scope.ramal.ramalMulti[fromIndex];
            $scope.ramal.ramalMulti.splice(fromIndex, 1);
            $scope.ramal.ramalMulti.splice(toIndex, 0, element);
        };



        $scope.loadRamais = function (page = 1) {
            let filters = [];

            // Adiciona a ordenação por nome
            filters.push("order=" + encodeURIComponent(JSON.stringify([{ "campo": $scope.order, "ordem": $scope.order_sentido }])));

            // Adiciona o filtro por nome
            if ($scope.Pesquisa) filters.push("name=" + encodeURIComponent($scope.Pesquisa));

            // Adiciona o filtro da página
            filters.push("page=" + (page || $scope.atual || 1));

            // Faz a requisição
            $http.get(`${options.api.restv3}/ramal?${filters.join("&")}`)
                .success(data => {
                    $scope.atual = (page || $scope.atual || 1);
                    $scope.table = data;
                    $scope.rows = data.dados;
                });
        };


        if ($rootScope.$state.current.url.indexOf('ver') > 0) {
            $scope.campos_disable = true;
        }

        if ($stateParams.id) {
            $scope.CarregarItem($stateParams.id);
        }
    });
});