define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ngTagsInput',
    'mathjs',
    'underscore'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.rotas', [
        'ui.router',
        'ngResource',
        'ngTagsInput'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.rotas', {
                url: '/rotas',                
                data: {
                    title: 'Rotas',
                    permissions: {
                        only: ['app.rotas']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/rotas/views/lista.html',
                        controller: 'RotasCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/rotas/controllers/RotasCtrl',
                                'modules/rotas/directives/ShowPopup',
                                'modules/rotas/directives/popcloneGrupo'
                            ])
                        }                      
                    }
                }
            })
            
            .state('app.rotas.editar', {
                url: '/editar/:prefixo/:context',
                data: {
                    title: 'Rotas',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/rotas/views/form.html',
                        controller: 'RotasCtrl',
                        parent: 'app.rotas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/rotas/controllers/RotasCtrl',
                                'modules/rotas/directives/rotasFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.rotas.novo', {
                url: '/novo/:context',
                data: {
                    title: 'Nova Rota',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/rotas/views/form.html',
                        controller: 'RotasCtrl',
                        parent: 'app.rotas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/rotas/controllers/RotasCtrl',
                                'modules/rotas/directives/rotasFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.rotas.new', {
                url: '/new',
                data: {
                    title: 'Nova Rota',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/rotas/views/form.html',
                        controller: 'RotasCtrl',
                        parent: 'app.rotas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/rotas/controllers/RotasCtrl',
                                'modules/rotas/directives/rotasFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.rotas.ver', {
                url: '/ver/:prefixo/:context',
                data: {
                    title: 'Rota',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/rotas/views/form.html',
                        controller: 'RotasCtrl',
                        parent: 'app.rotas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/rotas/controllers/RotasCtrl',
                                'modules/rotas/directives/rotasFormValidate'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$filter) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.rotas', function () {
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.rotas",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: "Você não tem acesso ao módulo de Rotas."
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});