define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.sms', [
        'ui.router',
        'ngResource'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.sms', {
                url: '/sms',                
                data: {
                    title: 'SMS',
                    permissions: {
                        only: ['app.sms']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/canais.html',
                        controller: 'CanaisCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/CanaisCtrl',
                                'modules/sms/directives/sendPopup'
                            ])
                        }                        
                    }
                }
            })
            
            .state('app.sms.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Sms',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/form.canais.html',
                        controller: 'CanaisCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/CanaisCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.novo', {
                url: '/novo',
                data: {
                    title: 'Novo Sms',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/form.canais.html',
                        controller: 'CanaisCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/CanaisCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.operadoras', {
                url: '/operadoras',
                data: {
                    title: 'Operadoras'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/operadoras.html',
                        controller: 'OperadorasCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/OperadorasCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.operadora_novo', {
                url: '/operadora_novo',
                data: {
                    title: 'Operadora',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/form.operadora.html',
                        controller: 'OperadorasCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/OperadorasCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.operadora_editar', {
                url: '/operadora_editar/:id',
                data: {
                    title: 'Operadora',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/form.operadora.html',
                        controller: 'OperadorasCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/OperadorasCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.grupos', {
                url: '/grupos',
                data: {
                    title: 'Grupos'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.grupo_novo', {
                url: '/grupo_novo',
                data: {
                    title: 'Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/form.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.sms.grupo_editar', {
                url: '/grupo_editar/:id',
                data: {
                    title: 'Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/sms/views/form.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.sms',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/sms/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$rootScope) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.sms', function () {            
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.sms",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: $rootScope.getWord("Você não tem acesso a Sms.")
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});