define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'angularUtils.directives.dirPagination'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.tags', [
        'ui.router',
        'ngResource',
        'angularUtils.directives.dirPagination'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.tags', {
                url: '/tags',
                data: {
                    title: 'Tags',
                    permissions: {
                        only: ['app.tags']
                    }
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/tags/views/lista.html',
                        controller: 'TagsCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/tags/controllers/TagsCtrl',
                                'modules/forms/directives/input/smartColorpicker',
                            ])
                        }
                    }
                }
            })

            .state('app.tags.grupos', {
                url: '/tags/grupos',
                data: {
                    title: 'Grupos'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/tags/views/lista.grupos.html',
                        controller: 'TagsGruposCtrl',
                        parent: 'app.tags',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/tags/controllers/TagsGruposCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato, Permission, $localStorage, ModuloPermissao) {
        module.lazy = $couchPotato;

        Permission.defineRole('app.tags', function () {
            if (!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.tags", $localStorage.User.modulos)) {
                $localStorage.msg = {
                    tipo: "Erro",
                    txt: "Você não tem acesso ao módulo de Tags."
                };
                return false;
            }
            else {
                return true;
            }
        });
    });

    return module;

});