define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'isteven-multi-select'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.troncos', [
        'ui.router',
        'ngResource',
        'isteven-multi-select'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.troncos', {
                url: '/troncos',                
                data: {
                    title: 'Troncos',
                    permissions: {
                        only: ['app.troncos']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/troncos/views/lista.html',
                        controller: 'TroncosCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/troncos/controllers/TroncosCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.troncos.editar', {
                url: '/editar/:id/:serial',
                data: {
                    title: 'Tronco',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/troncos/views/form.html',
                        controller: 'TroncosCtrl',
                        parent: 'app.troncos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/troncos/directives/channelsPopup',
                                'modules/troncos/controllers/TroncosCtrl',
                                'modules/troncos/directives/troncosFormValidate'
                            ])
                        }
                    }
                }
            })            
            
            .state('app.troncos.novo', {
                url: '/novo/:serial',
                data: {
                    title: 'Novo Tronco',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/troncos/views/form.html',
                        controller: 'TroncosCtrl',
                        parent: 'app.troncos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/troncos/directives/channelsPopup',
                                'modules/troncos/controllers/TroncosCtrl',
                                'modules/troncos/directives/troncosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.troncos.ver', {
                url: '/ver/:id/:serial',
                data: {
                    title: 'Tronco',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/troncos/views/form.html',
                        controller: 'TroncosCtrl',
                        parent: 'app.troncos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/troncos/directives/channelsPopup',
                                'modules/troncos/controllers/TroncosCtrl',
                                'modules/troncos/directives/troncosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.troncos.ip', {
                url: '/ip/:id',
                data: {
                    title: 'Tronco IP',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/troncos/views/sipform.html',
                        controller: 'TroncosCtrl',
                        parent: 'app.troncos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/ui/directives/smartHtmlPopover',
                                'modules/troncos/controllers/TroncosCtrl',
                                'modules/troncos/directives/sipFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.troncos.ipcriar', {
                url: '/ipcriar',
                data: {
                    title: 'Tronco IP'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/troncos/views/sipform.html',
                        controller: 'TroncosCtrl',
                        parent: 'app.troncos',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/ui/directives/smartHtmlPopover',
                                'modules/troncos/controllers/TroncosCtrl',
                                'modules/troncos/directives/sipFormValidate'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,$localStorage,ModuloPermissao) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.troncos', function () {
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.troncos",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: "Você não tem acesso aos Troncos."
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});