"use strict";
define(['modules/ura/module'], function (module) {
    return module.registerController('UraCtrl', function ($scope, $http, $stateParams, $rootScope) {
        $scope.carregando = false;
        $scope.filtro = {};
        $scope.popup = {};
        $scope.order = [{ "campo": "context", "ordem": "ASC" }];

        $scope.filterTurnos = function (query) {
            return $scope.turnos.filter(function (t) {
                return t.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };

        $scope.getAudios = function () {
            $http.get(options.api.restv3 + "/audio?max=-1").
                success(function (data) {
                    $scope.audios = data.dados;
                });
        };

        $scope.getNomeAudio = function (nome) {
            if (!nome) return "";
            return nome.substr(0, nome.lastIndexOf("."));
        };

        $scope.getRamais = function () {
            $http.get(options.api.restv3 + "/ramal?max=-1&order=" + encodeURI(JSON.stringify([{ campo: 'name', ordem: 'ASC' }]))).
                success(function (data) {
                    $scope.ramais = data.dados;
                });
        };

        $scope.getFilas = function () {
            $http.get(options.api.restv3 + "/fila?max=-1").
                success(function (data) {
                    $scope.filas = data.dados;
                });
        };

        $scope.getTurnos = function () {
            $http.get(options.api.restv3 + "/horario?max=-1&tipo=T").
                success(function (data) {
                    if (data.total > 0)
                        $scope.turnos = data.dados.map((x) => {
                            if (!x.descricao) x.descricao = x.nome;
                            return x;
                        });
                });
        };

        $scope.getTroncos = function () {
            $http.get(options.api.restv3 + "/tronco?max=-1&order=" + encodeURI(JSON.stringify([{ campo: 'nome', ordem: 'ASC' }]))).
                success(function (data) {
                    $scope.troncos = data.dados;
                });
        };

        $scope.getRotasSaida = function () {
            $http.get(options.api.restv3 + "/contexto?max=-1&page=1&sentido=" + encodeURI(JSON.stringify(["Saida"]))).
                success(function (data) {
                    $scope.rotas_saida = data.dados;
                });
        };

        $scope.getURAS = function () {
            $http.get(options.api.restv3 + "/contexto?max=-1&sentido=" + encodeURI(JSON.stringify(["URA"]))).
                success(function (data) {
                    $scope.rotas_uras = data.dados;
                });
        };

        $scope.AddOpcao = function () {
            $scope.popup.ura_menu.push({ op: undefined, rotas: [{ app: "", par1: false, par2: false, par3: 1 }] });
        };

        $scope.AddLinha = function (op) {
            op.rotas.push({ app: "", par1: false, par2: false, par3: 1 });
        };

        $scope.DelOpcao = function (idx) {
            for (var i = 0; i < $scope.opcoes.length; i++) {
                if ($scope.popup.ura_menu[idx].op == $scope.opcoes[i].valor) $scope.opcoes[i].sel = false;
            }
            $scope.popup.ura_menu.splice(idx, 1);
        };

        $scope.DelLinha = function (op, idx) {
            op.rotas.splice(idx, 1);
        };

        $scope.SelectOp = function (op) {
            $scope.opcoes = [
                { valor: 0, sel: false },
                { valor: 1, sel: false },
                { valor: 2, sel: false },
                { valor: 3, sel: false },
                { valor: 4, sel: false },
                { valor: 5, sel: false },
                { valor: 6, sel: false },
                { valor: 7, sel: false },
                { valor: 8, sel: false },
                { valor: 9, sel: false }
            ];
            for (var i = 0; i < $scope.opcoes.length; i++) {
                angular.forEach($scope.popup.ura_menu, function (m) {
                    if (m.op == $scope.opcoes[i].valor) $scope.opcoes[i].sel = true;
                });
                if ($scope.opcoes[i].valor == op) {
                    $scope.opcoes[i].sel = true;
                }
            }
        };

        $scope.CleanPars = function (op) {
            op.par1 = undefined;
            op.par2 = undefined;
            op.par3 = 1;
        };

        $scope.Aplicacoes = [
            { valor: "RAMAL", depara: "Dial", nome: $scope.getWord('Ramal Interno') },
            { valor: "FILA", depara: "Macro", nome: $scope.getWord('Fila') },
            { valor: "TTS", nome: $scope.getWord('TTS') },
            { valor: "URA", depara: "Goto", nome: $scope.getWord('URA') },
            { valor: "EXTERNO", depara: "Macro", nome: $scope.getWord('Número externo') },
            { valor: "PLAYBACK", depara: "Playback", nome: $scope.getWord('Executar áudio') },
            { valor: "CUSTON", nome: $scope.getWord('Personalizado') },
        ];

        $scope.UraMenu = [
            { valor: "s", nome: $scope.getWord('Inicio') },
            { valor: "0", nome: $scope.getWord('Opção 0') },
            { valor: "1", nome: $scope.getWord('Opção 1') },
            { valor: "2", nome: $scope.getWord('Opção 2') },
            { valor: "3", nome: $scope.getWord('Opção 3') },
            { valor: "4", nome: $scope.getWord('Opção 4') },
            { valor: "5", nome: $scope.getWord('Opção 5') },
            { valor: "6", nome: $scope.getWord('Opção 6') },
            { valor: "7", nome: $scope.getWord('Opção 7') },
            { valor: "8", nome: $scope.getWord('Opção 8') },
            { valor: "9", nome: $scope.getWord('Opção 9') },
        ];

        $scope.init = function () {
            $scope.filtro.page = { valor: 1 };
            $scope.load();
        };

        $scope.gotoPage = function (page) {
            $scope.filtro.page = { valor: page ? page : 1 };
            $scope.load();
        }

        $scope.load = function () {
            $scope.carregando = true;
            var par = [];
            if ($scope.filtro) {
                angular.forEach($scope.filtro, function (v, k) {
                    if (v) par.push(k + "=" + v.valor);
                });
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }
            $http.get(options.api.restv3 + "/contexto?max=15&sentido=" + encodeURI(JSON.stringify(["URA"])) + "&order=" + encodeURI(JSON.stringify($scope.order)) + pars).
                success(function (data) {
                    $scope.table = data;
                    $scope.carregando = false;
                });
        };

        $scope.showpopup = async function (r) {
            //$('#myTab1 li.active').removeClass('active');
            //$("#myTab1 li:first").addClass("active");        
            $('#myTab1 a[href="#s1"]').tab('show');

            $scope.audios = [];
            $scope.ramais = [];
            $scope.filas = [];
            $scope.turnos = [];
            $scope.rotas_saida = [];
            $scope.rotas_uras = [];

            await $scope.getAudios();
            await $scope.getRamais();
            await $scope.getFilas();
            await $scope.getTurnos();
            await $scope.getRotasSaida();
            await $scope.getURAS();

            $scope.opcoes = [
                { valor: 0, sel: false },
                { valor: 1, sel: false },
                { valor: 2, sel: false },
                { valor: 3, sel: false },
                { valor: 4, sel: false },
                { valor: 5, sel: false },
                { valor: 6, sel: false },
                { valor: 7, sel: false },
                { valor: 8, sel: false },
                { valor: 9, sel: false }
            ];
            $scope.popup = {};
            $scope.popup.turnos = [];
            $scope.popup.digito_timeout = 3;
            $scope.popup.resposta_timeout = 10;
            $scope.popup.repetir = 3;
            $scope.popup.ring = false;
            $scope.popup.atender = false;
            $scope.popup.redireciona_feriado = { app: "", par1: "", par2: "", par3: 1 };
            $scope.popup.redireciona_turno = { app: "", par1: "", par2: "", par3: 1 };

            // Edicao
            if (r) {
                $scope.popup.new = false;
                $scope.popup.row = r;
                r.load = true;
                $http.get(options.api.restv3 + "/rota/URA?context=" + encodeURI(r.context)).
                    success(function (data) {
                        DBtoURA(data.dados);
                        r.load = false;
                    });
            }
            else {
                $scope.popup.new = true;
                $scope.popup.ura_menu = [{ op: undefined, rotas: [{ app: undefined, par1: false, par2: false, par3: 1 }] }];
                $scope.showUraPopup = true;
            }
        };

        function formataRota(r) {
            var ob = { app: "CUSTON", par1: r.app, par2: r.appdata, par3: 1 };

            if (r.app == "Playback") {
                ob.app = "PLAYBACK";
                ob.par1 = r.appdata;
                ob.par2 = false;
            }
            if (r.app == "Macro" && r.appdata.includes('disca_ramal,')) {
                ob.app = "RAMAL";
                ob.par1 = r.appdata.split(',')[1];
                ob.par2 = false;
            }
            if (r.app == "Macro" && r.appdata.includes('grava_ramal,')) {
                ob.app = "RAMAL";
                ob.par1 = r.appdata.split(',')[1];
                ob.par2 = true;
            }
            if (r.app == "Macro" && r.appdata.includes('fila,')) {
                ob.app = "FILA";
                ob.par1 = r.appdata.split(',')[1];
                ob.par2 = true;
            }
            if (r.app == "Macro" && r.appdata.includes('fila_naograva,')) {
                ob.app = "FILA";
                ob.par1 = r.appdata.split(',')[1];
                ob.par2 = false;
            }
            if (r.app == "AGI" && r.appdata.includes('tts.php,')) {
                ob.app = "TTS";
                ob.par1 = r.appdata.substring(r.appdata.indexOf(',') + 1).replace(/\"/g, '');
                //ob.par1 = r.appdata.split(',')[1].replace(/\"/g, '');
                ob.par2 = false;
            }

            if (r.app == "Goto") {
                var externo = $scope.rotas_saida.find(item => r.appdata.includes(item['context']));
                var ura = $scope.rotas_uras.find(item => r.appdata.includes(item['context']));
                var appdata = r.appdata.split(',');
                if (externo) {
                    ob.app = "EXTERNO";
                    ob.par1 = appdata[0];
                    ob.par2 = appdata[1];
                }
                else if (ura) {
                    ob.app = "URA";
                    ob.par1 = appdata[0];
                    ob.par2 = appdata[1];
                    ob.par3 = appdata[2];
                }
            }
            return ob;
        }

        function DBtoURA(dados) {
            $scope.popup.ura_menu = [];
            angular.forEach(dados, function (ura) {
                $scope.popup.nome = ura.contexto;
                ura.rotas.forEach(r => {
                    if (r.exten === 's' && r.app === 'BackGround') {
                        $scope.popup.audio = r.appdata;
                    }
                    if (r.exten === 'i' && r.app === 'Playback') {
                        $scope.popup.audio_invalido = r.appdata;
                    }
                    if (r.exten === 's' && r.app === 'Set' && r.appdata.includes('REPETIR')) {
                        $scope.popup.repetir = r.appdata.split('=')[1];
                    }
                    if (r.exten === 's' && r.app === 'Set' && r.appdata.includes('TIMEOUT(digit)')) {
                        $scope.popup.digito_timeout = r.appdata.split('=')[1];
                    }
                    if (r.exten === 's' && r.app === 'Set' && r.appdata.includes('TIMEOUT(response)')) {
                        $scope.popup.resposta_timeout = r.appdata.split('=')[1];
                    }
                    if (r.exten === 's' && r.app === 'Ringing') {
                        $scope.popup.ring = true;
                    }
                    if (r.exten === 's' && r.app === 'Answer') {
                        $scope.popup.atender = true;
                    }
                    // Turnos
                    if (r.exten === 's' && r.app === 'GotoIf' && r.appdata.includes('XCONTACT_TURNO')) {
                        if (r.appdata.indexOf('ARRAY') > 0) {
                            var turnos = r.appdata.substr(r.appdata.indexOf('ARRAY') + 6);
                            turnos = turnos.substr(0, turnos.indexOf(']')).split(',');
                            angular.forEach(turnos, function (t) {
                                var t2 = $scope.turnos.find(item => item['id'] == t);
                                if (t2) {
                                    $scope.popup.turnos.push(t2);
                                }
                            });
                        }
                    }
                    if (r.exten === 'fora_turno') {
                        $scope.popup.verifica_turno = true;
                        $scope.popup.redireciona_turno = formataRota(r);
                    }
                    // Feriados
                    if (r.exten === 'feriado') {
                        $scope.popup.verifica_feriado = true;
                        $scope.popup.redireciona_feriado = formataRota(r);
                    }

                    // Opcoes            
                    var op = $scope.opcoes.find(item => item['valor'] == r.exten);
                    if (op) {
                        var rota = $scope.popup.ura_menu.find(item => item['op'] == r.exten);
                        var ob = formataRota(r);
                        if (rota) {
                            rota.rotas.push(ob);
                        }
                        else {
                            $scope.SelectOp(r.exten);
                            $scope.popup.ura_menu.push({ op: r.exten, rotas: [ob] });
                        }
                    }
                });
            });
            $scope.showUraPopup = true;
        }

        function ValidaApp(r, op) {
            if (r.app === undefined || r.app === null || r.app === "") {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa selecionar uma ação na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'RAMAL' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa selecionar um RAMAL na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'FILA' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa selecionar uma FILA na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'URA' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa selecionar uma URA na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'EXTERNO' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa selecionar uma rota para o número externo na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'EXTERNO' && (r.par2 === undefined || r.par2 === null || r.par2 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa informar um número externo na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'PLAYBACK' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa selecionar um áudio na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'CUSTON' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa informar uma aplicação na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            if (r.app === 'TTS' && (r.par1 === undefined || r.par1 === null || r.par1 === "")) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: $scope.getWord("Você precisa informar um texto para o TTS na opção ") + op,
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                return false;
            }
            return true;
        }

        function App2DB(r) {
            var ob = { app: r.par1, appdata: r.par2 };
            if (r.app == "PLAYBACK") {
                ob.app = "Playback";
                ob.appdata = r.par1;
            }
            if (r.app == "RAMAL") {
                ob.app = "Macro";
                ob.appdata = r.par2 ? 'grava_ramal,' + r.par1 : 'disca_ramal,' + r.par1;
            }
            if (r.app == "FILA") {
                ob.app = "Macro";
                ob.appdata = r.par2 ? 'fila,' + r.par1 : 'fila_naograva,' + r.par1;
            }
            if (r.app == "EXTERNO" || r.app == "URA") {
                ob.app = "Goto";
                ob.appdata = r.par1 + ',' + r.par2;
            }
            if (r.app == "URA") {
                ob.app = "Goto";
                ob.appdata = r.par1 + ',' + (r.par2 ? r.par2 : 's') + ',' + (r.par3 ? r.par3 : '1');
            }
            if (r.app == "TTS") {
                ob.app = "AGI";
                ob.appdata = 'tts.php,"' + r.par1 + '"';
            }
            return ob;
        }

        function URA2DB(u) {
            var ura = [];
            var si = 1;
            if (u.ring) {
                ura.push({ context: u.nome, exten: "s", priority: si++, app: "Ringing", appdata: "", nome: "Toque" });
                ura.push({ context: u.nome, exten: "s", priority: si++, app: "Wait", appdata: "2", nome: "Aguarda" });
            }
            if (u.atender) {
                ura.push({ context: u.nome, exten: "s", priority: si++, app: "Answer", appdata: "", nome: "Atende" });
            }
            if (u.verifica_feriado) {
                ura.push({ context: u.nome, exten: "s", priority: si++, app: "GotoIf", appdata: "$[\"${XCONTACT_FERIADO()}\" != \"0\"]?feriado,1", nome: "Verifica Feriado" });
                var fe = App2DB(u.redireciona_feriado);
                ura.push({ context: u.nome, exten: "feriado", priority: 1, app: fe.app, appdata: fe.appdata, nome: "Redireciona feriado" });
            }
            if (u.verifica_turno) {
                ura.push({ context: u.nome, exten: "s", priority: si++, app: "GotoIf", appdata: "$[\"${XCONTACT_TURNO(ARRAY[" + u.turnos.map(e => e.id).join(",") + "])}\" = \"0\"]?fora_turno,1", nome: "Verifica turno" });
                var fe = App2DB(u.redireciona_turno);
                ura.push({ context: u.nome, exten: "fora_turno", priority: 1, app: fe.app, appdata: fe.appdata, nome: "Redireciona turno" });
            }
            ura.push({ context: u.nome, exten: "s", priority: si++, app: "Set", appdata: "CONTADOR=0", nome: "Contato ini" });
            var repetir = si + "";
            ura.push({ context: u.nome, exten: "s", priority: si++, app: "Set", appdata: "REPETIR=" + u.repetir, nome: "Repetir" });
            ura.push({ context: u.nome, exten: "s", priority: si++, app: "Set", appdata: "TIMEOUT(digit)=" + u.digito_timeout, nome: "Digito timeout" });
            ura.push({ context: u.nome, exten: "s", priority: si++, app: "Set", appdata: "TIMEOUT(response)=" + u.resposta_timeout, nome: "Resposta timeout" });
            ura.push({ context: u.nome, exten: "s", priority: si++, app: "BackGround", appdata: u.audio, nome: "Audio principal" });
            ura.push({ context: u.nome, exten: "s", priority: si++, app: "WaitExten", appdata: "", nome: "Aguarda" });

            // Invalid
            var ii = 1;
            ura.push({ context: u.nome, exten: "i", priority: ii++, app: "Set", appdata: "CONTADOR=$[${CONTADOR} + 1]", nome: "Incrementa contator" });
            if (u.audio_invalido && u.audio_invalido != "") {
                ura.push({ context: u.nome, exten: "i", priority: ii++, app: "Playback", appdata: u.audio_invalido, nome: "Play audio para opcao errada" });
            }
            ura.push({ context: u.nome, exten: "i", priority: ii++, app: "GotoIf", appdata: "$[${CONTADOR} <= ${REPETIR}]?s," + repetir, nome: "Volta" });
            // Timeout
            ura.push({ context: u.nome, exten: "t", priority: 1, app: "Set", appdata: "CONTADOR=$[${CONTADOR} + 1]", nome: "Incrementa contator" });
            ura.push({ context: u.nome, exten: "t", priority: 2, app: "GotoIf", appdata: "$[${CONTADOR} <= ${REPETIR}]?s," + repetir, nome: "Volta" });
            // Opcoes
            angular.forEach(u.ura_menu, function (um) {
                var pri = 1;
                angular.forEach(um.rotas, function (rota) {
                    var op = App2DB(rota);
                    ura.push({ context: u.nome, exten: um.op, priority: pri++, app: op.app, appdata: op.appdata, nome: "Menu " + um.op });
                });
            });

            console.log(ura);
            console.log("-> ", u);
            console.log($scope);

            // Ura Nova
            if (u.new) {
                var nova = { context: u.nome, descricao: "Ura criada pelo Xcontact", sentido: "URA" };
                $http.post(options.api.restv3 + "/contexto", nova)
                    .success(function (data) {
                        $http.post(options.api.restv3 + "/rota/full", ura)
                            .success(function (data) {
                                $scope.showUraPopup = false;
                                $.smallBox({
                                    title: "SUCESSO!!",
                                    content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                    color: "#739E73",
                                    timeout: 2000
                                });
                                $scope.load();
                            });
                    });
            } else {
                var nova = { context: u.nome, descricao: u.row.descricao, sentido: u.row.sentido };
                $http.put(options.api.restv3 + "/contexto/" + u.row.id, nova)
                    .success(function (data) {
                        $http.post(options.api.restv3 + "/rota/full", ura)
                            .success(function (data) {
                                $scope.showUraPopup = false;
                                $.smallBox({
                                    title: "SUCESSO!!",
                                    content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                    color: "#739E73",
                                    timeout: 2000
                                });
                                $scope.load();
                            });
                    });
            }
        }

        $scope.Salvar = function (f1) {
            if (f1.$valid) {
                var valido = true;
                if ($scope.popup.verifica_turno && $scope.popup.turnos.length == 0) {
                    $.smallBox({
                        title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                        content: $scope.getWord("Você precisa informar os TURNOS aceitos"),
                        color: "#C46A69",
                        icon: "fa fa-warning shake animated"
                    });
                    valido = false;
                    return;
                }
                if ($scope.popup.verifica_turno && !ValidaApp($scope.popup.redireciona_turno, $scope.getWord('em caso de fora do turno'))) {
                    valido = false;
                    console.log('FORM INVALIDO...');
                    return;
                }
                if ($scope.popup.verifica_feriado && !ValidaApp($scope.popup.redireciona_feriado, $scope.getWord('em caso de feriado'))) {
                    valido = false;
                    console.log('FORM INVALIDO...');
                    return;
                }

                // Valida se não informou opções
                if ($scope.popup.ura_menu.length === 0 || $scope.popup.ura_menu[0].op === undefined) {
                    $.smallBox({
                        title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                        content: $scope.getWord("Você precisa selecionar ao menos uma opção para o menu da URA"),
                        color: "#C46A69",
                        icon: "fa fa-warning shake animated"
                    });
                    valido = false;
                    return;
                }
                // Valida opções
                angular.forEach($scope.popup.ura_menu, function (m) {
                    if (m.op === undefined) {
                        $.smallBox({
                            title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                            content: $scope.getWord("Você precisa selecionar uma opção para o menu"),
                            color: "#C46A69",
                            icon: "fa fa-warning shake animated"
                        });
                        valido = false;
                        return;
                    }
                    angular.forEach(m.rotas, function (r) {
                        if (!ValidaApp(r, m.op)) {
                            valido = false;
                            console.log('FORM INVALIDO...');
                            return;
                        }
                    });
                });
                if (valido) {
                    console.log('OK = FORM VALIDO...');
                    console.log($scope.popup);

                    URA2DB($scope.popup);
                    // Depois de Salvar
                    f1.$setPristine();
                    f1.$setUntouched();
                }
            }
            else {
                //alert($scope.getWord('Existem campos obrigatórios não preenchidos ou preenchidos incorretamente, verifique'));
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                    content: "Existem campos obrigatórios não preenchidos ou preenchidos incorretamente, verifique os campos em todas as tabas e tente novamente..",
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
                console.log('Form invalido');
                console.log(f1);
            }
        };

        $scope.Confirma = function (nome) {
            $.x5Box({
                title: $scope.getWord('Remover URA'),
                content: $scope.getWord('<p>Tem certeza que quer remover a URA ') + '<b>' + nome + "</b>?</p><p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Sim") {
                    Apagar(nome);
                }
            });
        };

        function Apagar(contexto) {
            $http.delete(options.api.restv3 + "/contexto/byname/" + contexto).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>URA removida com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });

                    $scope.load();
                })
        };

        $scope.getClass = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                return $scope.order[0].ordem === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.setOrder = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                $scope.order[0].ordem = $scope.order[0].ordem === 'ASC' ? 'DESC' : 'ASC';
            }
            else {
                $scope.order = [{ "campo": coluna, "ordem": "ASC" }];
            }
            $scope.load();
        };

        $scope.$watch('Pesquisa', function (t1, t2) {
            // Se os dois valores estirem em branco nao faz nada, isso aconte no load.
            if (!t1 && !t2)
                return false;

            setTimeout(function () {
                if (t1 === $scope.Pesquisa) {
                    if (t1 !== '') {
                        $scope.filtro.nome = { valor: t1 };
                    }
                    else {
                        $scope.filtro.nome = undefined;
                    }

                    $scope.load();
                }
            }, 1000);
        });
    });
});
