define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ngTagsInput'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.usuarios', [
        'ui.router',
        'ngResource',
        'ngTagsInput',
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.usuarios', {
                url: '/usuarios',                
                data: {
                    title: 'Usuarios',
                    permissions: {
                        only: ['app.usuarios']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/usuarios/views/lista.html',
                        controller: 'UsuariosCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/usuarios/controllers/UsuariosCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.usuarios.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Usuarios',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/usuarios/views/form.html',
                        controller: 'UsuariosCtrl',
                        parent: 'app.usuarios',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/usuarios/controllers/UsuariosCtrl',
                                'modules/usuarios/directives/usuariosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.usuarios.novo', {
                url: '/novo',
                data: {
                    title: 'Novo Usuario',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/usuarios/views/form.html',
                        controller: 'UsuariosCtrl',
                        parent: 'app.usuarios',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/usuarios/controllers/UsuariosCtrl',
                                'modules/usuarios/directives/usuariosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.usuarios.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Usuario',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/usuarios/views/form.html',
                        controller: 'UsuariosCtrl',
                        parent: 'app.usuarios',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/usuarios/controllers/UsuariosCtrl',
                                'modules/usuarios/directives/usuariosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.usuarios.modulos', {
                url: '/modulos/:id',
                data: {
                    title: 'Módulos',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/usuarios/views/modulos.html',
                        controller: 'UsuariosModulosCtrl',
                        parent: 'app.usuarios',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/usuarios/controllers/UsuariosModulosCtrl',
                                'modules/forms/directives/input/smartSelect2'
                            ])
                        }
                    }
                }
            })

            .state('app.usuarios.log', {
                url: '/log',
                data: {
                    title: 'LOG',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/usuarios/views/log.html',
                        controller: 'LogUserCtrl',
                        parent: 'app.usuarios',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/usuarios/controllers/LogUserCtrl',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,$localStorage,ModuloPermissao) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.usuarios', function () {
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.usuarios",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: "Você não tem acesso ao módulo de Usuarios."
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});