"use strict";
define(['modules/wizard_mensagens_ura/module'], function(module) {
    return module.registerController("WizardMensagensUraListaController", function($scope, $http, $stateParams, $rootScope) {
        $scope.carregando = false;
        $scope.filtro = {};
        $scope.order = [{ campo: "context", ordem: "ASC" }];
        $scope.popupVisible = false;
        $scope.contextoSelecionado = null;
        $scope.editando = false;
        $scope.variaveis = [];

        $scope.init = async function () {
            $scope.carregando = true;
            $scope.filtro.page = { valor: 1 };
            await $scope.getContextos();
            $scope.carregando = false;
        };

        //#region Chamadas de API
        $scope.carregarDadosDaAPI = function () {
            let promises = [
                $scope.getAudios(),
                $scope.getFilas(),
                $scope.getTurnos(),
                $scope.getRamais(),
                $scope.getRotasSaida(),
                $scope.getURAS(),
                $scope.getTroncos()
            ];

            return Promise.all(promises);
        };

        $scope.getContextos = function () {
            return new Promise((resolve, reject) => {
                var par = [];
                if ($scope.filtro) {
                    angular.forEach($scope.filtro, function (v, k) {
                        if (v) par.push(k + "=" + v.valor);
                    });
                }
                var pars = "";
                if (par.length > 0) {
                    pars = "&" + par.join("&");
                }

                let sentido = encodeURI(JSON.stringify(["URA_MENSAGEM"]));

                $http.get(options.api.restv3 + "/contexto?max=-1&sentido=" + sentido + "&order=" + encodeURI(JSON.stringify($scope.order)) + pars).
                    success(data => {   
                        $scope.table = data.dados;
                        resolve(data);
                    })
                    .error(error => reject(error)); 
            });
        };

        $scope.getAudios = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/audio?max=-1")
                    .success(data => {
                        $scope.audios = data.dados
                        resolve($scope.audios);
                    })
                    .error(error => {
                        reject(error);
                    }); 
            });            
        }; 

        $scope.getRotas = function (contexto) {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/rota/URA?context=" + encodeURI(contexto.context))
                    .success(data => {
                        $scope.rotas = data.dados.find(x => x).rotas;
                        resolve($scope.rotas);
                    })
                    .error(error => {
                        reject(error);
                    }); 
            });
        };

        $scope.getFilas = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/fila?max=-1")
                    .success(data => {
                        $scope.filas = data.dados;
                        resolve(data.dados);
                    })
                    .error(error => reject(error)); 
            });
        };

        $scope.getTurnos = function () {
            $http.get(options.api.restv3 + "/horario?max=-1&tipo=T").
                success(function (data) {
                    if (data.total > 0)
                        $scope.turnos = data.dados.map((x) => {
                            if (!x.descricao) x.descricao = x.nome;
                            return x;
                        });
                })
                .error(error => reject(error)); 
        };

        $scope.getRamais = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/ramal?max=-1&order=" + encodeURI(JSON.stringify([{ campo: 'name', ordem: 'ASC' }])))
                    .success(data => {
                        $scope.ramais = data.dados;
                        resolve(data.dados);
                    })
                    .error(error => reject(error)); 
            });            
        };

        $scope.getRotasSaida = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/contexto?max=-1&page=1&sentido=" + encodeURI(JSON.stringify(["Saida"])))
                    .success(data => {
                        $scope.rotas_saida = data.dados;
                        resolve(data.dados);
                    })
                    .error(error => reject(error)); 
            });            
        };

        $scope.getURAS = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/contexto?max=-1&sentido=" + encodeURI(JSON.stringify(["URA"])))
                    .success(data => {
                        $scope.rotas_uras = data.dados;
                        resolve(data.dados);
                    })
                    .error(error => reject(error)); 
            });
        };

        $scope.getTroncos = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv3 + "/tronco?max=-1&order=" + encodeURI(JSON.stringify([{ campo: 'nome', ordem: 'ASC' }])))
                    .success(data => {
                        $scope.troncos = data.dados;

                        resolve(data.dados);
                    })
                    .error(error => reject(error)); 
            });              
        };

        $scope.criarGrupo = function (grupo) {
            return new Promise((resolve, reject) => {
                let obj = { 
                    context: grupo,
                    descricao: "URA Criada pelo Wizard de mensagens do XCONTACT",
                    sentido: "URA_MENSAGEM"
                };

                $http.post(options.api.restv3 + "/contexto", obj)
                    .success(data => resolve(data))
                    .error(error => reject(error));
            });
        };

        $scope.atualizarGrupo = function (grupo) {
            return new Promise((resolve, reject) => {
                let obj = { 
                    context: grupo.context,
                    descricao: "URA Criada pelo Wizard de mensagens do XCONTACT",
                    sentido: "URA_MENSAGEM"
                };

                $http.put(options.api.restv3 + "/contexto/" + grupo.id, obj)
                    .success(data => resolve(data))
                    .error(error => reject(error));
            });
        };

        $scope.criarRotas = function (rotas) {
            return new Promise((resolve, reject) => {
                $http.post(options.api.restv3 + "/rota/full", rotas)
                    .success(data => resolve(data)) 
                    .error(data => reject(data)); 
            });            
        };

        $scope.deletarContexto = function (contexto) {
            return new Promise((resolve, reject) => {
                $http.delete(options.api.restv3 + "/contexto/byname/" + contexto)
                    .success(x => resolve(x))
                    .error(error => reject(error)); 
            });
        };

        $scope.deletarRotasDoContexto = function (contexto) {
            return new Promise((resolve, reject) => {
                $http.delete(options.api.restv3 + "/rota/limparGrupo/" + contexto)
                    .success(x => resolve(x))
                    .error(error => reject(error)); 
            });
        };
        //#endregion

        //#region Métodos para a UI
        $scope.adicionarPlaceholderVariavel = function() {
            $scope.variaveis.push({ 
                ignore: true,
                ativada: true// Inicializa a varíavel como ativada
            });
        };

        $scope.filterTurnos = function (query) {
            return $scope.turnos.filter(function (t) {
                return t.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };
        //#endregion

        //#region Utilidade
        $scope.mostrarMensagemErro = function (titulo, mensagem) {
            $.smallBox({
                title: "<i class='glyphicon glyphicon-remove'></i> " + titulo,
                content: mensagem,
                color: "#C46A69",
                icon: "fa fa-warning shake animated"
            });
        };

        $scope.mostrarMensagemSucesso = function (titulo, mensagem) {
            $.smallBox({
                title: titulo,
                content: `<i class='fa fa-check'></i> <i>${mensagem}</i>`,
                color: "#739E73",
                icon: "fa fa-warning shake animated"
            });
        };

        $scope.validarCamposDaAcao = function (variavel) {
            // Verifica se algum campo está inválido
            if (variavel.acao && variavel.acao != "" && (variavel.turnos && variavel.turnos.length > 0)) {
                try {
                    switch (variavel.acao) {
                        case "personalizado":
                            // Os argumentos são opcionais, então ignoramos a verificação
                            return variavel.personalizado && (variavel.personalizado.comando && variavel.personalizado.comando.length >= 1);
                            
                        case "audio":
                            return variavel.audio && variavel.audio.length >= 1;

                        case "ramal":
                            // A opção de gravar é opcional
                            return variavel.ramal;

                        case "fila":
                            // A opção de gravar é opcional
                            return variavel.fila;
                        
                        case "tts":
                            return variavel.texto_tts && variavel.texto_tts.length > 1;
                        
                        case "rota":
                            return variavel.rota && (variavel.numero && variavel.numero.length > 1);

                        case "ura":
                            return variavel.ura.bloco && (variavel.ura.prefixo && variavel.ura.prefixo.length >= 1) && (variavel.ura.indice && parseInt(variavel.ura.indice) > 0);

                        default:
                            return false;
                    }
                } catch (e) {
                    console.error(e);
                    console.trace(e);

                    return false;
                }
                
            } else {
                return false;
            }
        };

        $scope.verificarValidadeDoForm = function (form) {
            try {
                //let errors = form.$error.required.map(x => parseInt(x.$name.replace(/VAR_([0-9])_NOME/gi, "$1")))
                return form.$valid;
            } catch (e) {
                console.trace(e);
                console.error(e);
                return true;
            }            
        };
        //#endregion
        
        //#region Eventos
        $scope.showPopup = async function (row) {    
            if (row) {
                // Notifica que a linha está carregando o popup
                row.load = true;

                // Carrega os dados da API
                await $scope.carregarDadosDaAPI();

                // Salva o contexto para uso futuro
                $scope.contextoSelecionado = row;
                // Carrega as rotas do contexto
                await $scope.getRotas(row);         
                
                $scope.editando = true;
                $scope.variaveis = $scope.descompilarRotas(row.context, $scope.rotas);
            } else {
                // Carrega os dados da API
                $scope.carregando = true;                
                await $scope.carregarDadosDaAPI();
                $scope.carregando = false;                

                $scope.editando = false;
                $scope.contextoSelecionado = { };
                $scope.variaveis = [];
            }

            $scope.adicionarPlaceholderVariavel();

            // Mostra o popup
            $scope.popupVisible = true;
            // Notifica que a linha está carregando o popup
            if (row) row.load = false;
        }; 

        $scope.deletar = function (row) {
            $.x5Box({
                title: $scope.getWord('Remover URA'),
                content: $scope.getWord('<p>Tem certeza que quer remover a URA ') + '<b>' + row + "</b>?</p><p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, button => {
                if (button === "Sim") {
                    row.delete = true;
                    $scope.deletarRotasDoContexto(row.context)
                        .then(() => $scope.deletarContexto(row.context))
                        .then(() => {
                            $scope.mostrarMensagemSucesso("Sucesso", "URA removida com sucesso!");
                            $scope.init();
                        })
                        .catch(error => {
                            console.error(error);
                            $scope.mostrarMensagemErro("Atenção!", "Não foi possível deletar a URA!<br/>" + error);
                            row.delete = false;
                        });
                }
            });
        };

        $scope.salvar = async function (form) {
            try {
                if (!$scope.verificarValidadeDoForm(form)) {
                    console.error("Invalid form!");
                    $scope.mostrarMensagemErro("Atenção!", "O Formulário está inválido!<br/>Por favor, preencha os campos de acordo.");
                    return;
                }

                // Verifica se nenhuma variável foi adicionada
                if (!$scope.variaveis || $scope.variaveis.filter(x => !x.ignore).length <= 0) {
                    $scope.mostrarMensagemErro("Atenção!", "Nenhuma variável foi adicionada!<br/><br/>Você se esqueceu de apertar no botão de 'Adicionar'?");
                    return;
                }

                // Notifica que o popup está salvando os items
                $scope.carregando = true;

                // Salva o nome do grupo|contexto
                let nomeGrupo = $scope.contextoSelecionado.context;
                
                // Compila as rotas para AEL
                let variaveis = $scope.variaveis.filter(x => x && !x.ignore);                
                let rotas = $scope.compilarRotas(variaveis, nomeGrupo);
                
                // Se estiver editando algo já existente, apenas atualiza
                if ($scope.editando) {
                    await $scope.atualizarGrupo($scope.contextoSelecionado);
                    await $scope.criarRotas(rotas);
                } else {
                    // Caso contrário cria o grupo e as suas rotas
                    await $scope.criarGrupo(nomeGrupo);
                    await $scope.criarRotas(rotas);
                }

                // Atualiza a tabela
                await $scope.init();

                // Esconde o popup
                $scope.popupVisible = false;
                // Notifica que o popup está salvando os items
                $scope.carregando = false;
            } catch (e) {            
                console.trace(e);
                console.error("Erro ao salvar: " + e);
                $scope.mostrarMensagemErro("Erro!", "Erro inesperado ao salvar!\n" + e);
            }
        };

        $scope.adicionarVariavel = function (variavel) {
            // Verifica se alguma ação foi escolhida
            if ($scope.validarCamposDaAcao(variavel)) {
                variavel.nome = "MSG_" + $scope.variaveis.length;
                delete variavel.ignore;
    
                $scope.adicionarPlaceholderVariavel();    
            } else {
                // Ignore
            }
        }; 

        $scope.removerVariavel = function (variavel) {
            let index = $scope.variaveis.findIndex(x => x== variavel);

            if (index != -1) {
                $scope.variaveis.splice(index, 1);
            }
        };
        //#endregion
        
        //#region Processo de compilação
        $scope.formatarPrefixoVariavel = function (variavel) {
            return variavel.nome.toLowerCase().replace(/[-_\s]/gi, "");
        };

        $scope.formatarComando = function (prefixo, ordem, comando, argumentos, nome) {
            return {
                context: $scope.contextoSelecionado.context, 
                exten: prefixo, 
                priority: ordem, 
                app: comando, 
                appdata: argumentos, 
                nome: nome
            };
        };

        $scope.comandoTurnos = function (variavel) {
            let turnos = variavel.turnos.map(x => x.id).join(",");
            return "\"${" + `XCONTACT_TURNO(ARRAY[${turnos}])` + "}\"";
        };

        $scope.compilarAcao = function (prefixo, ordem, variavel, contexto) {
            let cmds = [];

            switch(variavel.acao) {
                case "personalizado":
                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, variavel.personalizado.comando, variavel.personalizado.argumentos, "Instrução personalizada definida pelo usuário"),
                    );
                    break;
                case "audio":
                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, "Playback", variavel.audio, "Roda o áudio definido na variavel"),
                    );
                    break;
                case "ramal":
                    let nomeMacroRamal = variavel.gravar ? "grava_ramal" : "disca_ramal";

                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, "macro", `${nomeMacroRamal},${variavel.ramal}`, "Disca para um ramal interno")
                    );
                    break;
                case "fila":
                    let nomeMacroFila = variavel.gravar ? "fila" : "fila_naograva";

                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, "macro", `${nomeMacroFila},${variavel.fila}`, "Direciona para uma fila")
                    );
                    break;
                case "tts":
                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, "agi", `tts.php,${variavel.texto_tts}`, "Fala em alta voz o texto especificado"),
                    );
                    break;
                case "rota":
                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, "goto", `${variavel.rota},${variavel.numero}`, "Direciona para uma rota"),
                    );
                    break;
                case "ura":
                    cmds.push(
                        $scope.formatarComando(prefixo, ordem, "goto", `${variavel.ura.bloco},${variavel.ura.prefixo},${variavel.ura.indice}`, "Direciona para uma URA"),
                    );
                    break;
            }

            return cmds;
        };

        $scope.compilarRotas = function (variaveis, contexto) {
            // Define um array para guardar as rotas geradas            
            let rotas = [];

            let vIndex = 0;

            // Para cada variavel provida, gerar as rotas
            variaveis.forEach(v => {
                // Transforma o nome da rota em um prefixo válido
                let prefixo = $scope.formatarPrefixoVariavel(v);

                let cmds = [];
                
                let varHabilitadaNome = `${contexto}_${prefixo}_HABILITADA`.toUpperCase();

                cmds.push(
                    // s 1 Set $VAR$_ATIVA=SIM
                    $scope.formatarComando(prefixo, cmds.length+1, "Set", `${varHabilitadaNome}=${v.ativada ? 'SIM' : 'NAO'}`, "Por padrão define o valor da variável como 'sim'")
                );

                cmds.push(
                    // s 2 ExecIf $["$VAR$_ATIVA" = "SIM"]?Return                    
                    $scope.formatarComando(prefixo, cmds.length+1, "ExecIf", `$[\${${varHabilitadaNome}} = NAO]?Return`, "Verifica a mensagem está ativa")
                )

                cmds.push(
                    // s 3 ExecIf $["${XCONTACT_TURNO(ARRAY[$TURNOS_ID$])}" != "0"]?Return                    
                    $scope.formatarComando(prefixo, cmds.length+1, "ExecIf", `$[${$scope.comandoTurnos(v)} = "0"]?Return`, "Verifica se o turno é valido"),
                )

                cmds = cmds.concat($scope.compilarAcao(prefixo, cmds.length+1, v, contexto))

                cmds.push(
                    // s 5 Return
                    $scope.formatarComando(prefixo, cmds.length+1, "Return", "", "Retorna da sub-rotina"),
                )

                // Adiciona os comandos no fim do array de rotas
                rotas = rotas.concat(cmds);
            });

            // Define o prefixo 's'
            // Onde serão definidos os GoSub's inicias
            let i = 0;
            do {
                // Transforma o nome da rota em um prefixo válido
                let prefixo = $scope.formatarPrefixoVariavel(variaveis[i]);

                // s i Gosub $VAR$,1
                rotas.push($scope.formatarComando("s", i+1, "Gosub", `${prefixo},1`));

                i++;
            } while (i < variaveis.length);
            
            // Adiciona um return no final do prefixo 's'
            rotas.push($scope.formatarComando("s", i + 1, "Return", "", "Retorna da sub-rotina"));

            console.log("Rotas -> ", rotas);

            return rotas;
        };
        //#endregion
        
        //#region Processo de descompilação
        $scope.descompilarRotas = function (contexto, rotas) {
            // Define o regex para extrair os ID's dos turnos do comando de turnos
            const regexComandoTurnos = /\$\["\${XCONTACT_TURNO\(ARRAY\[([0-9]+(?:\s*,\s*[0-9]+)*)+\]\)}" = "0"]\?Return/;

            // Agrupa as rotas pelos prefixos
            let prefixos = _.groupBy(rotas, x => x.exten);

            // Define a variável temporária que irá guardar todas as rotas
            let variaveis = [];

            // Itera sobre todos os prefixos
            for (let indentificadorVariavel in prefixos) {
                try {
                    // Ignora o prefixo de início
                    if (indentificadorVariavel.toLowerCase() === "s") continue;

                    // Define o esqueleto da variável
                    let variavel = {
                        ativada: false,
                        nome: null,
                        turnos: [],
                        audio: null,
                    };

                    // Guarda o array de rotas do prefixo 
                    let entries = prefixos[indentificadorVariavel];

                    // Verifica o tamanho das rotas, caso diferente do esperado, ignora (Provavelmente a rota foi editada por terceiros, então o Wizard irá ignora-la)
                    if (entries.length != 5) continue;

                    // 1 - Seta a varíavel como ativa ou não
                    let setEntry = entries[0];
                    let fragments = setEntry.appdata.split("=");
                    variavel.ativada = fragments[1].toLowerCase() === "sim";
                    variavel.nome = fragments[0].substring(entries[0].context.length + 1, fragments[0].indexOf("_HABILITADA"));

                    // 2 - Executa a verificação da variável ativa
                    // ignore

                    // 3 - Executa a verificação de turnos
                    let turnoEntry = entries[2];
                    let match = regexComandoTurnos.exec(turnoEntry.appdata);
                    if (match && match[1] && match[1].length > 0) {// Verifica se teve algum match
                        // Extrai os ids dos turnos
                        let arrayTurnoIds = match[1].split(",").filter(x => x).map(x => parseInt(x.trim()));
                        // Popula com o objeto dos turnos
                        variavel.turnos = arrayTurnoIds.map(x => $scope.turnos.find(y => y.id === x));
                    } else throw "O Parser não conseguiu identificar os id's dos turnos da variável '" + (variavel.nome || indentificadorVariavel) + "'";
                    
                    // 4 - Ação
                    let acaoEntry = entries[3];
                    let appl = acaoEntry.app.toLowerCase(); 
                    let app = acaoEntry.app;
                    let appdata = acaoEntry.appdata;
                    if (appl === "playback") {
                        // Comando de playback traduz para a ação de tocar um áudio
                        variavel.acao = "audio";
                        variavel.audio = appdata;
                    } else if (appl === "macro" && appdata.startsWith("disca_ramal,")) {
                        // Comando de macro disca_ramal traduz para a ação de discar um ramal sem gravar
                        variavel.acao = "ramal";
                        variavel.ramal = appdata.substring("disca_ramal,".length);
                        variavel.gravar = false;
                    } else if (appl === "macro" && appdata.startsWith("grava_ramal,")) {
                        // Comando de macro grava_ramal traduz para a ação de discar um ramal gravando
                        variavel.acao = "ramal";
                        variavel.ramal = appdata.substring("grava_ramal,".length);
                        variavel.gravar = true;
                    } else if (appl === "macro" && appdata.startsWith("fila,")) {
                        // Comando de macro fila traduz para a ação de direcionar para uma fila gravando
                        variavel.acao = "fila";
                        variavel.fila = appdata.substring("fila,".length);
                        variavel.gravar = true;
                    } else if (appl === "macro" && appdata.startsWith("fila_naograva,")) {
                        // Comando de macro fila_naograva traduz para a ação de direcionar para uma fila sem gravar
                        variavel.acao = "fila";
                        variavel.fila = appdata.substring("fila_naograva,".length);
                        variavel.gravar = false;
                    } else if (appl === "agi" && appdata.startsWith("tts.php,")) {
                        // Comando de agi tts traduz para a ação de Text-to-Speech
                        variavel.acao = "tts";
                        variavel.texto_tts = appdata.substring("tts.php,".length).replace(/\"/g, '');                        
                    } else if (appl === "goto") {
                        // Comando goto traduz para as ações de RotaExterna ou Ura
                        let externo = $scope.rotas_saida.find(x => appdata.includes(x.context));
                        let ura = $scope.rotas_uras.find(x => appdata.includes(x.context));

                        if (externo) {
                            // Traduz para o GOTO de rota externa
                            variavel.acao = "rota";
                            let [ rota, numero ] = appdata.split(",");
                            variavel.rota = rota;
                            variavel.numero = numero;
                        } else if (ura) {
                            // Traduz para o GOTO de URA
                            variavel.acao = "ura";
                            let [ bloco, prefixo, indice ] = appdata.split(",");
                            variavel.ura = {
                                bloco,
                                prefixo,
                                indice: parseInt(indice)
                            };
                        } else {
                            // O comando não foi reconhecido, então será tratado como comando personalizado
                            variavel.acao = "personalizado";
                            variavel.personalizado = {
                                comando: app,
                                argumentos: appdata
                            };
                        }
                    } else {
                        // O comando não foi reconhecido, então será tratado como comando personalizado
                        variavel.acao = "personalizado";
                        variavel.personalizado = {
                            comando: app,
                            argumentos: appdata
                        };
                    }

                    // 5 - Return
                    // ignore

                    // Adiciona a variável no array
                    variaveis.push(variavel);
                } catch (e) {
                    console.error(e);
                    console.trace(e);
                }
            }

            return variaveis;
        };
        //#endregion

    });
});