define(['app'], function (app) {
    "use strict";

    app.filter('tel', function () {
        return function (tel) {
            if (!tel) { return ''; }
            // Se nao for numero
            if (!isFinite(tel)) return tel;

            tel = tel.replace(/\D/g, ""); //Remove tudo o que não é dígito

            // Tira os zeros da frente
            while (tel.indexOf('0') === 0) {
                tel = tel.substring(1);
            }

            switch (tel.length) {
                case 8: // xxxx-xxxx
                    tel = tel.replace(/(\d)(\d{4})$/, "$1-$2"); //Coloca hífen entre o quarto e o quinto dígitos
                    break;

                case 9: // x xxxx-xxxx --- Telefones SP
                    tel = tel.replace(/^(\d{1})(\d)/g, "$1 $2"); //Coloca um espaço entre o primeiro e segundo digito
                    tel = tel.replace(/(\d)(\d{4})$/, "$1-$2"); //Coloca hífen entre o quinto e o sexto dígitos
                    break;

                case 10: // (xx)xxxx-xxxx   --- Com DDDD
                    tel = tel.replace(/^(\d{2})(\d)/g, "($1) $2"); //Coloca parênteses em volta dos dois primeiros dígitos
                    tel = tel.replace(/(\d)(\d{4})$/, "$1-$2"); //Coloca hífen entre o quarto e o quinto dígitos
                    break;

                case 11: // (xx)x xxxx-xxxx   --- Com DDDD SP
                    tel = tel.replace(/^(\d{2})(\d)/g, "($1) $2"); //Coloca parênteses em volta dos dois primeiros dígitos
                    tel = tel.replace(/^(\d{3})(\d)/g, "$1 $2"); //Coloca um espaço entre o quarto e quinto
                    tel = tel.replace(/(\d)(\d{4})$/, "$1-$2"); //Coloca hífen entre o quarto e o quinto dígitos
                    break;

                case 12: // (xx)xxxxx-xxxx --- Retira operadora
                    tel = tel.substring(2);
                    tel = tel.replace(/^(\d{2})(\d)/g, "($1) $2"); //Coloca parênteses em volta dos dois primeiros dígitos
                    tel = tel.replace(/(\d)(\d{4})$/, "$1-$2"); //Coloca hífen entre o quarto e o quinto dígitos
                    break;

                case 13: // (xx)x xxxx-xxxx   --- Retira operadora Com nono digito
                    tel = tel.substring(2);
                    tel = tel.replace(/^(\d{2})(\d)/g, "($1) $2"); //Coloca parênteses em volta dos dois primeiros dígitos
                    tel = tel.replace(/^(\d{3})(\d)/g, "$1 $2"); //Coloca um espaço entre o quarto e quinto
                    tel = tel.replace(/(\d)(\d{4})$/, "$1-$2"); //Coloca hífen entre o quarto e o quinto dígitos
                    break;
            }

            return tel;
        };
    })

    app.filter('OpcoesUnicas', function () {
        return function (full, sel) {
            if (!sel) {
                return full;
            }
            var sel_ops = sel.map(function (obj) { return obj.op; });
            console.log(sel_ops);
            return full;
            //return full.filter(function(obj) { return sel_ops.indexOf(obj) == -1; });
        }
    })

    app.filter('data', function ($filter) {
        return function (d) {
            if (!d) return '';
            return $filter('date')(d, 'dd/MM/yyyy');
        };
    })

    app.filter('decimal', function () {
        return function (d) {
            if (!d) return '';
            return Number(d).toFixed(2).replace('.', ',');
        };
    })

    app.filter('moeda', function () {
        return function (d) {
            if (!d) return '';
            return 'R$' + Number(d).toFixed(2).replace('.', ',');
        };
    })

    app.filter('datahora', function ($filter) {
        return function (d) {
            if (!d) return '';
            return $filter('date')(d, 'dd/MM/yyyy HH:mm:ss');
        };
    })

    app.filter('hora', function ($filter) {
        return function (d) {
            if (!d) return '';
            return $filter('date')(d, 'HH:mm:ss');
        };
    })
    // Transforma tempo no formato 00:00:00 em segundos
    app.filter('timetosec', function () {
        return function (d) {
            if (!d) return 0;
            var tt = d.split(":");
            return parseInt(tt[0]) * 3600 + parseInt(tt[1]) * 60 + parseInt(tt[2]) * 1;
        };
    })
    // Transforma segundos em tempo no formato 00:00:00
    app.filter('sectotime', function () {
        return function (seconds) {
            if (!seconds) return '00:00:00';
            var moment = require('moment');
            var t = moment.duration(seconds, 'seconds');

            var horas = Math.floor(t.asHours());
            horas = horas < 10 ? ("00" + horas).slice(-2) : horas;
            return horas + ':' + ("00" + t.minutes()).slice(-2) + ':' + ("00" + t.seconds()).slice(-2);
        };
    })
    // Transforma Unix timestamp em tempo no formato 00:00:00
    app.filter('unixsectotime', function () {
        return function (timestamp) {
            if (!timestamp) return '00:00:00';
            if (timestamp === '0') return '00:00:00';

            var moment = require('moment');
            return moment.unix(timestamp).format("HH:mm:ss");
        };
    })
    // Retorna o nome do mes pelo número (1-Janeiro...12-Dezembro)
    app.filter('mes', function () {
        return function (mes) {
            var messes = ['', 'Janeiro', 'Fevereiro', 'Março', 'Abril', 'Maio', 'Junho', 'Julho', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Dezembro'];
            return messes[mes];
        };
    })
    // Retorna o dia da semana pelo numero (0-Domingo...6-Sábado)
    app.filter('diadasemana', function () {
        return function (dia) {
            var dias_semana = ['Domingo', 'Segunda', 'Terça', 'Quarta', 'Quinta', 'Sexta', 'Sábado'];
            return dias_semana[dia];
        };
    })
    // Retorna Sim para true e Não para false
    app.filter('Xbool', function () {
        return function (b) {
            return b === true ? 'Sim' : 'Não';
        };
    })

    return app;
});
