'use strict';

(function () {
    // Bem, achei o problema de ficar duplicando os eventos no online.
    // O que ocorre é que cada vez que entra no online ele adiciona mais um listener para os eventos.
    // Em tese teria que controlar os listener por scope e quando aquele scope for destrido desregistra os listeners dele.
    angular.module('ngStore', []).
        factory('$Store', _storageFactory('localStorage'));
    function _storageFactory (storageType) {
        return [
            '$rootScope',
            '$window',
            function ($rootScope, $window) {
                console.log('Chamou o Store');
                let store = {};
                let _events = {};
                store.isConnect = false;

                if (options.api.ip_server == 'localhost') {
                    options.api.ip_server = $window.location.hostname;
                    options.api.modo = $window.location.protocol;
                    options.api.ws_port = options.api.modo === 'http:' ? 8000 : 8003;
                    options.api.restv2_port = options.api.modo === 'http:' ? 8001 : 8004;

                    options.api.gravacao = `${options.api.modo}//${options.api.ip_server}`;
                    options.api.rest = `${options.api.modo}//${options.api.ip_server}/xcontact-rest`;
                    options.api.restv2 = `${options.api.modo}//${options.api.ip_server}:${options.api.restv2_port}/api/v2`;
                    options.api.restv3 = `${options.api.modo}//${options.api.ip_server}:${options.api.restv2_port}/api/v3`;
                    options.api.restv4 = `${options.api.modo}//${options.api.ip_server}:${options.api.restv2_port}/api/v4`;
                    options.api.ws = `${options.api.modo}//${options.api.ip_server}:${options.api.ws_port}`;
                }

                //console.log(`Vai abrir WS ${options.api.ws}`);
                var io = require('socket-io2')(`${options.api.ws}`, { autoConnect: false });

                var wrapCallback = function (callback) {
                    var wrappedCallback = angular.noop;

                    if (callback) {
                        wrappedCallback = function () {
                            var args = arguments;
                            $rootScope.$apply(function () {
                                callback.apply(io, args);
                            });
                        };
                    }
                    return wrappedCallback;
                };


                store.on = (event, callback) => {
                    let ioEvent = JSON.parse(JSON.stringify(event));
                    if (event.includes('.')) {
                        let parts = event.split('.');
                        ioEvent = parts[1];
                    }
                    //console.log(`Listener ${event} para evento ${ioEvent}`);

                    return io.on(ioEvent, callback);

                    //return _events[event] = io.on(ioEvent, callback);

                    /**
                    if (_events.hasOwnProperty(event)) {
                        //console.log(`Listener já ativo para o evento ${event}`);
                        return _events[event];
                    }

                    let ioEvent = JSON.parse(JSON.stringify(event));
                    if (event.includes('.')) {
                        let parts = event.split('.');
                        ioEvent = parts[1];
                    }
                    //console.log(`Listener ${event} para evento ${ioEvent}`);

                    return _events[event] = io.on(ioEvent, (...args) => {
                        $rootScope.$apply(_ => {
                            callback.apply(io, args);
                        });
                    });
                    **/
                }

                store.emit = function (name, data, callback) {
                    if (callback) {
                        io.emit(name, data, wrapCallback(callback));
                    }
                    else {
                        io.emit(name, data);
                    }
                }

                store.disconect = () => {
                    io.disconnect();
                }

                store.removeListener = (ev, listener) => {
                    //console.log(`Removendo Listener do evento ${ev}`);
                    io.off(ev, listener);
                }

                store.removeAllListeners = (ev) => {
                    if (ev) {
                        io.removeAllListeners(ev);
                    }
                    else {
                        io.removeAllListeners();
                    }
                }

                store.connect = () => {
                    //console.log(`Abrindo conexao WS ${options.api.ws}`);
                    io.connect();

                    io.on('connect', () => {
                        //console.log('Conectado no WS');
                        store.isConnect = true;
                    });
                    io.on('disconnect', () => {
                        //console.log('Desconectou');
                        store.isConnect = false;
                    });
                }
                return store;
            }
        ]
    }
})();